@@ -2,13 +2,7 @@
 
   Copyright (c) 2017, Linaro, Ltd. All rights reserved.
 
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -29,7 +23,7 @@ LcdIdentify (
   VOID
   )
 {
-  return EFI_SUCCESS;
+  return EFI_NOT_FOUND;
 }
 
 /**
