@@ -1398,6 +1398,18 @@ WriteSections64 (
             SymName = (const UINT8 *)"<unknown>";
           }
 
+          if (mEhdr->e_machine == EM_X86_64) {
+            //
+            // For x86_64, we can ignore R_X86_64_NONE relocations.
+            // They are used to indicate that the symbol is not defined
+            // in the current module, but in a shared library that may be
+            // used when building modules for inclusion in host-based unit tests.
+            //
+            if (ELF_R_TYPE(Rel->r_info) == R_X86_64_NONE) {
+              continue;
+            }
+          }
+
           //
           // Skip error on EM_RISCV64 and EM_LOONGARCH because no symbol name is built
           // from RISC-V and LoongArch toolchain.
