@@ -2,13 +2,7 @@
 # This file is used to be the main entrance of EOT tool
 #
 # Copyright (c) 2008 - 2018, Intel Corporation. All rights reserved.<BR>
-# This program and the accompanying materials
-# are licensed and made available under the terms and conditions of the BSD License
-# which accompanies this distribution.  The full text of the license may be found at
-# http://opensource.org/licenses/bsd-license.php
-#
-# THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-# WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+# SPDX-License-Identifier: BSD-2-Clause-Patent
 #
 
 ##
@@ -21,7 +15,8 @@ import Eot.EotGlobalData as EotGlobalData
 from optparse import OptionParser
 from Common.StringUtils import NormPath
 from Common import BuildToolError
-from Common.Misc import GuidStructureStringToGuidString, sdict
+from Common.Misc import GuidStructureStringToGuidString
+from collections import OrderedDict as sdict
 from Eot.Parser import *
 from Eot.InfParserLite import EdkInfParser
 from Common.StringUtils import GetSplitValueList
@@ -30,7 +25,6 @@ from Eot import Database
 from array import array
 from Eot.Report import Report
 from Common.BuildVersion import gBUILD_VERSION
-from Eot.Parser import ConvertGuid
 from Common.LongFilePathSupport import OpenLongFilePath as open
 import struct
 import uuid
@@ -390,7 +384,7 @@ class FirmwareVolume(Image):
         FfsDxeCoreGuid = None
         FfsPeiPrioriGuid = None
         FfsDxePrioriGuid = None
-        for FfsID in self.UnDispatchedFfsDict.keys():
+        for FfsID in list(self.UnDispatchedFfsDict.keys()):
             Ffs = self.UnDispatchedFfsDict[FfsID]
             if Ffs.Type == 0x03:
                 FfsSecCoreGuid = FfsID
@@ -496,7 +490,7 @@ class FirmwareVolume(Image):
     def DisPatchDxe(self, Db):
         IsInstalled = False
         ScheduleList = sdict()
-        for FfsID in self.UnDispatchedFfsDict.keys():
+        for FfsID in list(self.UnDispatchedFfsDict.keys()):
             CouldBeLoaded = False
             DepexString = ''
             FileDepex = None
@@ -561,7 +555,7 @@ class FirmwareVolume(Image):
 
     def DisPatchPei(self, Db):
         IsInstalled = False
-        for FfsID in self.UnDispatchedFfsDict.keys():
+        for FfsID in list(self.UnDispatchedFfsDict.keys()):
             CouldBeLoaded = True
             DepexString = ''
             FileDepex = None
@@ -929,8 +923,6 @@ class Ffs(Image):
     _SIZE_      = struct.Struct("20x 3B")
     _STATE_     = struct.Struct("23x 1B")
 
-    VTF_GUID = "1BA0062E-C779-4582-8566-336AE8F78F09"
-
     FFS_ATTRIB_FIXED              = 0x04
     FFS_ATTRIB_DATA_ALIGNMENT     = 0x38
     FFS_ATTRIB_CHECKSUM           = 0x40
@@ -1017,9 +1009,6 @@ class Ffs(Image):
     def Pack(self):
         pass
 
-    def SetFreeSpace(self, Size):
-        self.FreeSpace = Size
-
     def _GetGuid(self):
         return gGuidStringFormat % self.Name
 
@@ -1106,7 +1095,7 @@ class MultipleFv(FirmwareVolume):
             Fv.frombuffer(Buf, 0, len(Buf))
 
             self.BasicInfo.append([Fv.Name, Fv.FileSystemGuid, Fv.Size])
-            self.FfsDict.append(Fv.FfsDict)
+            self.FfsDict.update(Fv.FfsDict)
 
 ## Class Eot
 #
@@ -1516,7 +1505,7 @@ class Eot(object):
                             % (Identifier, '.NotifyPpi', '->NotifyPpi', MODEL_IDENTIFIER_FUNCTION_CALLING)
             SearchPpi(SqlCommand, Identifier, SourceFileID, SourceFileFullPath, ItemMode)
 
-            # Find Procotols
+            # Find Protocols
             ItemMode = 'Produced'
             SqlCommand = """select Value, Name, BelongsToFile, StartLine, EndLine from %s
                             where (Name like '%%%s%%' or Name like '%%%s%%' or Name like '%%%s%%' or Name like '%%%s%%') and Model = %s""" \
@@ -1598,7 +1587,7 @@ class Eot(object):
             if not InfFile:
                 continue
             EdkLogger.quiet("Parsing %s ..."  % str(InfFile))
-            EdkInfParser(InfFile, EotGlobalData.gDb, Inf_Files[InfFile], '')
+            EdkInfParser(InfFile, EotGlobalData.gDb, Inf_Files[InfFile])
 
         EotGlobalData.gDb.Conn.commit()
         EdkLogger.quiet("Building database for meta data files done!")
