@@ -21,6 +21,7 @@ from .GuidSection import GuidSection
 from .FvImageSection import FvImageSection
 from .Ffs import FdfFvFileTypeToFileType
 from .GenFdsGlobalVariable import GenFdsGlobalVariable
+import shutil
 
 ## generate FFS from FILE
 #
@@ -32,8 +33,6 @@ class FileStatement (FileStatementClassObject):
     #
     def __init__(self):
         FileStatementClassObject.__init__(self)
-        self.CurrentLineNum = None
-        self.CurrentLineContent = None
         self.FileName = None
         self.InfFileName = None
         self.SubAlignment = None
@@ -67,6 +66,8 @@ class FileStatement (FileStatementClassObject):
         if FvName:
             Str += FvName
         OutputDir = os.path.join(GenFdsGlobalVariable.FfsDir, Str)
+        if os.path.exists(OutputDir):
+            shutil.rmtree(OutputDir)
         if not os.path.exists(OutputDir):
             os.makedirs(OutputDir)
 
