@@ -65,7 +65,6 @@ def resetFdsGlobalVariable():
     # will be FvDir + os.sep + 'Ffs'
     GenFdsGlobalVariable.FfsDir = ''
     GenFdsGlobalVariable.FdfParser = None
-    GenFdsGlobalVariable.LibDir = ''
     GenFdsGlobalVariable.WorkSpace = None
     GenFdsGlobalVariable.WorkSpaceDir = ''
     GenFdsGlobalVariable.ConfDir = ''
@@ -134,7 +133,7 @@ def GenFdsApi(FdsCommandDict, WorkSpaceDataBase=None):
             EdkLogger.error("GenFds", PARAMETER_INVALID, "WORKSPACE is invalid",
                             ExtraData="Please use '-w' switch to pass it or set the WORKSPACE environment variable.")
         else:
-            Workspace = os.path.normcase(FdsCommandDict.get("Workspace",os.environ.get('WORKSPACE')))
+            Workspace = os.path.normpath(FdsCommandDict.get("Workspace",os.environ.get('WORKSPACE')))
             GenFdsGlobalVariable.WorkSpaceDir = Workspace
             if FdsCommandDict.get("debug"):
                 GenFdsGlobalVariable.VerboseLogger("Using Workspace:" + Workspace)
@@ -200,7 +199,7 @@ def GenFdsApi(FdsCommandDict, WorkSpaceDataBase=None):
                 ConfDirectoryPath = os.path.join(GenFdsGlobalVariable.WorkSpaceDir, ConfDirectoryPath)
         else:
             if "CONF_PATH" in os.environ:
-                ConfDirectoryPath = os.path.normcase(os.environ["CONF_PATH"])
+                ConfDirectoryPath = os.path.normpath(os.environ["CONF_PATH"])
             else:
                 # Get standard WORKSPACE/Conf, use the absolute path to the WORKSPACE/Conf
                 ConfDirectoryPath = mws.join(GenFdsGlobalVariable.WorkSpaceDir, 'Conf')
@@ -549,41 +548,6 @@ class GenFds(object):
 
         return GenFdsGlobalVariable.FfsCmdDict
 
-    ## GetFvBlockSize()
-    #
-    #   @param  FvObj           Whose block size to get
-    #   @retval int             Block size value
-    #
-    @staticmethod
-    def GetFvBlockSize(FvObj):
-        DefaultBlockSize = 0x1
-        FdObj = None
-        if GenFds.OnlyGenerateThisFd is not None and GenFds.OnlyGenerateThisFd.upper() in GenFdsGlobalVariable.FdfParser.Profile.FdDict:
-            FdObj = GenFdsGlobalVariable.FdfParser.Profile.FdDict[GenFds.OnlyGenerateThisFd.upper()]
-        if FdObj is None:
-            for ElementFd in GenFdsGlobalVariable.FdfParser.Profile.FdDict.values():
-                for ElementRegion in ElementFd.RegionList:
-                    if ElementRegion.RegionType == BINARY_FILE_TYPE_FV:
-                        for ElementRegionData in ElementRegion.RegionDataList:
-                            if ElementRegionData is not None and ElementRegionData.upper() == FvObj.UiFvName:
-                                if FvObj.BlockSizeList != []:
-                                    return FvObj.BlockSizeList[0][0]
-                                else:
-                                    return ElementRegion.BlockSizeOfRegion(ElementFd.BlockSizeList)
-            if FvObj.BlockSizeList != []:
-                return FvObj.BlockSizeList[0][0]
-            return DefaultBlockSize
-        else:
-            for ElementRegion in FdObj.RegionList:
-                    if ElementRegion.RegionType == BINARY_FILE_TYPE_FV:
-                        for ElementRegionData in ElementRegion.RegionDataList:
-                            if ElementRegionData is not None and ElementRegionData.upper() == FvObj.UiFvName:
-                                if FvObj.BlockSizeList != []:
-                                    return FvObj.BlockSizeList[0][0]
-                                else:
-                                    return ElementRegion.BlockSizeOfRegion(ElementFd.BlockSizeList)
-            return DefaultBlockSize
-
     ## DisplayFvSpaceInfo()
     #
     #   @param  FvObj           Whose block size to get
