@@ -53,12 +53,12 @@ def GetLibInstanceInfo(String, WorkSpace, LineNo, CurrentInfFileName):
     #
     # To deal with library instance specified by GUID and version
     #
-    RegFormatGuidPattern = re.compile("\s*([0-9a-fA-F]){8}-"
+    RegFormatGuidPattern = re.compile(r"\s*([0-9a-fA-F]){8}-"
                                        "([0-9a-fA-F]){4}-"
                                        "([0-9a-fA-F]){4}-"
                                        "([0-9a-fA-F]){4}-"
-                                       "([0-9a-fA-F]){12}\s*")
-    VersionPattern = re.compile('[\t\s]*\d+(\.\d+)?[\t\s]*')
+                                       r"([0-9a-fA-F]){12}\s*")
+    VersionPattern = re.compile(r'[\t\s]*\d+(\.\d+)?[\t\s]*')
     GuidMatchedObj = RegFormatGuidPattern.search(String)
 
     if String.upper().startswith('GUID') and GuidMatchedObj and 'Version' in String:
@@ -75,8 +75,8 @@ def GetLibInstanceInfo(String, WorkSpace, LineNo, CurrentInfFileName):
     FileLinesList = GetFileLineContent(String, WorkSpace, LineNo, OriginalString)
 
 
-    ReFindFileGuidPattern = re.compile("^\s*FILE_GUID\s*=.*$")
-    ReFindVerStringPattern = re.compile("^\s*VERSION_STRING\s*=.*$")
+    ReFindFileGuidPattern = re.compile(r"^\s*FILE_GUID\s*=.*$")
+    ReFindVerStringPattern = re.compile(r"^\s*VERSION_STRING\s*=.*$")
 
     for Line in FileLinesList:
         if ReFindFileGuidPattern.match(Line):
@@ -106,8 +106,8 @@ def GetPackageListInfo(FileNameString, WorkSpace, LineNo):
 
     FileLinesList = GetFileLineContent(FileNameString, WorkSpace, LineNo, '')
 
-    RePackageHeader = re.compile('^\s*\[Packages.*\].*$')
-    ReDefineHeader = re.compile('^\s*\[Defines].*$')
+    RePackageHeader = re.compile(r'^\s*\[Packages.*\].*$')
+    ReDefineHeader = re.compile(r'^\s*\[Defines].*$')
 
     PackageHederFlag = False
     DefineHeaderFlag = False
@@ -215,69 +215,3 @@ def GetFileLineContent(FileName, WorkSpace, LineNo, OriginalString):
     FileLinesList = ProcessLineExtender(FileLinesList)
 
     return FileLinesList
-
-##
-# Get all INF files from current workspace
-#
-#
-def GetInfsFromWorkSpace(WorkSpace):
-    InfFiles = []
-    for top, dirs, files in os.walk(WorkSpace):
-        dirs = dirs # just for pylint
-        for File in files:
-            if File.upper().endswith(".INF"):
-                InfFiles.append(os.path.join(top, File))
-
-    return InfFiles
-
-##
-# Get GUID and version from library instance file
-#
-#
-def GetGuidVerFormLibInstance(Guid, Version, WorkSpace, CurrentInfFileName):
-    for InfFile in GetInfsFromWorkSpace(WorkSpace):
-        try:
-            if InfFile.strip().upper() == CurrentInfFileName.strip().upper():
-                continue
-            InfFile = InfFile.replace('\\', '/')
-            if InfFile not in GlobalData.gLIBINSTANCEDICT:
-                InfFileObj = open(InfFile, "r")
-                GlobalData.gLIBINSTANCEDICT[InfFile] = InfFileObj
-            else:
-                InfFileObj = GlobalData.gLIBINSTANCEDICT[InfFile]
-
-        except BaseException:
-            Logger.Error("InfParser",
-                         ToolError.FILE_READ_FAILURE,
-                         ST.ERR_FILE_OPEN_FAILURE,
-                         File=InfFile)
-        try:
-            FileLinesList = InfFileObj.readlines()
-            FileLinesList = ProcessLineExtender(FileLinesList)
-
-            ReFindFileGuidPattern = re.compile("^\s*FILE_GUID\s*=.*$")
-            ReFindVerStringPattern = re.compile("^\s*VERSION_STRING\s*=.*$")
-
-            for Line in FileLinesList:
-                if ReFindFileGuidPattern.match(Line):
-                    FileGuidString = Line
-                if ReFindVerStringPattern.match(Line):
-                    VerString = Line
-
-            if FileGuidString:
-                FileGuidString = GetSplitValueList(FileGuidString, '=', 1)[1]
-            if VerString:
-                VerString = GetSplitValueList(VerString, '=', 1)[1]
-
-            if FileGuidString.strip().upper() == Guid.upper() and \
-                VerString.strip().upper() == Version.upper():
-                return Guid, Version
-
-        except BaseException:
-            Logger.Error("InfParser", ToolError.FILE_READ_FAILURE, ST.ERR_FILE_OPEN_FAILURE, File=InfFile)
-        finally:
-            InfFileObj.close()
-
-    return '', ''
-
-
