@@ -2,15 +2,9 @@ from __future__ import print_function
 ## @file
 # Utility functions and classes for BaseTools unit tests
 #
-#  Copyright (c) 2008 - 2018, Intel Corporation. All rights reserved.<BR>
+#  Copyright (c) 2008 - 2025, Intel Corporation. All rights reserved.<BR>
 #
-#  This program and the accompanying materials
-#  are licensed and made available under the terms and conditions of the BSD License
-#  which accompanies this distribution.  The full text of the license may be found at
-#  http://opensource.org/licenses/bsd-license.php
-#
-#  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-#  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+#  SPDX-License-Identifier: BSD-2-Clause-Patent
 #
 
 ##
@@ -24,6 +18,7 @@ import shutil
 import subprocess
 import sys
 import unittest
+import codecs
 
 TestsDir = os.path.realpath(os.path.split(sys.argv[0])[0])
 BaseToolsDir = os.path.realpath(os.path.join(TestsDir, '..'))
@@ -40,7 +35,7 @@ if PythonSourceDir not in sys.path:
 
 def MakeTheTestSuite(localItems):
     tests = []
-    for name, item in localItems.iteritems():
+    for name, item in localItems.items():
         if isinstance(item, type):
             if issubclass(item, unittest.TestCase):
                 tests.append(unittest.TestLoader().loadTestsFromTestCase(item))
@@ -50,7 +45,7 @@ def MakeTheTestSuite(localItems):
 
 def GetBaseToolsPaths():
     if sys.platform in ('win32', 'win64'):
-        return [ os.path.join(BaseToolsDir, 'Bin', sys.platform.title()) ]
+        return [ os.path.join(BaseToolsDir, 'Bin', x) for x in ('Win32', 'Win64') ]
     else:
         uname = os.popen('uname -sm').read().strip()
         for char in (' ', '/'):
@@ -114,6 +109,8 @@ class BaseToolsTest(unittest.TestCase):
         else: logFile = None
 
         if toolName is None: toolName = self.toolName
+        if sys.platform == "win32":
+            toolName += ".exe"
         bin = self.FindToolBin(toolName)
         if logFile is not None:
             logFile = open(os.path.join(self.testDir, logFile), 'w')
@@ -140,15 +137,18 @@ class BaseToolsTest(unittest.TestCase):
         return open(os.path.join(self.testDir, fileName), mode)
 
     def ReadTmpFile(self, fileName):
-        f = open(self.GetTmpFilePath(fileName), 'r')
+        f = open(self.GetTmpFilePath(fileName), 'rb')
         data = f.read()
         f.close()
         return data
 
     def WriteTmpFile(self, fileName, data):
-        f = open(self.GetTmpFilePath(fileName), 'w')
-        f.write(data)
-        f.close()
+        if isinstance(data, bytes):
+            with open(self.GetTmpFilePath(fileName), 'wb') as f:
+                f.write(data)
+        else:
+            with codecs.open(self.GetTmpFilePath(fileName), 'w', encoding='utf-8') as f:
+                f.write(data)
 
     def GenRandomFileData(self, fileName, minlen = None, maxlen = None):
         if maxlen is None: maxlen = minlen
@@ -161,7 +161,7 @@ class BaseToolsTest(unittest.TestCase):
         if maxlen is None: maxlen = minlen
         return ''.join(
             [chr(random.randint(0, 255))
-             for x in xrange(random.randint(minlen, maxlen))
+             for x in range(random.randint(minlen, maxlen))
             ])
 
     def setUp(self):
