@@ -16,6 +16,7 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 #include <Library/BaseMemoryLib.h>
 #include <Library/DebugLib.h>
 #include <Library/PrintLib.h>
+#include <Library/TimerLib.h>
 
 #define OPENSSLDIR  ""
 #define ENGINESDIR  ""
@@ -23,17 +24,6 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 
 #define MAX_STRING_SIZE  0x1000
 
-//
-// We already have "no-ui" in out Configure invocation.
-// but the code still fails to compile.
-// Ref:  https://github.com/openssl/openssl/issues/8904
-//
-// This is defined in CRT library(stdio.h).
-//
-#ifndef BUFSIZ
-#define BUFSIZ  8192
-#endif
-
 //
 // OpenSSL relies on explicit configuration for word size in crypto/bn,
 // but we want it to be automatically inferred from the target. So we
@@ -87,6 +77,11 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 #define UINT_MAX      0xFFFFFFFF      /* Maximum unsigned int value */
 #define ULONG_MAX     0xFFFFFFFF      /* Maximum unsigned long value */
 #define CHAR_BIT      8               /* Number of bits in a char */
+#define SIZE_MAX      0xFFFFFFFF      /* Maximum unsigned size_t */
+
+#define INT32_MIN   INT_MIN
+#define INT32_MAX   INT_MAX
+#define UINT32_MAX  UINT_MAX
 
 //
 // Address families.
@@ -107,9 +102,10 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 typedef UINTN   size_t;
 typedef UINTN   off_t;
 typedef UINTN   u_int;
+typedef UINTN   intptr_t;
 typedef INTN    ptrdiff_t;
 typedef INTN    ssize_t;
-typedef INT32   time_t;
+typedef INT64   time_t;
 typedef UINT8   __uint8_t;
 typedef UINT8   sa_family_t;
 typedef UINT8   u_char;
@@ -145,6 +141,8 @@ struct timeval {
   long    tv_usec;  /* time value, in microseconds */
 };
 
+struct timezone;
+
 struct sockaddr {
   __uint8_t      sa_len;      /* total length */
   sa_family_t    sa_family;   /* address family */
@@ -156,6 +154,7 @@ struct sockaddr {
 //
 extern int   errno;
 extern FILE  *stderr;
+extern long  timezone;
 
 //
 // Function prototypes of CRT Library routines
@@ -333,6 +332,22 @@ gmtime     (
   const time_t *
   );
 
+unsigned int
+sleep (
+  unsigned int  seconds
+  );
+
+int
+gettimeofday (
+  struct timeval   *tv,
+  struct timezone  *tz
+  );
+
+time_t
+mktime (
+  struct tm  *t
+  );
+
 uid_t
 getuid      (
   void
@@ -402,25 +417,41 @@ strcpy (
   const char  *strSource
   );
 
+char *
+strncpy (
+  char        *strDest,
+  const char  *strSource,
+  size_t      count
+  );
+
+char *
+strcat (
+  char        *strDest,
+  const char  *strSource
+  );
+
+char *
+strpbrk (
+  const char  *s,
+  const char  *accept
+  );
+
 //
 // Macros that directly map functions to BaseLib, BaseMemoryLib, and DebugLib functions
 //
-#define memcpy(dest, source, count)         CopyMem(dest,source,(UINTN)(count))
-#define memset(dest, ch, count)             SetMem(dest,(UINTN)(count),(UINT8)(ch))
-#define memchr(buf, ch, count)              ScanMem8(buf,(UINTN)(count),(UINT8)ch)
-#define memcmp(buf1, buf2, count)           (int)(CompareMem(buf1,buf2,(UINTN)(count)))
-#define memmove(dest, source, count)        CopyMem(dest,source,(UINTN)(count))
-#define strlen(str)                         (size_t)(AsciiStrnLenS(str,MAX_STRING_SIZE))
-#define strncpy(strDest, strSource, count)  AsciiStrnCpyS(strDest,MAX_STRING_SIZE,strSource,(UINTN)count)
-#define strcat(strDest, strSource)          AsciiStrCatS(strDest,MAX_STRING_SIZE,strSource)
-#define strncmp(string1, string2, count)    (int)(AsciiStrnCmp(string1,string2,(UINTN)(count)))
-#define strcasecmp(str1, str2)              (int)AsciiStriCmp(str1,str2)
-#define strstr(s1, s2)                      AsciiStrStr(s1,s2)
-#define sprintf(buf, ...)                   AsciiSPrint(buf,MAX_STRING_SIZE,__VA_ARGS__)
-#define localtime(timer)                    NULL
+#define memcpy(dest, source, count)       CopyMem(dest,source,(UINTN)(count))
+#define memset(dest, ch, count)           SetMem(dest,(UINTN)(count),(UINT8)(ch))
+#define memchr(buf, ch, count)            ScanMem8(buf,(UINTN)(count),(UINT8)ch)
+#define memcmp(buf1, buf2, count)         (int)(CompareMem(buf1,buf2,(UINTN)(count)))
+#define memmove(dest, source, count)      CopyMem(dest,source,(UINTN)(count))
+#define strlen(str)                       (size_t)(AsciiStrnLenS(str,MAX_STRING_SIZE))
+#define strncmp(string1, string2, count)  (int)(AsciiStrnCmp(string1,string2,(UINTN)(count)))
+#define strcasecmp(str1, str2)            (int)AsciiStriCmp(str1,str2)
+#define strstr(s1, s2)                    AsciiStrStr(s1,s2)
+#define sprintf(buf, ...)                 AsciiSPrint(buf,MAX_STRING_SIZE,__VA_ARGS__)
+#define localtime(timer)                  NULL
 #define assert(expression)
 #define offsetof(type, member)  OFFSET_OF(type,member)
 #define atoi(nptr)              AsciiStrDecimalToUintn(nptr)
-#define gettimeofday(tvp, tz)   do { (tvp)->tv_sec = time(NULL); (tvp)->tv_usec = 0; } while (0)
 
 #endif
