@@ -53,6 +53,8 @@ def openssl_configure(openssldir, target, ec = True):
         'no-module',
         'no-md4',
         'no-mdc2',
+        'no-ml-dsa',
+        'no-ml-kem',
         'no-multiblock',
         'no-nextprotoneg',
         'no-pic',
@@ -62,6 +64,7 @@ def openssl_configure(openssldir, target, ec = True):
         'no-padlockeng',
         'no-poly1305',
         'no-posix-io',
+        'no-quic',
         'no-rc2',
         'no-rc4',
         'no-rc5',
@@ -72,6 +75,7 @@ def openssl_configure(openssldir, target, ec = True):
         'no-shared',
         'no-siphash',
         'no-siv',
+        'no-slh-dsa',
         'no-sm2',
         'no-sm4',
         'no-sock',
