@@ -17,7 +17,7 @@
 #include <Protocol/DebugPort.h>
 
 EFI_DEBUGPORT_PROTOCOL  *gDebugPort = NULL;
-UINTN                   gTimeOut    = 0;
+UINT32                  gTimeOut    = 0;
 
 /**
   The constructor function initializes the UART.
@@ -51,7 +51,7 @@ GdbSerialLibDebugPortConstructor (
   data buts, and stop bits on a serial device. This call is optional as the serial
   port will be set up with defaults base on PCD values.
 
-  @param  BaudRate         The requested baud rate. A BaudRate value of 0 will use the the
+  @param  BaudRate         The requested baud rate. A BaudRate value of 0 will use the
                            device's default interface speed.
   @param  Parity           The type of parity to use on this serial device. A Parity value of
                            DefaultParity will use the device's default parity value.
