@@ -83,20 +83,29 @@ WaitForNotify (
   DEBUG ((DEBUG_INFO | DEBUG_INIT, "FSP is waiting for NOTIFY\n"));
   FspSiliconInitDone2 (EFI_SUCCESS);
 
+  //
+  // Fsp notify phase check
+  //
+  DEBUG ((DEBUG_INFO | DEBUG_INIT, "Fsp Notify Phase called ...\n"));
+  ASSERT (GetFspApiCallingIndex () == NotifyPhaseApiIndex);
+
+  while (GetFspApiCallingIndex () != NotifyPhaseApiIndex) {
+    SetFspApiReturnStatus (EFI_UNSUPPORTED);
+    Pei2LoaderSwitchStack ();
+  }
+
   //
   // BootLoader called FSP again through NotifyPhase
   //
   FspWaitForNotify ();
 
-  if (GetFspGlobalDataPointer ()->FspMode == FSP_IN_API_MODE) {
-    //
-    // Should not come here
-    //
-    while (TRUE) {
-      DEBUG ((DEBUG_ERROR, "No FSP API should be called after FSP is DONE!\n"));
-      SetFspApiReturnStatus (EFI_UNSUPPORTED);
-      Pei2LoaderSwitchStack ();
-    }
+  //
+  // Should not come here
+  //
+  while (TRUE) {
+    DEBUG ((DEBUG_ERROR, "No FSP API should be called after FSP is DONE!\n"));
+    SetFspApiReturnStatus (EFI_UNSUPPORTED);
+    Pei2LoaderSwitchStack ();
   }
 
   return EFI_SUCCESS;
