@@ -837,6 +837,10 @@ XhcTransfer (
   }
 
   Xhc->PciIo->Flush (Xhc->PciIo);
+  //
+  // Do not free URB data, since it is passed in as an external argument
+  // and allocated and managed by the caller.
+  //
   XhcFreeUrb (Xhc, Urb);
   return Status;
 }
@@ -1227,8 +1231,9 @@ ON_EXIT:
                                 sending or receiving.
   @param  DataBuffersNumber     Number of data buffers prepared for the transfer.
   @param  Data                  Array of pointers to the buffers of data to transmit
-                                from or receive into.
-  @param  DataLength            The lenght of the data buffer.
+                                from or receive into. The caller is responsible for freeing
+                                the buffers after the transfers are completed.
+  @param  DataLength            The length of the data buffer.
   @param  DataToggle            On input, the initial data toggle for the transfer;
                                 On output, it is updated to to next data toggle to
                                 use of the subsequent bulk transfer.
