@@ -2,14 +2,8 @@
 
     Unified interface for RootHub and Hub.
 
-Copyright (c) 2007 - 2012, Intel Corporation. All rights reserved.<BR> 
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+Copyright (c) 2007 - 2018, Intel Corporation. All rights reserved.<BR>
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -23,19 +17,19 @@ WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
 // bit maps.
 //
 USB_CHANGE_FEATURE_MAP  mHubFeatureMap[] = {
-  {USB_PORT_STAT_C_CONNECTION,  EfiUsbPortConnectChange},
-  {USB_PORT_STAT_C_ENABLE,      EfiUsbPortEnableChange},
-  {USB_PORT_STAT_C_SUSPEND,     EfiUsbPortSuspendChange},
-  {USB_PORT_STAT_C_OVERCURRENT, EfiUsbPortOverCurrentChange},
-  {USB_PORT_STAT_C_RESET,       EfiUsbPortResetChange}
+  { USB_PORT_STAT_C_CONNECTION,  EfiUsbPortConnectChange     },
+  { USB_PORT_STAT_C_ENABLE,      EfiUsbPortEnableChange      },
+  { USB_PORT_STAT_C_SUSPEND,     EfiUsbPortSuspendChange     },
+  { USB_PORT_STAT_C_OVERCURRENT, EfiUsbPortOverCurrentChange },
+  { USB_PORT_STAT_C_RESET,       EfiUsbPortResetChange       }
 };
 
 USB_CHANGE_FEATURE_MAP  mRootHubFeatureMap[] = {
-  {USB_PORT_STAT_C_CONNECTION,  EfiUsbPortConnectChange},
-  {USB_PORT_STAT_C_ENABLE,      EfiUsbPortEnableChange},
-  {USB_PORT_STAT_C_SUSPEND,     EfiUsbPortSuspendChange},
-  {USB_PORT_STAT_C_OVERCURRENT, EfiUsbPortOverCurrentChange},
-  {USB_PORT_STAT_C_RESET,       EfiUsbPortResetChange},
+  { USB_PORT_STAT_C_CONNECTION,  EfiUsbPortConnectChange     },
+  { USB_PORT_STAT_C_ENABLE,      EfiUsbPortEnableChange      },
+  { USB_PORT_STAT_C_SUSPEND,     EfiUsbPortSuspendChange     },
+  { USB_PORT_STAT_C_OVERCURRENT, EfiUsbPortOverCurrentChange },
+  { USB_PORT_STAT_C_RESET,       EfiUsbPortResetChange       },
 };
 
 //
@@ -43,6 +37,7 @@ USB_CHANGE_FEATURE_MAP  mRootHubFeatureMap[] = {
 // is related to an interface, these requests are sent
 // to the control endpoint of the device.
 //
+
 /**
   USB hub control transfer to set the hub depth.
 
@@ -55,11 +50,11 @@ USB_CHANGE_FEATURE_MAP  mRootHubFeatureMap[] = {
 **/
 EFI_STATUS
 UsbHubCtrlSetHubDepth (
-  IN  USB_DEVICE          *HubDev,
-  IN  UINT16              Depth
+  IN  USB_DEVICE  *HubDev,
+  IN  UINT16      Depth
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
   Status = UsbCtrlRequest (
              HubDev,
@@ -88,11 +83,11 @@ UsbHubCtrlSetHubDepth (
 **/
 EFI_STATUS
 UsbHubCtrlClearHubFeature (
-  IN USB_DEVICE           *HubDev,
-  IN UINT16               Feature
+  IN USB_DEVICE  *HubDev,
+  IN UINT16      Feature
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
   Status = UsbCtrlRequest (
              HubDev,
@@ -109,7 +104,6 @@ UsbHubCtrlClearHubFeature (
   return Status;
 }
 
-
 /**
   Clear the feature of the device's port.
 
@@ -123,12 +117,12 @@ UsbHubCtrlClearHubFeature (
 **/
 EFI_STATUS
 UsbHubCtrlClearPortFeature (
-  IN USB_DEVICE           *HubDev,
-  IN UINT8                Port,
-  IN UINT16               Feature
+  IN USB_DEVICE  *HubDev,
+  IN UINT8       Port,
+  IN UINT16      Feature
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
   //
   // In USB bus, all the port index starts from 0. But HUB
@@ -141,7 +135,7 @@ UsbHubCtrlClearPortFeature (
              USB_HUB_TARGET_PORT,
              USB_HUB_REQ_CLEAR_FEATURE,
              Feature,
-             (UINT16) (Port + 1),
+             (UINT16)(Port + 1),
              NULL,
              0
              );
@@ -149,7 +143,6 @@ UsbHubCtrlClearPortFeature (
   return Status;
 }
 
-
 /**
   Clear the transaction translate buffer if full/low
   speed control/bulk transfer failed and the transfer
@@ -169,21 +162,21 @@ UsbHubCtrlClearPortFeature (
 **/
 EFI_STATUS
 UsbHubCtrlClearTTBuffer (
-  IN USB_DEVICE           *HubDev,
-  IN UINT8                Port,
-  IN UINT16               DevAddr,
-  IN UINT16               EpNum,
-  IN UINT16               EpType
+  IN USB_DEVICE  *HubDev,
+  IN UINT8       Port,
+  IN UINT16      DevAddr,
+  IN UINT16      EpNum,
+  IN UINT16      EpType
   )
 {
-  EFI_STATUS              Status;
-  UINT16                  Value;
+  EFI_STATUS  Status;
+  UINT16      Value;
 
   //
   // Check USB2.0 spec page 424 for wValue's encoding
   //
-  Value = (UINT16) ((EpNum & 0x0F) | (DevAddr << 4) |
-          ((EpType & 0x03) << 11) | ((EpNum & 0x80) << 15));
+  Value = (UINT16)((EpNum & 0x0F) | (DevAddr << 4) |
+                   ((EpType & 0x03) << 11) | ((EpNum & 0x80) << 15));
 
   Status = UsbCtrlRequest (
              HubDev,
@@ -192,7 +185,7 @@ UsbHubCtrlClearTTBuffer (
              USB_HUB_TARGET_PORT,
              USB_HUB_REQ_CLEAR_TT,
              Value,
-             (UINT16) (Port + 1),
+             (UINT16)(Port + 1),
              NULL,
              0
              );
@@ -201,42 +194,7 @@ UsbHubCtrlClearTTBuffer (
 }
 
 /**
-  Usb hub control transfer to get the super speed hub descriptor.
-
-  @param  HubDev                The hub device.
-  @param  Buf                   The buffer to hold the descriptor.
-
-  @retval EFI_SUCCESS           The hub descriptor is retrieved.
-  @retval Others                Failed to retrieve the hub descriptor.
-
-**/
-EFI_STATUS
-UsbHubCtrlGetSuperSpeedHubDesc (
-  IN  USB_DEVICE          *HubDev,
-  OUT VOID                *Buf
-  )
-{
-  EFI_STATUS              Status;
-  
-  Status = EFI_INVALID_PARAMETER;
-  
-  Status = UsbCtrlRequest (
-             HubDev,
-             EfiUsbDataIn,
-             USB_REQ_TYPE_CLASS,
-             USB_HUB_TARGET_HUB,
-             USB_HUB_REQ_GET_DESC,
-             (UINT16) (USB_DESC_TYPE_HUB_SUPER_SPEED << 8),
-             0,
-             Buf,
-             32
-             );
-
-  return Status;
-}
-
-/**
-  Usb hub control transfer to get the hub descriptor.
+  Usb hub control transfer to get the (super speed) hub descriptor.
 
   @param  HubDev                The hub device.
   @param  Buf                   The buffer to hold the descriptor.
@@ -248,12 +206,17 @@ UsbHubCtrlGetSuperSpeedHubDesc (
 **/
 EFI_STATUS
 UsbHubCtrlGetHubDesc (
-  IN  USB_DEVICE          *HubDev,
-  OUT VOID                *Buf,
-  IN  UINTN               Len
+  IN  USB_DEVICE  *HubDev,
+  OUT VOID        *Buf,
+  IN  UINTN       Len
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
+  UINT8       DescType;
+
+  DescType = (HubDev->Speed == EFI_USB_SPEED_SUPER) ?
+             USB_DESC_TYPE_HUB_SUPER_SPEED :
+             USB_DESC_TYPE_HUB;
 
   Status = UsbCtrlRequest (
              HubDev,
@@ -261,7 +224,7 @@ UsbHubCtrlGetHubDesc (
              USB_REQ_TYPE_CLASS,
              USB_HUB_TARGET_HUB,
              USB_HUB_REQ_GET_DESC,
-             (UINT16) (USB_DESC_TYPE_HUB << 8),
+             (UINT16)(DescType << 8),
              0,
              Buf,
              Len
@@ -270,7 +233,6 @@ UsbHubCtrlGetHubDesc (
   return Status;
 }
 
-
 /**
   Usb hub control transfer to get the hub status.
 
@@ -283,11 +245,11 @@ UsbHubCtrlGetHubDesc (
 **/
 EFI_STATUS
 UsbHubCtrlGetHubStatus (
-  IN  USB_DEVICE          *HubDev,
-  OUT UINT32              *State
+  IN  USB_DEVICE  *HubDev,
+  OUT UINT32      *State
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
   Status = UsbCtrlRequest (
              HubDev,
@@ -304,7 +266,6 @@ UsbHubCtrlGetHubStatus (
   return Status;
 }
 
-
 /**
   Usb hub control transfer to get the port status.
 
@@ -318,12 +279,12 @@ UsbHubCtrlGetHubStatus (
 **/
 EFI_STATUS
 UsbHubCtrlGetPortStatus (
-  IN  USB_DEVICE          *HubDev,
-  IN  UINT8               Port,
-  OUT VOID                *State
+  IN  USB_DEVICE  *HubDev,
+  IN  UINT8       Port,
+  OUT VOID        *State
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
   //
   // In USB bus, all the port index starts from 0. But HUB
@@ -338,7 +299,7 @@ UsbHubCtrlGetPortStatus (
              USB_HUB_TARGET_PORT,
              USB_HUB_REQ_GET_STATUS,
              0,
-             (UINT16) (Port + 1),
+             (UINT16)(Port + 1),
              State,
              4
              );
@@ -346,75 +307,6 @@ UsbHubCtrlGetPortStatus (
   return Status;
 }
 
-
-/**
-  Usb hub control transfer to reset the TT (Transaction Transaltor).
-
-  @param  HubDev                The hub device.
-  @param  Port                  The port of the hub.
-
-  @retval EFI_SUCCESS           The TT of the hub is reset.
-  @retval Others                Failed to reset the port.
-
-**/
-EFI_STATUS
-UsbHubCtrlResetTT (
-  IN  USB_DEVICE          *HubDev,
-  IN  UINT8               Port
-  )
-{
-  EFI_STATUS              Status;
-
-  Status = UsbCtrlRequest (
-             HubDev,
-             EfiUsbNoData,
-             USB_REQ_TYPE_CLASS,
-             USB_HUB_TARGET_HUB,
-             USB_HUB_REQ_RESET_TT,
-             0,
-             (UINT16) (Port + 1),
-             NULL,
-             0
-             );
-
-  return Status;
-}
-
-
-/**
-  Usb hub control transfer to set the hub feature.
-
-  @param  HubDev                The hub device.
-  @param  Feature               The feature to set.
-
-  @retval EFI_SUCESS            The feature is set for the hub.
-  @retval Others                Failed to set the feature.
-
-**/
-EFI_STATUS
-UsbHubCtrlSetHubFeature (
-  IN  USB_DEVICE          *HubDev,
-  IN  UINT8               Feature
-  )
-{
-  EFI_STATUS              Status;
-
-  Status = UsbCtrlRequest (
-             HubDev,
-             EfiUsbNoData,
-             USB_REQ_TYPE_CLASS,
-             USB_HUB_TARGET_HUB,
-             USB_HUB_REQ_SET_FEATURE,
-             Feature,
-             0,
-             NULL,
-             0
-             );
-
-  return Status;
-}
-
-
 /**
   Usb hub control transfer to set the port feature.
 
@@ -428,12 +320,12 @@ UsbHubCtrlSetHubFeature (
 **/
 EFI_STATUS
 UsbHubCtrlSetPortFeature (
-  IN USB_DEVICE           *HubDev,
-  IN UINT8                Port,
-  IN UINT8                Feature
+  IN USB_DEVICE  *HubDev,
+  IN UINT8       Port,
+  IN UINT8       Feature
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
   //
   // In USB bus, all the port index starts from 0. But HUB
@@ -446,7 +338,7 @@ UsbHubCtrlSetPortFeature (
              USB_HUB_TARGET_PORT,
              USB_HUB_REQ_SET_FEATURE,
              Feature,
-             (UINT16) (Port + 1),
+             (UINT16)(Port + 1),
              NULL,
              0
              );
@@ -454,7 +346,6 @@ UsbHubCtrlSetPortFeature (
   return Status;
 }
 
-
 /**
   Read the whole usb hub descriptor. It is necessary
   to do it in two steps because hub descriptor is of
@@ -473,35 +364,23 @@ UsbHubReadDesc (
   OUT EFI_USB_HUB_DESCRIPTOR  *HubDesc
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
-  if (HubDev->Speed == EFI_USB_SPEED_SUPER) {
-    //
-    // Get the super speed hub descriptor
-    //
-    Status = UsbHubCtrlGetSuperSpeedHubDesc (HubDev, HubDesc);
-  } else {
+  //
+  // First get the hub descriptor length
+  //
+  Status = UsbHubCtrlGetHubDesc (HubDev, HubDesc, 2);
 
-    //
-    // First get the hub descriptor length
-    //
-    Status = UsbHubCtrlGetHubDesc (HubDev, HubDesc, 2);
-
-    if (EFI_ERROR (Status)) {
-      return Status;
-    }
-
-    //
-    // Get the whole hub descriptor
-    //
-    Status = UsbHubCtrlGetHubDesc (HubDev, HubDesc, HubDesc->Length);
+  if (EFI_ERROR (Status)) {
+    return Status;
   }
 
-  return Status;
+  //
+  // Get the whole hub descriptor
+  //
+  return UsbHubCtrlGetHubDesc (HubDev, HubDesc, HubDesc->Length);
 }
 
-
-
 /**
   Ack the hub change bits. If these bits are not ACKed, Hub will
   always return changed bit map from its interrupt endpoint.
@@ -514,13 +393,13 @@ UsbHubReadDesc (
 **/
 EFI_STATUS
 UsbHubAckHubStatus (
-  IN  USB_DEVICE         *HubDev
+  IN  USB_DEVICE  *HubDev
   )
 {
-  EFI_USB_PORT_STATUS     HubState;
-  EFI_STATUS              Status;
+  EFI_USB_PORT_STATUS  HubState;
+  EFI_STATUS           Status;
 
-  Status = UsbHubCtrlGetHubStatus (HubDev, (UINT32 *) &HubState);
+  Status = UsbHubCtrlGetHubStatus (HubDev, (UINT32 *)&HubState);
 
   if (EFI_ERROR (Status)) {
     return Status;
@@ -537,7 +416,6 @@ UsbHubAckHubStatus (
   return EFI_SUCCESS;
 }
 
-
 /**
   Test whether the interface is a hub interface.
 
@@ -549,7 +427,7 @@ UsbHubAckHubStatus (
 **/
 BOOLEAN
 UsbIsHubInterface (
-  IN USB_INTERFACE        *UsbIf
+  IN USB_INTERFACE  *UsbIf
   )
 {
   EFI_USB_INTERFACE_DESCRIPTOR  *Setting;
@@ -561,15 +439,14 @@ UsbIsHubInterface (
   Setting = &UsbIf->IfSetting->Desc;
 
   if ((Setting->InterfaceClass == USB_HUB_CLASS_CODE) &&
-      (Setting->InterfaceSubClass == USB_HUB_SUBCLASS_CODE)) {
-
+      (Setting->InterfaceSubClass == USB_HUB_SUBCLASS_CODE))
+  {
     return TRUE;
   }
 
   return FALSE;
 }
 
-
 /**
   The callback function to the USB hub status change
   interrupt endpoint. It is called periodically by
@@ -587,20 +464,20 @@ UsbIsHubInterface (
 EFI_STATUS
 EFIAPI
 UsbOnHubInterrupt (
-  IN  VOID                *Data,
-  IN  UINTN               DataLength,
-  IN  VOID                *Context,
-  IN  UINT32              Result
+  IN  VOID    *Data,
+  IN  UINTN   DataLength,
+  IN  VOID    *Context,
+  IN  UINT32  Result
   )
 {
-  USB_INTERFACE               *HubIf;
-  EFI_USB_IO_PROTOCOL         *UsbIo;
-  EFI_USB_ENDPOINT_DESCRIPTOR *EpDesc;
-  EFI_STATUS                  Status;
+  USB_INTERFACE                *HubIf;
+  EFI_USB_IO_PROTOCOL          *UsbIo;
+  EFI_USB_ENDPOINT_DESCRIPTOR  *EpDesc;
+  EFI_STATUS                   Status;
 
-  HubIf   = (USB_INTERFACE *) Context;
-  UsbIo   = &(HubIf->UsbIo);
-  EpDesc  = &(HubIf->HubEp->Desc);
+  HubIf  = (USB_INTERFACE *)Context;
+  UsbIo  = &(HubIf->UsbIo);
+  EpDesc = &(HubIf->HubEp->Desc);
 
   if (Result != EFI_USB_NOERROR) {
     //
@@ -630,7 +507,7 @@ UsbOnHubInterrupt (
                       );
 
     if (EFI_ERROR (Status)) {
-      DEBUG (( EFI_D_ERROR, "UsbOnHubInterrupt: failed to remove async transfer - %r\n", Status));
+      DEBUG ((DEBUG_ERROR, "UsbOnHubInterrupt: failed to remove async transfer - %r\n", Status));
       return Status;
     }
 
@@ -645,7 +522,7 @@ UsbOnHubInterrupt (
                       );
 
     if (EFI_ERROR (Status)) {
-      DEBUG (( EFI_D_ERROR, "UsbOnHubInterrupt: failed to submit new async transfer - %r\n", Status));
+      DEBUG ((DEBUG_ERROR, "UsbOnHubInterrupt: failed to submit new async transfer - %r\n", Status));
     }
 
     return Status;
@@ -659,7 +536,7 @@ UsbOnHubInterrupt (
   // OK, actually something is changed, save the change map
   // then signal the HUB to do enumeration. This is a good
   // practise since UsbOnHubInterrupt is called in the context
-  // of host contrller's AsyncInterrupt monitor.
+  // of host controller's AsyncInterrupt monitor.
   //
   HubIf->ChangeMap = AllocateZeroPool (DataLength);
 
@@ -673,9 +550,6 @@ UsbOnHubInterrupt (
   return EFI_SUCCESS;
 }
 
-
-
-
 /**
   Initialize the device for a non-root hub.
 
@@ -687,10 +561,11 @@ UsbOnHubInterrupt (
 **/
 EFI_STATUS
 UsbHubInit (
-  IN USB_INTERFACE        *HubIf
+  IN USB_INTERFACE  *HubIf
   )
 {
-  EFI_USB_HUB_DESCRIPTOR  HubDesc;
+  UINT8                   HubDescBuffer[256];
+  EFI_USB_HUB_DESCRIPTOR  *HubDesc;
   USB_ENDPOINT_DESC       *EpDesc;
   USB_INTERFACE_SETTING   *Setting;
   EFI_USB_IO_PROTOCOL     *UsbIo;
@@ -703,11 +578,11 @@ UsbHubInit (
   //
   // Locate the interrupt endpoint for port change map
   //
-  HubIf->IsHub  = FALSE;
-  Setting       = HubIf->IfSetting;
-  HubDev        = HubIf->Device;
-  EpDesc        = NULL;
-  NumEndpoints  = Setting->Desc.NumEndpoints;
+  HubIf->IsHub = FALSE;
+  Setting      = HubIf->IfSetting;
+  HubDev       = HubIf->Device;
+  EpDesc       = NULL;
+  NumEndpoints = Setting->Desc.NumEndpoints;
 
   for (Index = 0; Index < NumEndpoints; Index++) {
     ASSERT ((Setting->Endpoints != NULL) && (Setting->Endpoints[Index] != NULL));
@@ -715,30 +590,36 @@ UsbHubInit (
     EpDesc = Setting->Endpoints[Index];
 
     if (USB_BIT_IS_SET (EpDesc->Desc.EndpointAddress, USB_ENDPOINT_DIR_IN) &&
-       (USB_ENDPOINT_TYPE (&EpDesc->Desc) == USB_ENDPOINT_INTERRUPT)) {
+        (USB_ENDPOINT_TYPE (&EpDesc->Desc) == USB_ENDPOINT_INTERRUPT))
+    {
       break;
     }
   }
 
-  if (Index == NumEndpoints) {
-    DEBUG (( EFI_D_ERROR, "UsbHubInit: no interrupt endpoint found for hub %d\n", HubDev->Address));
+  if ((Index == NumEndpoints) || (EpDesc == NULL)) {
+    DEBUG ((DEBUG_ERROR, "UsbHubInit: no interrupt endpoint found for hub %d\n", HubDev->Address));
     return EFI_DEVICE_ERROR;
   }
 
-  Status = UsbHubReadDesc (HubDev, &HubDesc);
+  //
+  // The length field of descriptor is UINT8 type, so the buffer
+  // with 256 bytes is enough to hold the descriptor data.
+  //
+  HubDesc = (EFI_USB_HUB_DESCRIPTOR *)HubDescBuffer;
+  Status  = UsbHubReadDesc (HubDev, HubDesc);
 
   if (EFI_ERROR (Status)) {
-    DEBUG (( EFI_D_ERROR, "UsbHubInit: failed to read HUB descriptor %r\n", Status));
+    DEBUG ((DEBUG_ERROR, "UsbHubInit: failed to read HUB descriptor - %r\n", Status));
     return Status;
   }
 
-  HubIf->NumOfPort = HubDesc.NumPorts;
+  HubIf->NumOfPort = HubDesc->NumPorts;
 
-  DEBUG (( EFI_D_INFO, "UsbHubInit: hub %d has %d ports\n", HubDev->Address,HubIf->NumOfPort));
+  DEBUG ((DEBUG_INFO, "UsbHubInit: hub %d has %d ports\n", HubDev->Address, HubIf->NumOfPort));
 
   //
   // OK, set IsHub to TRUE. Now usb bus can handle this device
-  // as a working HUB. If failed eariler, bus driver will not
+  // as a working HUB. If failed earlier, bus driver will not
   // recognize it as a hub. Other parts of the bus should be able
   // to work.
   //
@@ -748,27 +629,28 @@ UsbHubInit (
 
   if (HubIf->Device->Speed == EFI_USB_SPEED_SUPER) {
     Depth = (UINT16)(HubIf->Device->Tier - 1);
-    DEBUG ((EFI_D_INFO, "UsbHubInit: Set Hub Depth as 0x%x\n", Depth));
+    DEBUG ((DEBUG_INFO, "UsbHubInit: Set Hub Depth as 0x%x\n", Depth));
     UsbHubCtrlSetHubDepth (HubIf->Device, Depth);
-    
-    for (Index = 0; Index < HubDesc.NumPorts; Index++) {
+
+    for (Index = 0; Index < HubDesc->NumPorts; Index++) {
       UsbHubCtrlSetPortFeature (HubIf->Device, Index, USB_HUB_PORT_REMOTE_WAKE_MASK);
-    }    
+    }
   } else {
     //
     // Feed power to all the hub ports. It should be ok
     // for both gang/individual powered hubs.
     //
-    for (Index = 0; Index < HubDesc.NumPorts; Index++) {
-      UsbHubCtrlSetPortFeature (HubIf->Device, Index, (EFI_USB_PORT_FEATURE) USB_HUB_PORT_POWER);
+    for (Index = 0; Index < HubDesc->NumPorts; Index++) {
+      UsbHubCtrlSetPortFeature (HubIf->Device, Index, (EFI_USB_PORT_FEATURE)USB_HUB_PORT_POWER);
     }
 
     //
     // Update for the usb hub has no power on delay requirement
     //
-    if (HubDesc.PwrOn2PwrGood > 0) {
-      gBS->Stall (HubDesc.PwrOn2PwrGood * USB_SET_PORT_POWER_STALL);
+    if (HubDesc->PwrOn2PwrGood > 0) {
+      gBS->Stall (HubDesc->PwrOn2PwrGood * USB_SET_PORT_POWER_STALL);
     }
+
     UsbHubAckHubStatus (HubIf->Device);
   }
 
@@ -784,8 +666,12 @@ UsbHubInit (
                   );
 
   if (EFI_ERROR (Status)) {
-    DEBUG (( EFI_D_ERROR, "UsbHubInit: failed to create signal for hub %d - %r\n",
-                HubDev->Address, Status));
+    DEBUG ((
+      DEBUG_ERROR,
+      "UsbHubInit: failed to create signal for hub %d - %r\n",
+      HubDev->Address,
+      Status
+      ));
 
     return Status;
   }
@@ -809,8 +695,12 @@ UsbHubInit (
                     );
 
   if (EFI_ERROR (Status)) {
-    DEBUG (( EFI_D_ERROR, "UsbHubInit: failed to queue interrupt transfer for hub %d - %r\n",
-                HubDev->Address, Status));
+    DEBUG ((
+      DEBUG_ERROR,
+      "UsbHubInit: failed to queue interrupt transfer for hub %d - %r\n",
+      HubDev->Address,
+      Status
+      ));
 
     gBS->CloseEvent (HubIf->HubNotify);
     HubIf->HubNotify = NULL;
@@ -818,12 +708,10 @@ UsbHubInit (
     return Status;
   }
 
-  DEBUG (( EFI_D_INFO, "UsbHubInit: hub %d initialized\n", HubDev->Address));
+  DEBUG ((DEBUG_INFO, "UsbHubInit: hub %d initialized\n", HubDev->Address));
   return Status;
 }
 
-
-
 /**
   Get the port status. This function is required to
   ACK the port change bits although it will return
@@ -840,20 +728,18 @@ UsbHubInit (
 **/
 EFI_STATUS
 UsbHubGetPortStatus (
-  IN  USB_INTERFACE       *HubIf,
-  IN  UINT8               Port,
-  OUT EFI_USB_PORT_STATUS *PortState
+  IN  USB_INTERFACE        *HubIf,
+  IN  UINT8                Port,
+  OUT EFI_USB_PORT_STATUS  *PortState
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
-  Status  = UsbHubCtrlGetPortStatus (HubIf->Device, Port, PortState);
+  Status = UsbHubCtrlGetPortStatus (HubIf->Device, Port, PortState);
 
   return Status;
 }
 
-
-
 /**
   Clear the port change status.
 
@@ -863,8 +749,8 @@ UsbHubGetPortStatus (
 **/
 VOID
 UsbHubClearPortChange (
-  IN USB_INTERFACE        *HubIf,
-  IN UINT8                Port
+  IN USB_INTERFACE  *HubIf,
+  IN UINT8          Port
   )
 {
   EFI_USB_PORT_STATUS     PortState;
@@ -884,17 +770,15 @@ UsbHubClearPortChange (
   // It may lead to extra port state report. USB bus should
   // be able to handle this.
   //
-  for (Index = 0; Index < sizeof (mHubFeatureMap) / sizeof (mHubFeatureMap[0]); Index++) {
+  for (Index = 0; Index < ARRAY_SIZE (mHubFeatureMap); Index++) {
     Map = &mHubFeatureMap[Index];
 
     if (USB_BIT_IS_SET (PortState.PortChangeStatus, Map->ChangedBit)) {
-      UsbHubCtrlClearPortFeature (HubIf->Device, Port, (UINT16) Map->Feature);
+      UsbHubCtrlClearPortFeature (HubIf->Device, Port, (UINT16)Map->Feature);
     }
   }
 }
 
-
-
 /**
   Function to set the port feature for non-root hub.
 
@@ -908,19 +792,18 @@ UsbHubClearPortChange (
 **/
 EFI_STATUS
 UsbHubSetPortFeature (
-  IN USB_INTERFACE        *HubIf,
-  IN UINT8                Port,
-  IN EFI_USB_PORT_FEATURE Feature
+  IN USB_INTERFACE         *HubIf,
+  IN UINT8                 Port,
+  IN EFI_USB_PORT_FEATURE  Feature
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
-  Status = UsbHubCtrlSetPortFeature (HubIf->Device, Port, (UINT8) Feature);
+  Status = UsbHubCtrlSetPortFeature (HubIf->Device, Port, (UINT8)Feature);
 
   return Status;
 }
 
-
 /**
   Interface function to clear the port feature for non-root hub.
 
@@ -934,19 +817,18 @@ UsbHubSetPortFeature (
 **/
 EFI_STATUS
 UsbHubClearPortFeature (
-  IN USB_INTERFACE        *HubIf,
-  IN UINT8                Port,
-  IN EFI_USB_PORT_FEATURE Feature
+  IN USB_INTERFACE         *HubIf,
+  IN UINT8                 Port,
+  IN EFI_USB_PORT_FEATURE  Feature
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
-  Status = UsbHubCtrlClearPortFeature (HubIf->Device, Port, (UINT8) Feature);
+  Status = UsbHubCtrlClearPortFeature (HubIf->Device, Port, (UINT8)Feature);
 
   return Status;
 }
 
-
 /**
   Interface function to reset the port.
 
@@ -960,15 +842,15 @@ UsbHubClearPortFeature (
 **/
 EFI_STATUS
 UsbHubResetPort (
-  IN USB_INTERFACE        *HubIf,
-  IN UINT8                Port
+  IN USB_INTERFACE  *HubIf,
+  IN UINT8          Port
   )
 {
-  EFI_USB_PORT_STATUS     PortState;
-  UINTN                   Index;
-  EFI_STATUS              Status;
+  EFI_USB_PORT_STATUS  PortState;
+  UINTN                Index;
+  EFI_STATUS           Status;
 
-  Status  = UsbHubSetPortFeature (HubIf, Port, (EFI_USB_PORT_FEATURE) USB_HUB_PORT_RESET);
+  Status = UsbHubSetPortFeature (HubIf, Port, (EFI_USB_PORT_FEATURE)USB_HUB_PORT_RESET);
 
   if (EFI_ERROR (Status)) {
     return Status;
@@ -993,7 +875,8 @@ UsbHubResetPort (
     }
 
     if (!EFI_ERROR (Status) &&
-        USB_BIT_IS_SET (PortState.PortChangeStatus, USB_PORT_STAT_C_RESET)) {
+        USB_BIT_IS_SET (PortState.PortChangeStatus, USB_PORT_STAT_C_RESET))
+    {
       gBS->Stall (USB_SET_PORT_RECOVERY_STALL);
       return EFI_SUCCESS;
     }
@@ -1004,7 +887,6 @@ UsbHubResetPort (
   return EFI_TIMEOUT;
 }
 
-
 /**
   Release the hub's control of the interface.
 
@@ -1015,11 +897,11 @@ UsbHubResetPort (
 **/
 EFI_STATUS
 UsbHubRelease (
-  IN USB_INTERFACE        *HubIf
+  IN USB_INTERFACE  *HubIf
   )
 {
-  EFI_USB_IO_PROTOCOL     *UsbIo;
-  EFI_STATUS              Status;
+  EFI_USB_IO_PROTOCOL  *UsbIo;
+  EFI_STATUS           Status;
 
   UsbIo  = &HubIf->UsbIo;
   Status = UsbIo->UsbAsyncInterruptTransfer (
@@ -1038,17 +920,15 @@ UsbHubRelease (
 
   gBS->CloseEvent (HubIf->HubNotify);
 
-  HubIf->IsHub      = FALSE;
-  HubIf->HubApi     = NULL;
-  HubIf->HubEp      = NULL;
-  HubIf->HubNotify  = NULL;
+  HubIf->IsHub     = FALSE;
+  HubIf->HubApi    = NULL;
+  HubIf->HubEp     = NULL;
+  HubIf->HubNotify = NULL;
 
-  DEBUG (( EFI_D_INFO, "UsbHubRelease: hub device %d released\n", HubIf->Device->Address));
+  DEBUG ((DEBUG_INFO, "UsbHubRelease: hub device %d released\n", HubIf->Device->Address));
   return EFI_SUCCESS;
 }
 
-
-
 /**
   Initialize the interface for root hub.
 
@@ -1060,13 +940,13 @@ UsbHubRelease (
 **/
 EFI_STATUS
 UsbRootHubInit (
-  IN USB_INTERFACE        *HubIf
+  IN USB_INTERFACE  *HubIf
   )
 {
-  EFI_STATUS              Status;
-  UINT8                   MaxSpeed;
-  UINT8                   NumOfPort;
-  UINT8                   Support64;
+  EFI_STATUS  Status;
+  UINT8       MaxSpeed;
+  UINT8       NumOfPort;
+  UINT8       Support64;
 
   Status = UsbHcGetCapability (HubIf->Device->Bus, &MaxSpeed, &NumOfPort, &Support64);
 
@@ -1074,15 +954,20 @@ UsbRootHubInit (
     return Status;
   }
 
-  DEBUG (( EFI_D_INFO, "UsbRootHubInit: root hub %p - max speed %d, %d ports\n",
-              HubIf, MaxSpeed, NumOfPort));
+  DEBUG ((
+    DEBUG_INFO,
+    "UsbRootHubInit: root hub %p - max speed %d, %d ports\n",
+    HubIf,
+    MaxSpeed,
+    NumOfPort
+    ));
 
-  HubIf->IsHub      = TRUE;
-  HubIf->HubApi     = &mUsbRootHubApi;
-  HubIf->HubEp      = NULL;
-  HubIf->MaxSpeed   = MaxSpeed;
-  HubIf->NumOfPort  = NumOfPort;
-  HubIf->HubNotify  = NULL;
+  HubIf->IsHub     = TRUE;
+  HubIf->HubApi    = &mUsbRootHubApi;
+  HubIf->HubEp     = NULL;
+  HubIf->MaxSpeed  = MaxSpeed;
+  HubIf->NumOfPort = NumOfPort;
+  HubIf->HubNotify = NULL;
 
   //
   // Create a timer to poll root hub ports periodically
@@ -1118,7 +1003,6 @@ UsbRootHubInit (
   return Status;
 }
 
-
 /**
   Get the port status. This function is required to
   ACK the port change bits although it will return
@@ -1135,21 +1019,20 @@ UsbRootHubInit (
 **/
 EFI_STATUS
 UsbRootHubGetPortStatus (
-  IN  USB_INTERFACE       *HubIf,
-  IN  UINT8               Port,
-  OUT EFI_USB_PORT_STATUS *PortState
+  IN  USB_INTERFACE        *HubIf,
+  IN  UINT8                Port,
+  OUT EFI_USB_PORT_STATUS  *PortState
   )
 {
-  USB_BUS                 *Bus;
-  EFI_STATUS              Status;
+  USB_BUS     *Bus;
+  EFI_STATUS  Status;
 
-  Bus     = HubIf->Device->Bus;
-  Status  = UsbHcGetRootHubPortStatus (Bus, Port, PortState);
+  Bus    = HubIf->Device->Bus;
+  Status = UsbHcGetRootHubPortStatus (Bus, Port, PortState);
 
   return Status;
 }
 
-
 /**
   Clear the port change status.
 
@@ -1159,8 +1042,8 @@ UsbRootHubGetPortStatus (
 **/
 VOID
 UsbRootHubClearPortChange (
-  IN USB_INTERFACE        *HubIf,
-  IN UINT8                Port
+  IN USB_INTERFACE  *HubIf,
+  IN UINT8          Port
   )
 {
   EFI_USB_PORT_STATUS     PortState;
@@ -1180,16 +1063,15 @@ UsbRootHubClearPortChange (
   // It may lead to extra port state report. USB bus should
   // be able to handle this.
   //
-  for (Index = 0; Index < sizeof (mRootHubFeatureMap) / sizeof (mRootHubFeatureMap[0]); Index++) {
+  for (Index = 0; Index < ARRAY_SIZE (mRootHubFeatureMap); Index++) {
     Map = &mRootHubFeatureMap[Index];
 
     if (USB_BIT_IS_SET (PortState.PortChangeStatus, Map->ChangedBit)) {
-      UsbHcClearRootHubPortFeature (HubIf->Device->Bus, Port, (EFI_USB_PORT_FEATURE) Map->Feature);
+      UsbHcClearRootHubPortFeature (HubIf->Device->Bus, Port, (EFI_USB_PORT_FEATURE)Map->Feature);
     }
   }
 }
 
-
 /**
   Set the root hub port feature.
 
@@ -1203,19 +1085,18 @@ UsbRootHubClearPortChange (
 **/
 EFI_STATUS
 UsbRootHubSetPortFeature (
-  IN USB_INTERFACE        *HubIf,
-  IN UINT8                Port,
-  IN EFI_USB_PORT_FEATURE Feature
+  IN USB_INTERFACE         *HubIf,
+  IN UINT8                 Port,
+  IN EFI_USB_PORT_FEATURE  Feature
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
-  Status  = UsbHcSetRootHubPortFeature (HubIf->Device->Bus, Port, Feature);
+  Status = UsbHcSetRootHubPortFeature (HubIf->Device->Bus, Port, Feature);
 
   return Status;
 }
 
-
 /**
   Clear the root hub port feature.
 
@@ -1229,19 +1110,18 @@ UsbRootHubSetPortFeature (
 **/
 EFI_STATUS
 UsbRootHubClearPortFeature (
-  IN USB_INTERFACE        *HubIf,
-  IN UINT8                Port,
-  IN EFI_USB_PORT_FEATURE Feature
+  IN USB_INTERFACE         *HubIf,
+  IN UINT8                 Port,
+  IN EFI_USB_PORT_FEATURE  Feature
   )
 {
-  EFI_STATUS              Status;
+  EFI_STATUS  Status;
 
-  Status  = UsbHcClearRootHubPortFeature (HubIf->Device->Bus, Port, Feature);
+  Status = UsbHcClearRootHubPortFeature (HubIf->Device->Bus, Port, Feature);
 
   return Status;
 }
 
-
 /**
   Interface function to reset the root hub port.
 
@@ -1257,25 +1137,26 @@ UsbRootHubClearPortFeature (
 **/
 EFI_STATUS
 UsbRootHubResetPort (
-  IN USB_INTERFACE        *RootIf,
-  IN UINT8                Port
+  IN USB_INTERFACE  *RootIf,
+  IN UINT8          Port
   )
 {
-  USB_BUS                 *Bus;
-  EFI_STATUS              Status;
-  EFI_USB_PORT_STATUS     PortState;
-  UINTN                   Index;
+  USB_BUS              *Bus;
+  EFI_STATUS           Status;
+  EFI_USB_PORT_STATUS  PortState;
+  UINTN                Index;
 
   //
   // Notice: although EHCI requires that ENABLED bit be cleared
   // when reset the port, we don't need to care that here. It
   // should be handled in the EHCI driver.
   //
-  Bus     = RootIf->Device->Bus;
-  Status  = UsbHcSetRootHubPortFeature (Bus, Port, EfiUsbPortReset);
+  Bus = RootIf->Device->Bus;
+
+  Status = UsbHcSetRootHubPortFeature (Bus, Port, EfiUsbPortReset);
 
   if (EFI_ERROR (Status)) {
-    DEBUG (( EFI_D_ERROR, "UsbRootHubResetPort: failed to start reset on port %d\n", Port));
+    DEBUG ((DEBUG_ERROR, "UsbRootHubResetPort: failed to start reset on port %d\n", Port));
     return Status;
   }
 
@@ -1288,7 +1169,7 @@ UsbRootHubResetPort (
   Status = UsbHcClearRootHubPortFeature (Bus, Port, EfiUsbPortReset);
 
   if (EFI_ERROR (Status)) {
-    DEBUG (( EFI_D_ERROR, "UsbRootHubResetPort: failed to clear reset on port %d\n", Port));
+    DEBUG ((DEBUG_ERROR, "UsbRootHubResetPort: failed to clear reset on port %d\n", Port));
     return Status;
   }
 
@@ -1315,7 +1196,7 @@ UsbRootHubResetPort (
   }
 
   if (Index == USB_WAIT_PORT_STS_CHANGE_LOOP) {
-    DEBUG ((EFI_D_ERROR, "UsbRootHubResetPort: reset not finished in time on port %d\n", Port));
+    DEBUG ((DEBUG_ERROR, "UsbRootHubResetPort: reset not finished in time on port %d\n", Port));
     return EFI_TIMEOUT;
   }
 
@@ -1327,17 +1208,15 @@ UsbRootHubResetPort (
     // automatically enable the port, we need to enable it manually.
     //
     if (RootIf->MaxSpeed == EFI_USB_SPEED_HIGH) {
-      DEBUG (( EFI_D_ERROR, "UsbRootHubResetPort: release low/full speed device (%d) to UHCI\n", Port));
+      DEBUG ((DEBUG_ERROR, "UsbRootHubResetPort: release low/full speed device (%d) to UHCI\n", Port));
 
       UsbRootHubSetPortFeature (RootIf, Port, EfiUsbPortOwner);
       return EFI_NOT_FOUND;
-
     } else {
-
       Status = UsbRootHubSetPortFeature (RootIf, Port, EfiUsbPortEnable);
 
       if (EFI_ERROR (Status)) {
-        DEBUG (( EFI_D_ERROR, "UsbRootHubResetPort: failed to enable port %d for UHCI\n", Port));
+        DEBUG ((DEBUG_ERROR, "UsbRootHubResetPort: failed to enable port %d for UHCI\n", Port));
         return Status;
       }
 
@@ -1348,7 +1227,6 @@ UsbRootHubResetPort (
   return EFI_SUCCESS;
 }
 
-
 /**
   Release the root hub's control of the interface.
 
@@ -1360,10 +1238,10 @@ UsbRootHubResetPort (
 **/
 EFI_STATUS
 UsbRootHubRelease (
-  IN USB_INTERFACE        *HubIf
+  IN USB_INTERFACE  *HubIf
   )
 {
-  DEBUG (( EFI_D_INFO, "UsbRootHubRelease: root hub released for hub %p\n", HubIf));
+  DEBUG ((DEBUG_INFO, "UsbRootHubRelease: root hub released for hub %p\n", HubIf));
 
   gBS->SetTimer (HubIf->HubNotify, TimerCancel, USB_ROOTHUB_POLL_INTERVAL);
   gBS->CloseEvent (HubIf->HubNotify);
@@ -1371,7 +1249,7 @@ UsbRootHubRelease (
   return EFI_SUCCESS;
 }
 
-USB_HUB_API mUsbHubApi = {
+USB_HUB_API  mUsbHubApi = {
   UsbHubInit,
   UsbHubGetPortStatus,
   UsbHubClearPortChange,
@@ -1381,7 +1259,7 @@ USB_HUB_API mUsbHubApi = {
   UsbHubRelease
 };
 
-USB_HUB_API mUsbRootHubApi = {
+USB_HUB_API  mUsbRootHubApi = {
   UsbRootHubInit,
   UsbRootHubGetPortStatus,
   UsbRootHubClearPortChange,
