@@ -2,14 +2,8 @@
   Implementation of the USB mass storage Bulk-Only Transport protocol,
   according to USB Mass Storage Class Bulk-Only Transport, Revision 1.0.
 
-Copyright (c) 2007 - 2011, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+Copyright (c) 2007 - 2018, Intel Corporation. All rights reserved.<BR>
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -18,7 +12,7 @@ WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
 //
 // Definition of USB BOT Transport Protocol
 //
-USB_MASS_TRANSPORT mUsbBotTransport = {
+USB_MASS_TRANSPORT  mUsbBotTransport = {
   USB_MASS_STORE_BOT,
   UsbBotInit,
   UsbBotExecCommand,
@@ -44,8 +38,8 @@ USB_MASS_TRANSPORT mUsbBotTransport = {
 **/
 EFI_STATUS
 UsbBotInit (
-  IN  EFI_USB_IO_PROTOCOL       *UsbIo,
-  OUT VOID                      **Context OPTIONAL
+  IN  EFI_USB_IO_PROTOCOL  *UsbIo,
+  OUT VOID                 **Context OPTIONAL
   )
 {
   USB_BOT_PROTOCOL              *UsbBot;
@@ -90,17 +84,17 @@ UsbBotInit (
     }
 
     if (USB_IS_IN_ENDPOINT (EndPoint.EndpointAddress) &&
-       (UsbBot->BulkInEndpoint == NULL)) {
-
-      UsbBot->BulkInEndpoint  = (EFI_USB_ENDPOINT_DESCRIPTOR *) (UsbBot + 1);
-      CopyMem(UsbBot->BulkInEndpoint, &EndPoint, sizeof (EndPoint));
+        (UsbBot->BulkInEndpoint == NULL))
+    {
+      UsbBot->BulkInEndpoint = (EFI_USB_ENDPOINT_DESCRIPTOR *)(UsbBot + 1);
+      CopyMem (UsbBot->BulkInEndpoint, &EndPoint, sizeof (EndPoint));
     }
 
     if (USB_IS_OUT_ENDPOINT (EndPoint.EndpointAddress) &&
-       (UsbBot->BulkOutEndpoint == NULL)) {
-
-      UsbBot->BulkOutEndpoint   = (EFI_USB_ENDPOINT_DESCRIPTOR *) (UsbBot + 1) + 1;
-      CopyMem (UsbBot->BulkOutEndpoint, &EndPoint, sizeof(EndPoint));
+        (UsbBot->BulkOutEndpoint == NULL))
+    {
+      UsbBot->BulkOutEndpoint = (EFI_USB_ENDPOINT_DESCRIPTOR *)(UsbBot + 1) + 1;
+      CopyMem (UsbBot->BulkOutEndpoint, &EndPoint, sizeof (EndPoint));
     }
   }
 
@@ -151,19 +145,19 @@ ON_ERROR:
 **/
 EFI_STATUS
 UsbBotSendCommand (
-  IN USB_BOT_PROTOCOL         *UsbBot,
-  IN UINT8                    *Cmd,
-  IN UINT8                    CmdLen,
-  IN EFI_USB_DATA_DIRECTION   DataDir,
-  IN UINT32                   TransLen,
-  IN UINT8                    Lun
+  IN USB_BOT_PROTOCOL        *UsbBot,
+  IN UINT8                   *Cmd,
+  IN UINT8                   CmdLen,
+  IN EFI_USB_DATA_DIRECTION  DataDir,
+  IN UINT32                  TransLen,
+  IN UINT8                   Lun
   )
 {
-  USB_BOT_CBW               Cbw;
-  EFI_STATUS                Status;
-  UINT32                    Result;
-  UINTN                     DataLen;
-  UINTN                     Timeout;
+  USB_BOT_CBW  Cbw;
+  EFI_STATUS   Status;
+  UINT32       Result;
+  UINTN        DataLen;
+  UINTN        Timeout;
 
   ASSERT ((CmdLen > 0) && (CmdLen <= USB_BOT_MAX_CMDLEN));
 
@@ -173,7 +167,7 @@ UsbBotSendCommand (
   Cbw.Signature = USB_BOT_CBW_SIGNATURE;
   Cbw.Tag       = UsbBot->CbwTag;
   Cbw.DataLen   = TransLen;
-  Cbw.Flag      = (UINT8) ((DataDir == EfiUsbDataIn) ? BIT7 : 0);
+  Cbw.Flag      = (UINT8)((DataDir == EfiUsbDataIn) ? BIT7 : 0);
   Cbw.Lun       = Lun;
   Cbw.CmdLen    = CmdLen;
 
@@ -196,7 +190,7 @@ UsbBotSendCommand (
                             &Result
                             );
   if (EFI_ERROR (Status)) {
-    if (USB_IS_ERROR (Result, EFI_USB_ERR_STALL) && DataDir == EfiUsbDataOut) {
+    if (USB_IS_ERROR (Result, EFI_USB_ERR_STALL) && (DataDir == EfiUsbDataOut)) {
       //
       // Respond to Bulk-Out endpoint stall with a Reset Recovery,
       // according to section 5.3.1 of USB Mass Storage Class Bulk-Only Transport Spec, v1.0.
@@ -210,7 +204,6 @@ UsbBotSendCommand (
   return Status;
 }
 
-
 /**
   Transfer the data between the device and host.
 
@@ -232,16 +225,16 @@ UsbBotSendCommand (
 **/
 EFI_STATUS
 UsbBotDataTransfer (
-  IN USB_BOT_PROTOCOL         *UsbBot,
-  IN EFI_USB_DATA_DIRECTION   DataDir,
-  IN OUT UINT8                *Data,
-  IN OUT UINTN                *TransLen,
-  IN UINT32                   Timeout
+  IN USB_BOT_PROTOCOL        *UsbBot,
+  IN EFI_USB_DATA_DIRECTION  DataDir,
+  IN OUT UINT8               *Data,
+  IN OUT UINTN               *TransLen,
+  IN UINT32                  Timeout
   )
 {
-  EFI_USB_ENDPOINT_DESCRIPTOR *Endpoint;
-  EFI_STATUS                  Status;
-  UINT32                      Result;
+  EFI_USB_ENDPOINT_DESCRIPTOR  *Endpoint;
+  EFI_STATUS                   Status;
+  UINT32                       Result;
 
   //
   // If no data to transfer, just return EFI_SUCCESS.
@@ -272,23 +265,23 @@ UsbBotDataTransfer (
                             );
   if (EFI_ERROR (Status)) {
     if (USB_IS_ERROR (Result, EFI_USB_ERR_STALL)) {
-      DEBUG ((EFI_D_INFO, "UsbBotDataTransfer: (%r)\n", Status));      
-      DEBUG ((EFI_D_INFO, "UsbBotDataTransfer: DataIn Stall\n"));
+      DEBUG ((DEBUG_INFO, "UsbBotDataTransfer: (%r)\n", Status));
+      DEBUG ((DEBUG_INFO, "UsbBotDataTransfer: DataIn Stall\n"));
       UsbClearEndpointStall (UsbBot->UsbIo, Endpoint->EndpointAddress);
     } else if (USB_IS_ERROR (Result, EFI_USB_ERR_NAK)) {
       Status = EFI_NOT_READY;
     } else {
-      DEBUG ((EFI_D_ERROR, "UsbBotDataTransfer: (%r)\n", Status));
+      DEBUG ((DEBUG_ERROR, "UsbBotDataTransfer: (%r)\n", Status));
     }
-    if(Status == EFI_TIMEOUT){
-      UsbBotResetDevice(UsbBot, FALSE);
+
+    if (Status == EFI_TIMEOUT) {
+      UsbBotResetDevice (UsbBot, FALSE);
     }
   }
 
   return Status;
 }
 
-
 /**
   Get the command execution status from device.
 
@@ -310,19 +303,19 @@ UsbBotDataTransfer (
 **/
 EFI_STATUS
 UsbBotGetStatus (
-  IN  USB_BOT_PROTOCOL      *UsbBot,
-  IN  UINT32                TransLen,
-  OUT UINT8                 *CmdStatus
+  IN  USB_BOT_PROTOCOL  *UsbBot,
+  IN  UINT32            TransLen,
+  OUT UINT8             *CmdStatus
   )
 {
-  USB_BOT_CSW               Csw;
-  UINTN                     Len;
-  UINT8                     Endpoint;
-  EFI_STATUS                Status;
-  UINT32                    Result;
-  EFI_USB_IO_PROTOCOL       *UsbIo;
-  UINT32                    Index;
-  UINTN                     Timeout;
+  USB_BOT_CSW          Csw;
+  UINTN                Len;
+  UINT8                Endpoint;
+  EFI_STATUS           Status;
+  UINT32               Result;
+  EFI_USB_IO_PROTOCOL  *UsbIo;
+  UINT32               Index;
+  UINTN                Timeout;
 
   *CmdStatus = USB_BOT_COMMAND_ERROR;
   Status     = EFI_DEVICE_ERROR;
@@ -332,7 +325,7 @@ UsbBotGetStatus (
 
   for (Index = 0; Index < USB_BOT_RECV_CSW_RETRY; Index++) {
     //
-    // Attemp to the read Command Status Wrapper from bulk in endpoint
+    // Attempt to the read Command Status Wrapper from bulk in endpoint
     //
     ZeroMem (&Csw, sizeof (USB_BOT_CSW));
     Result = 0;
@@ -345,10 +338,11 @@ UsbBotGetStatus (
                       Timeout,
                       &Result
                       );
-    if (EFI_ERROR(Status)) {
+    if (EFI_ERROR (Status)) {
       if (USB_IS_ERROR (Result, EFI_USB_ERR_STALL)) {
         UsbClearEndpointStall (UsbIo, Endpoint);
       }
+
       continue;
     }
 
@@ -367,15 +361,15 @@ UsbBotGetStatus (
       break;
     }
   }
+
   //
-  //The tag is increased even if there is an error.
+  // The tag is increased even if there is an error.
   //
   UsbBot->CbwTag++;
 
   return Status;
 }
 
-
 /**
   Call the USB Mass Storage Class BOT protocol to issue
   the command/data/status circle to execute the commands.
@@ -392,7 +386,7 @@ UsbBotGetStatus (
   @param  CmdStatus             The result of high level command execution
 
   @retval EFI_SUCCESS           The command is executed successfully.
-  @retval Other                 Failed to excute command
+  @retval Other                 Failed to execute command
 
 **/
 EFI_STATUS
@@ -408,13 +402,13 @@ UsbBotExecCommand (
   OUT UINT32                  *CmdStatus
   )
 {
-  USB_BOT_PROTOCOL          *UsbBot;
-  EFI_STATUS                Status;
-  UINTN                     TransLen;
-  UINT8                     Result;
+  USB_BOT_PROTOCOL  *UsbBot;
+  EFI_STATUS        Status;
+  UINTN             TransLen;
+  UINT8             Result;
 
-  *CmdStatus  = USB_MASS_CMD_FAIL;
-  UsbBot      = (USB_BOT_PROTOCOL *) Context;
+  *CmdStatus = USB_MASS_CMD_FAIL;
+  UsbBot     = (USB_BOT_PROTOCOL *)Context;
 
   //
   // Send the command to the device. Return immediately if device
@@ -422,7 +416,7 @@ UsbBotExecCommand (
   //
   Status = UsbBotSendCommand (UsbBot, Cmd, CmdLen, DataDir, DataLen, Lun);
   if (EFI_ERROR (Status)) {
-    DEBUG ((EFI_D_ERROR, "UsbBotExecCommand: UsbBotSendCommand (%r)\n", Status));
+    DEBUG ((DEBUG_ERROR, "UsbBotExecCommand: UsbBotSendCommand (%r)\n", Status));
     return Status;
   }
 
@@ -431,7 +425,7 @@ UsbBotExecCommand (
   // failed. The host should attempt to receive the CSW no matter
   // whether it succeeds or fails.
   //
-  TransLen = (UINTN) DataLen;
+  TransLen = (UINTN)DataLen;
   UsbBotDataTransfer (UsbBot, DataDir, Data, &TransLen, Timeout);
 
   //
@@ -439,7 +433,7 @@ UsbBotExecCommand (
   //
   Status = UsbBotGetStatus (UsbBot, DataLen, &Result);
   if (EFI_ERROR (Status)) {
-    DEBUG ((EFI_D_ERROR, "UsbBotExecCommand: UsbBotGetStatus (%r)\n", Status));
+    DEBUG ((DEBUG_ERROR, "UsbBotExecCommand: UsbBotGetStatus (%r)\n", Status));
     return Status;
   }
 
@@ -450,14 +444,12 @@ UsbBotExecCommand (
   return EFI_SUCCESS;
 }
 
-
 /**
   Reset the USB mass storage device by BOT protocol.
 
   @param  Context               The context of the BOT protocol, that is,
                                 USB_BOT_PROTOCOL.
-  @param  ExtendedVerification  If FALSE, just issue Bulk-Only Mass Storage Reset request.
-                                If TRUE, additionally reset parent hub port.
+  @param  ExtendedVerification  ExtendedVerification is ignored in this implementation.
 
   @retval EFI_SUCCESS           The device is reset.
   @retval Others                Failed to reset the device..
@@ -465,8 +457,8 @@ UsbBotExecCommand (
 **/
 EFI_STATUS
 UsbBotResetDevice (
-  IN  VOID                    *Context,
-  IN  BOOLEAN                 ExtendedVerification
+  IN  VOID     *Context,
+  IN  BOOLEAN  ExtendedVerification
   )
 {
   USB_BOT_PROTOCOL        *UsbBot;
@@ -475,17 +467,7 @@ UsbBotResetDevice (
   UINT32                  Result;
   UINT32                  Timeout;
 
-  UsbBot = (USB_BOT_PROTOCOL *) Context;
-
-  if (ExtendedVerification) {
-    //
-    // If we need to do strictly reset, reset its parent hub port
-    //
-    Status = UsbBot->UsbIo->UsbPortReset (UsbBot->UsbIo);
-    if (EFI_ERROR (Status)) {
-      return EFI_DEVICE_ERROR;
-    }
-  }
+  UsbBot = (USB_BOT_PROTOCOL *)Context;
 
   //
   // Issue a class specific Bulk-Only Mass Storage Reset request,
@@ -528,7 +510,6 @@ UsbBotResetDevice (
   return Status;
 }
 
-
 /**
   Get the max LUN (Logical Unit Number) of USB mass storage device.
 
@@ -542,8 +523,8 @@ UsbBotResetDevice (
 **/
 EFI_STATUS
 UsbBotGetMaxLun (
-  IN  VOID                    *Context,
-  OUT UINT8                   *MaxLun
+  IN  VOID   *Context,
+  OUT UINT8  *MaxLun
   )
 {
   USB_BOT_PROTOCOL        *UsbBot;
@@ -552,12 +533,14 @@ UsbBotGetMaxLun (
   UINT32                  Result;
   UINT32                  Timeout;
 
-  ASSERT (Context);
-  
-  UsbBot = (USB_BOT_PROTOCOL *) Context;
+  if ((Context == NULL) || (MaxLun == NULL)) {
+    return EFI_INVALID_PARAMETER;
+  }
+
+  UsbBot = (USB_BOT_PROTOCOL *)Context;
 
   //
-  // Issue a class specific Bulk-Only Mass Storage get max lun reqest.
+  // Issue a class specific Bulk-Only Mass Storage get max lun request.
   // according to section 3.2 of USB Mass Storage Class Bulk-Only Transport Spec, v1.0.
   //
   Request.RequestType = 0xA1;
@@ -572,12 +555,24 @@ UsbBotGetMaxLun (
                             &Request,
                             EfiUsbDataIn,
                             Timeout,
-                            (VOID *) MaxLun,
+                            (VOID *)MaxLun,
                             1,
                             &Result
                             );
+  if (EFI_ERROR (Status) || (*MaxLun > USB_BOT_MAX_LUN)) {
+    //
+    // If the Get LUN request returns an error or the MaxLun is larger than
+    // the maximum LUN value (0x0f) supported by the USB Mass Storage Class
+    // Bulk-Only Transport Spec, then set MaxLun to 0.
+    //
+    // This improves compatibility with USB FLASH drives that have a single LUN
+    // and either do not return a max LUN value or return an invalid maximum LUN
+    // value.
+    //
+    *MaxLun = 0;
+  }
 
-  return Status;
+  return EFI_SUCCESS;
 }
 
 /**
@@ -590,10 +585,9 @@ UsbBotGetMaxLun (
 **/
 EFI_STATUS
 UsbBotCleanUp (
-  IN  VOID                    *Context
+  IN  VOID  *Context
   )
 {
   FreePool (Context);
   return EFI_SUCCESS;
 }
-
