@@ -44,8 +44,8 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 /// HII specific Vendor Device Path definition.
 ///
 typedef struct {
-  VENDOR_DEVICE_PATH             VendorDevicePath;
-  EFI_DEVICE_PATH_PROTOCOL       End;
+  VENDOR_DEVICE_PATH          VendorDevicePath;
+  EFI_DEVICE_PATH_PROTOCOL    End;
 } HII_VENDOR_DEVICE_PATH;
 #pragma pack()
 
@@ -53,19 +53,19 @@ typedef struct {
 // Constants which are variable names used to access variables
 //
 
-#define VAR_CON_OUT_MODE L"ConOutMode"
+#define VAR_CON_OUT_MODE  L"ConOutMode"
 
 //
 // Variable created with this flag will be "Efi:...."
 //
 #define VAR_FLAG  EFI_VARIABLE_BOOTSERVICE_ACCESS | EFI_VARIABLE_RUNTIME_ACCESS | EFI_VARIABLE_NON_VOLATILE
 
-extern EFI_GUID mBootMaintGuid;
-extern CHAR16   mBootMaintStorageName[];
+extern EFI_GUID  mBootMaintGuid;
+extern CHAR16    mBootMaintStorageName[];
 //
 // These are the VFR compiler generated data representing our VFR data.
 //
-extern UINT8    BootMaintenanceManagerBin[];
+extern UINT8  BootMaintenanceManagerBin[];
 
 //
 // Below are the number of options in Baudrate, Databits,
@@ -79,7 +79,7 @@ extern UINT8    BootMaintenanceManagerBin[];
 //
 // Callback function helper
 //
-#define BMM_CALLBACK_DATA_SIGNATURE     SIGNATURE_32 ('C', 'b', 'c', 'k')
+#define BMM_CALLBACK_DATA_SIGNATURE  SIGNATURE_32 ('C', 'b', 'c', 'k')
 #define BMM_CALLBACK_DATA_FROM_THIS(a)  CR (a, BMM_CALLBACK_DATA, BmmConfigAccess, BMM_CALLBACK_DATA_SIGNATURE)
 
 //
@@ -88,7 +88,7 @@ extern UINT8    BootMaintenanceManagerBin[];
 typedef UINT8 BBS_TYPE;
 
 typedef enum _TYPE_OF_TERMINAL {
-  TerminalTypePcAnsi                = 0,
+  TerminalTypePcAnsi = 0,
   TerminalTypeVt100,
   TerminalTypeVt100Plus,
   TerminalTypeVtUtf8,
@@ -110,20 +110,20 @@ typedef enum _TYPE_OF_TERMINAL {
 #define BM_TERMINAL_OPTION_SIGNATURE  SIGNATURE_32 ('t', 'r', 'm', 'l')
 #define BM_MENU_ENTRY_SIGNATURE       SIGNATURE_32 ('e', 'n', 't', 'r')
 
-#define BM_LOAD_CONTEXT_SELECT        0x0
-#define BM_CONSOLE_CONTEXT_SELECT     0x1
-#define BM_FILE_CONTEXT_SELECT        0x2
-#define BM_HANDLE_CONTEXT_SELECT      0x3
-#define BM_TERMINAL_CONTEXT_SELECT    0x5
+#define BM_LOAD_CONTEXT_SELECT      0x0
+#define BM_CONSOLE_CONTEXT_SELECT   0x1
+#define BM_FILE_CONTEXT_SELECT      0x2
+#define BM_HANDLE_CONTEXT_SELECT    0x3
+#define BM_TERMINAL_CONTEXT_SELECT  0x5
 
-#define BM_CONSOLE_IN_CONTEXT_SELECT  0x6
-#define BM_CONSOLE_OUT_CONTEXT_SELECT 0x7
-#define BM_CONSOLE_ERR_CONTEXT_SELECT 0x8
+#define BM_CONSOLE_IN_CONTEXT_SELECT   0x6
+#define BM_CONSOLE_OUT_CONTEXT_SELECT  0x7
+#define BM_CONSOLE_ERR_CONTEXT_SELECT  0x8
 
 //
 // Buffer size for update data
 //
-#define UPDATE_DATA_SIZE        0x100000
+#define UPDATE_DATA_SIZE  0x100000
 
 //
 // Namespace of callback keys used in display and file system navigation
@@ -148,12 +148,12 @@ typedef enum _TYPE_OF_TERMINAL {
 // all these values are computed from the structure
 // defined below
 //
-#define VAR_OFFSET(Field)              ((UINT16) ((UINTN) &(((BMM_FAKE_NV_DATA *) 0)->Field)))
+#define VAR_OFFSET(Field)  ((UINT16) ((UINTN) &(((BMM_FAKE_NV_DATA *) 0)->Field)))
 
 //
 // Question Id of Zero is invalid, so add an offset to it
 //
-#define QUESTION_ID(Field)             (VAR_OFFSET (Field) + CONFIG_OPTION_OFFSET)
+#define QUESTION_ID(Field)  (VAR_OFFSET (Field) + CONFIG_OPTION_OFFSET)
 
 #define BOOT_TIME_OUT_VAR_OFFSET        VAR_OFFSET (BootTimeOut)
 #define BOOT_NEXT_VAR_OFFSET            VAR_OFFSET (BootNext)
@@ -193,47 +193,47 @@ typedef enum _TYPE_OF_TERMINAL {
 #define COM_TERMINAL_VAR_OFFSET         VAR_OFFSET (COMTerminalType)
 #define COM_FLOWCONTROL_VAR_OFFSET      VAR_OFFSET (COMFlowControl)
 
-#define BOOT_TIME_OUT_QUESTION_ID       QUESTION_ID (BootTimeOut)
-#define BOOT_NEXT_QUESTION_ID           QUESTION_ID (BootNext)
-#define COM1_BAUD_RATE_QUESTION_ID      QUESTION_ID (COM1BaudRate)
-#define COM1_DATA_RATE_QUESTION_ID      QUESTION_ID (COM1DataRate)
-#define COM1_STOP_BITS_QUESTION_ID      QUESTION_ID (COM1StopBits)
-#define COM1_PARITY_QUESTION_ID         QUESTION_ID (COM1Parity)
-#define COM1_TERMINAL_QUESTION_ID       QUESTION_ID (COM2TerminalType)
-#define COM2_BAUD_RATE_QUESTION_ID      QUESTION_ID (COM2BaudRate)
-#define COM2_DATA_RATE_QUESTION_ID      QUESTION_ID (COM2DataRate)
-#define COM2_STOP_BITS_QUESTION_ID      QUESTION_ID (COM2StopBits)
-#define COM2_PARITY_QUESTION_ID         QUESTION_ID (COM2Parity)
-#define COM2_TERMINAL_QUESTION_ID       QUESTION_ID (COM2TerminalType)
-#define DRV_ADD_HANDLE_DESC_QUESTION_ID QUESTION_ID (DriverAddHandleDesc)
-#define DRV_ADD_ACTIVE_QUESTION_ID      QUESTION_ID (DriverAddActive)
-#define DRV_ADD_RECON_QUESTION_ID       QUESTION_ID (DriverAddForceReconnect)
-#define CON_IN_COM1_QUESTION_ID         QUESTION_ID (ConsoleInputCOM1)
-#define CON_IN_COM2_QUESTION_ID         QUESTION_ID (ConsoleInputCOM2)
-#define CON_OUT_COM1_QUESTION_ID        QUESTION_ID (ConsoleOutputCOM1)
-#define CON_OUT_COM2_QUESTION_ID        QUESTION_ID (ConsoleOutputCOM2)
-#define CON_ERR_COM1_QUESTION_ID        QUESTION_ID (ConsoleErrorCOM1)
-#define CON_ERR_COM2_QUESTION_ID        QUESTION_ID (ConsoleErrorCOM2)
-#define CON_MODE_QUESTION_ID            QUESTION_ID (ConsoleOutMode)
-#define CON_DEVICE_QUESTION_ID          QUESTION_ID (ConsoleCheck)
-#define CON_IN_DEVICE_QUESTION_ID       QUESTION_ID (ConsoleInCheck)
-#define CON_OUT_DEVICE_QUESTION_ID      QUESTION_ID (ConsoleOutCheck)
-#define CON_ERR_DEVICE_QUESTION_ID      QUESTION_ID (ConsoleErrCheck)
-#define BOOT_OPTION_ORDER_QUESTION_ID   QUESTION_ID (BootOptionOrder)
-#define DRIVER_OPTION_ORDER_QUESTION_ID QUESTION_ID (DriverOptionOrder)
-#define BOOT_OPTION_DEL_QUESTION_ID     QUESTION_ID (BootOptionDel)
-#define DRIVER_OPTION_DEL_QUESTION_ID   QUESTION_ID (DriverOptionDel)
-#define DRIVER_ADD_OPTION_QUESTION_ID   QUESTION_ID (DriverAddHandleOptionalData)
-#define COM_BAUD_RATE_QUESTION_ID       QUESTION_ID (COMBaudRate)
-#define COM_DATA_RATE_QUESTION_ID       QUESTION_ID (COMDataRate)
-#define COM_STOP_BITS_QUESTION_ID       QUESTION_ID (COMStopBits)
-#define COM_PARITY_QUESTION_ID          QUESTION_ID (COMParity)
-#define COM_TERMINAL_QUESTION_ID        QUESTION_ID (COMTerminalType)
-#define COM_FLOWCONTROL_QUESTION_ID     QUESTION_ID (COMFlowControl)
+#define BOOT_TIME_OUT_QUESTION_ID        QUESTION_ID (BootTimeOut)
+#define BOOT_NEXT_QUESTION_ID            QUESTION_ID (BootNext)
+#define COM1_BAUD_RATE_QUESTION_ID       QUESTION_ID (COM1BaudRate)
+#define COM1_DATA_RATE_QUESTION_ID       QUESTION_ID (COM1DataRate)
+#define COM1_STOP_BITS_QUESTION_ID       QUESTION_ID (COM1StopBits)
+#define COM1_PARITY_QUESTION_ID          QUESTION_ID (COM1Parity)
+#define COM1_TERMINAL_QUESTION_ID        QUESTION_ID (COM2TerminalType)
+#define COM2_BAUD_RATE_QUESTION_ID       QUESTION_ID (COM2BaudRate)
+#define COM2_DATA_RATE_QUESTION_ID       QUESTION_ID (COM2DataRate)
+#define COM2_STOP_BITS_QUESTION_ID       QUESTION_ID (COM2StopBits)
+#define COM2_PARITY_QUESTION_ID          QUESTION_ID (COM2Parity)
+#define COM2_TERMINAL_QUESTION_ID        QUESTION_ID (COM2TerminalType)
+#define DRV_ADD_HANDLE_DESC_QUESTION_ID  QUESTION_ID (DriverAddHandleDesc)
+#define DRV_ADD_ACTIVE_QUESTION_ID       QUESTION_ID (DriverAddActive)
+#define DRV_ADD_RECON_QUESTION_ID        QUESTION_ID (DriverAddForceReconnect)
+#define CON_IN_COM1_QUESTION_ID          QUESTION_ID (ConsoleInputCOM1)
+#define CON_IN_COM2_QUESTION_ID          QUESTION_ID (ConsoleInputCOM2)
+#define CON_OUT_COM1_QUESTION_ID         QUESTION_ID (ConsoleOutputCOM1)
+#define CON_OUT_COM2_QUESTION_ID         QUESTION_ID (ConsoleOutputCOM2)
+#define CON_ERR_COM1_QUESTION_ID         QUESTION_ID (ConsoleErrorCOM1)
+#define CON_ERR_COM2_QUESTION_ID         QUESTION_ID (ConsoleErrorCOM2)
+#define CON_MODE_QUESTION_ID             QUESTION_ID (ConsoleOutMode)
+#define CON_DEVICE_QUESTION_ID           QUESTION_ID (ConsoleCheck)
+#define CON_IN_DEVICE_QUESTION_ID        QUESTION_ID (ConsoleInCheck)
+#define CON_OUT_DEVICE_QUESTION_ID       QUESTION_ID (ConsoleOutCheck)
+#define CON_ERR_DEVICE_QUESTION_ID       QUESTION_ID (ConsoleErrCheck)
+#define BOOT_OPTION_ORDER_QUESTION_ID    QUESTION_ID (BootOptionOrder)
+#define DRIVER_OPTION_ORDER_QUESTION_ID  QUESTION_ID (DriverOptionOrder)
+#define BOOT_OPTION_DEL_QUESTION_ID      QUESTION_ID (BootOptionDel)
+#define DRIVER_OPTION_DEL_QUESTION_ID    QUESTION_ID (DriverOptionDel)
+#define DRIVER_ADD_OPTION_QUESTION_ID    QUESTION_ID (DriverAddHandleOptionalData)
+#define COM_BAUD_RATE_QUESTION_ID        QUESTION_ID (COMBaudRate)
+#define COM_DATA_RATE_QUESTION_ID        QUESTION_ID (COMDataRate)
+#define COM_STOP_BITS_QUESTION_ID        QUESTION_ID (COMStopBits)
+#define COM_PARITY_QUESTION_ID           QUESTION_ID (COMParity)
+#define COM_TERMINAL_QUESTION_ID         QUESTION_ID (COMTerminalType)
+#define COM_FLOWCONTROL_QUESTION_ID      QUESTION_ID (COMFlowControl)
 
-#define STRING_DEPOSITORY_NUMBER        8
+#define STRING_DEPOSITORY_NUMBER  8
 
-#define NONE_BOOTNEXT_VALUE             (0xFFFF + 1)
+#define NONE_BOOTNEXT_VALUE  (0xFFFF + 1)
 
 ///
 /// Serial Ports attributes, first one is the value for
@@ -241,125 +241,122 @@ typedef enum _TYPE_OF_TERMINAL {
 /// display the value properly
 ///
 typedef struct {
-  UINTN   Value;
-  UINT16  StringToken;
+  UINTN     Value;
+  UINT16    StringToken;
 } COM_ATTR;
 
 typedef struct {
-  UINT64                    BaudRate;
-  UINT8                     DataBits;
-  UINT8                     Parity;
-  UINT8                     StopBits;
+  UINT64                      BaudRate;
+  UINT8                       DataBits;
+  UINT8                       Parity;
+  UINT8                       StopBits;
 
-  UINT8                     BaudRateIndex;
-  UINT8                     DataBitsIndex;
-  UINT8                     ParityIndex;
-  UINT8                     StopBitsIndex;
+  UINT8                       BaudRateIndex;
+  UINT8                       DataBitsIndex;
+  UINT8                       ParityIndex;
+  UINT8                       StopBitsIndex;
 
-  UINT8                     FlowControl;
+  UINT8                       FlowControl;
 
-  UINT8                     IsConIn;
-  UINT8                     IsConOut;
-  UINT8                     IsStdErr;
-  UINT8                     TerminalType;
+  UINT8                       IsConIn;
+  UINT8                       IsConOut;
+  UINT8                       IsStdErr;
+  UINT8                       TerminalType;
 
-  EFI_DEVICE_PATH_PROTOCOL  *DevicePath;
+  EFI_DEVICE_PATH_PROTOCOL    *DevicePath;
 } BM_TERMINAL_CONTEXT;
 
 typedef struct {
-  BOOLEAN                   IsBootNext;
-  BOOLEAN                   Deleted;
+  BOOLEAN                     IsBootNext;
+  BOOLEAN                     Deleted;
 
-  BOOLEAN                   IsLegacy;
+  BOOLEAN                     IsLegacy;
 
-  UINT32                    Attributes;
-  UINT16                    FilePathListLength;
-  UINT16                    *Description;
-  EFI_DEVICE_PATH_PROTOCOL  *FilePathList;
-  UINT8                     *OptionalData;
+  UINT32                      Attributes;
+  UINT16                      FilePathListLength;
+  UINT16                      *Description;
+  EFI_DEVICE_PATH_PROTOCOL    *FilePathList;
+  UINT8                       *OptionalData;
 } BM_LOAD_CONTEXT;
 
 typedef struct {
+  BOOLEAN                     IsActive;
 
-  BOOLEAN                   IsActive;
+  BOOLEAN                     IsTerminal;
 
-  BOOLEAN                   IsTerminal;
-
-  EFI_DEVICE_PATH_PROTOCOL  *DevicePath;
+  EFI_DEVICE_PATH_PROTOCOL    *DevicePath;
 } BM_CONSOLE_CONTEXT;
 
 typedef struct {
-  UINTN   Column;
-  UINTN   Row;
+  UINTN    Column;
+  UINTN    Row;
 } CONSOLE_OUT_MODE;
 
 typedef struct {
-  EFI_HANDLE                        Handle;
-  EFI_DEVICE_PATH_PROTOCOL          *DevicePath;
-  EFI_FILE_HANDLE                   FHandle;
-  UINT16                            *FileName;
-  EFI_FILE_SYSTEM_VOLUME_LABEL      *Info;
+  EFI_HANDLE                      Handle;
+  EFI_DEVICE_PATH_PROTOCOL        *DevicePath;
+  EFI_FILE_HANDLE                 FHandle;
+  UINT16                          *FileName;
+  EFI_FILE_SYSTEM_VOLUME_LABEL    *Info;
 
-  BOOLEAN                           IsRoot;
-  BOOLEAN                           IsDir;
-  BOOLEAN                           IsRemovableMedia;
-  BOOLEAN                           IsLoadFile;
-  BOOLEAN                           IsBootLegacy;
+  BOOLEAN                         IsRoot;
+  BOOLEAN                         IsDir;
+  BOOLEAN                         IsRemovableMedia;
+  BOOLEAN                         IsLoadFile;
+  BOOLEAN                         IsBootLegacy;
 } BM_FILE_CONTEXT;
 
 typedef struct {
-  EFI_HANDLE                Handle;
-  EFI_DEVICE_PATH_PROTOCOL  *DevicePath;
+  EFI_HANDLE                  Handle;
+  EFI_DEVICE_PATH_PROTOCOL    *DevicePath;
 } BM_HANDLE_CONTEXT;
 
 typedef struct {
-  UINTN           Signature;
-  LIST_ENTRY      Head;
-  UINTN           MenuNumber;
+  UINTN         Signature;
+  LIST_ENTRY    Head;
+  UINTN         MenuNumber;
 } BM_MENU_OPTION;
 
 typedef struct {
-  UINTN           Signature;
-  LIST_ENTRY      Link;
-  UINTN           OptionNumber;
-  UINT16          *DisplayString;
-  UINT16          *HelpString;
-  EFI_STRING_ID   DisplayStringToken;
-  EFI_STRING_ID   HelpStringToken;
-  UINTN           ContextSelection;
-  VOID            *VariableContext;
+  UINTN            Signature;
+  LIST_ENTRY       Link;
+  UINTN            OptionNumber;
+  UINT16           *DisplayString;
+  UINT16           *HelpString;
+  EFI_STRING_ID    DisplayStringToken;
+  EFI_STRING_ID    HelpStringToken;
+  UINTN            ContextSelection;
+  VOID             *VariableContext;
 } BM_MENU_ENTRY;
 
 typedef struct {
+  UINTN                             Signature;
 
-  UINTN                          Signature;
-
-  EFI_HII_HANDLE                 BmmHiiHandle;
-  EFI_HANDLE                     BmmDriverHandle;
+  EFI_HII_HANDLE                    BmmHiiHandle;
+  EFI_HANDLE                        BmmDriverHandle;
   ///
   /// Boot Maintenance  Manager Produced protocols
   ///
-  EFI_HII_CONFIG_ACCESS_PROTOCOL BmmConfigAccess;
-  EFI_FORM_BROWSER2_PROTOCOL     *FormBrowser2;
+  EFI_HII_CONFIG_ACCESS_PROTOCOL    BmmConfigAccess;
+  EFI_FORM_BROWSER2_PROTOCOL        *FormBrowser2;
 
-  BM_MENU_ENTRY                  *MenuEntry;
-  BM_HANDLE_CONTEXT              *HandleContext;
-  BM_FILE_CONTEXT                *FileContext;
-  BM_LOAD_CONTEXT                *LoadContext;
-  BM_TERMINAL_CONTEXT            *TerminalContext;
-  UINTN                          CurrentTerminal;
-  BBS_TYPE                       BbsType;
+  BM_MENU_ENTRY                     *MenuEntry;
+  BM_HANDLE_CONTEXT                 *HandleContext;
+  BM_FILE_CONTEXT                   *FileContext;
+  BM_LOAD_CONTEXT                   *LoadContext;
+  BM_TERMINAL_CONTEXT               *TerminalContext;
+  UINTN                             CurrentTerminal;
+  BBS_TYPE                          BbsType;
 
   //
   // BMM main formset callback data.
   //
 
-  EFI_FORM_ID                    BmmCurrentPageId;
-  EFI_FORM_ID                    BmmPreviousPageId;
-  BOOLEAN                        BmmAskSaveOrNot;
-  BMM_FAKE_NV_DATA               BmmFakeNvData;
-  BMM_FAKE_NV_DATA               BmmOldFakeNVData;
-
+  EFI_FORM_ID                       BmmCurrentPageId;
+  EFI_FORM_ID                       BmmPreviousPageId;
+  BOOLEAN                           BmmAskSaveOrNot;
+  BMM_FAKE_NV_DATA                  BmmFakeNvData;
+  BMM_FAKE_NV_DATA                  BmmOldFakeNVData;
 } BMM_CALLBACK_DATA;
 
 /**
@@ -390,7 +387,7 @@ BOpt_FindDrivers (
 **/
 EFI_STATUS
 BOpt_GetBootOptions (
-  IN  BMM_CALLBACK_DATA         *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -399,14 +396,14 @@ BOpt_GetBootOptions (
 
   @param CallbackData The BMM context data.
 
-  @return EFI_SUCESS The functin completes successfully.
+  @return EFI_SUCCESS          The function completes successfully.
   @retval EFI_OUT_OF_RESOURCES Not enough memory to compete the operation.
 
 
 **/
 EFI_STATUS
 BOpt_GetDriverOptions (
-  IN  BMM_CALLBACK_DATA         *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -417,7 +414,7 @@ BOpt_GetDriverOptions (
 **/
 VOID
 BOpt_FreeMenu (
-  BM_MENU_OPTION        *FreeMenu
+  BM_MENU_OPTION  *FreeMenu
   );
 
 /**
@@ -431,7 +428,7 @@ BOpt_FreeMenu (
 **/
 UINT16
 BOpt_GetOptionNumber (
-  CHAR16        *Type
+  CHAR16  *Type
   );
 
 /**
@@ -470,7 +467,7 @@ BOpt_GetDriverOptionNumber (
 **/
 BM_MENU_ENTRY                     *
 BOpt_CreateMenuEntry (
-  UINTN           MenuType
+  UINTN  MenuType
   );
 
 /**
@@ -481,7 +478,7 @@ BOpt_CreateMenuEntry (
 **/
 VOID
 BOpt_DestroyMenuEntry (
-  BM_MENU_ENTRY         *MenuEntry
+  BM_MENU_ENTRY  *MenuEntry
   );
 
 /**
@@ -498,8 +495,8 @@ BOpt_DestroyMenuEntry (
 **/
 BM_MENU_ENTRY                     *
 BOpt_GetMenuEntry (
-  BM_MENU_OPTION      *MenuOption,
-  UINTN               MenuNumber
+  BM_MENU_OPTION  *MenuOption,
+  UINTN           MenuNumber
   );
 
 /**
@@ -510,7 +507,7 @@ BOpt_GetMenuEntry (
 **/
 VOID
 GetBootOrder (
-  IN  BMM_CALLBACK_DATA    *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -521,12 +518,13 @@ GetBootOrder (
 **/
 VOID
 GetDriverOrder (
-  IN  BMM_CALLBACK_DATA    *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 //
 // Locate all serial io devices for console
 //
+
 /**
   Build a list containing all serial devices.
 
@@ -542,6 +540,7 @@ LocateSerialIo (
 //
 // Initializing Console menu
 //
+
 /**
   Build up ConsoleOutMenu, ConsoleInpMenu and ConsoleErrMenu
 
@@ -549,13 +548,14 @@ LocateSerialIo (
 
 **/
 EFI_STATUS
-GetAllConsoles(
+GetAllConsoles (
   VOID
   );
 
 //
 // Get current mode information
 //
+
 /**
   Get mode number according to column and row
 
@@ -563,12 +563,13 @@ GetAllConsoles(
 **/
 VOID
 GetConsoleOutMode (
-  IN  BMM_CALLBACK_DATA    *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 //
 // Cleaning up console menu
 //
+
 /**
   Free ConsoleOutMenu, ConsoleInpMenu and ConsoleErrMenu
 
@@ -607,12 +608,13 @@ ChangeVariableDevicePath (
 EFI_STATUS
 ChangeTerminalDevicePath (
   IN OUT EFI_DEVICE_PATH_PROTOCOL  *DevicePath,
-  IN BOOLEAN                   ChangeTerminal
+  IN BOOLEAN                       ChangeTerminal
   );
 
 //
 // Variable operation by menu selection
 //
+
 /**
   This function create a currently loaded Boot Option from
   the BMM. It then appends this Boot Option to the end of
@@ -627,7 +629,7 @@ ChangeTerminalDevicePath (
 **/
 EFI_STATUS
 Var_UpdateBootOption (
-  IN  BMM_CALLBACK_DATA                   *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -661,11 +663,11 @@ Var_DelBootOption (
 **/
 EFI_STATUS
 Var_UpdateDriverOption (
-  IN  BMM_CALLBACK_DATA         *CallbackData,
-  IN  EFI_HII_HANDLE            HiiHandle,
-  IN  UINT16                    *DescriptionData,
-  IN  UINT16                    *OptionalData,
-  IN  UINT8                     ForceReconnect
+  IN  BMM_CALLBACK_DATA  *CallbackData,
+  IN  EFI_HII_HANDLE     HiiHandle,
+  IN  UINT16             *DescriptionData,
+  IN  UINT16             *OptionalData,
+  IN  UINT8              ForceReconnect
   );
 
 /**
@@ -725,7 +727,7 @@ Var_UpdateErrorOutOption (
 **/
 EFI_STATUS
 Var_UpdateOutOfBandOption (
-  IN  UINT16           MenuIndex
+  IN  UINT16  MenuIndex
   );
 
 /**
@@ -741,7 +743,7 @@ Var_UpdateOutOfBandOption (
 **/
 EFI_STATUS
 Var_UpdateBootNext (
-  IN BMM_CALLBACK_DATA            *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -757,7 +759,7 @@ Var_UpdateBootNext (
 **/
 EFI_STATUS
 Var_UpdateBootOrder (
-  IN BMM_CALLBACK_DATA            *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -774,7 +776,7 @@ Var_UpdateBootOrder (
 **/
 EFI_STATUS
 Var_UpdateDriverOrder (
-  IN BMM_CALLBACK_DATA            *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -788,12 +790,13 @@ Var_UpdateDriverOrder (
 **/
 EFI_STATUS
 Var_UpdateConMode (
-  IN BMM_CALLBACK_DATA            *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 //
 // Following are page create and refresh functions
 //
+
 /**
  Create the global UpdateData structure.
 
@@ -823,8 +826,8 @@ RefreshUpdateData (
 **/
 VOID
 CleanUpPage (
-  IN UINT16                           LabelId,
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN UINT16             LabelId,
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -836,7 +839,7 @@ CleanUpPage (
 **/
 VOID
 UpdateBootDelPage (
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -846,7 +849,7 @@ UpdateBootDelPage (
 **/
 VOID
 UpdateDrvAddHandlePage (
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -857,7 +860,7 @@ UpdateDrvAddHandlePage (
 **/
 VOID
 UpdateDrvDelPage (
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -867,7 +870,7 @@ UpdateDrvDelPage (
 **/
 VOID
 UpdateDriverAddHandleDescPage (
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -878,8 +881,8 @@ UpdateDriverAddHandleDescPage (
 **/
 VOID
 UpdatePageBody (
-  IN UINT16                           UpdatePageId,
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN UINT16             UpdatePageId,
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -890,7 +893,7 @@ UpdatePageBody (
 **/
 VOID
 UpdateTerminalPage (
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -900,7 +903,7 @@ UpdateTerminalPage (
 **/
 VOID
 UpdateConModePage (
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -911,7 +914,7 @@ UpdateConModePage (
 **/
 VOID
 UpdateConCOMPage (
-  IN BMM_CALLBACK_DATA                *CallbackData
+  IN BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -923,10 +926,10 @@ UpdateConCOMPage (
 
 **/
 VOID
-UpdateOptionPage(
-  IN   BMM_CALLBACK_DATA        *CallbackData,
-  IN   EFI_FORM_ID              FormId,
-  IN   EFI_DEVICE_PATH_PROTOCOL *DevicePath
+UpdateOptionPage (
+  IN   BMM_CALLBACK_DATA         *CallbackData,
+  IN   EFI_FORM_ID               FormId,
+  IN   EFI_DEVICE_PATH_PROTOCOL  *DevicePath
   );
 
 /**
@@ -946,8 +949,8 @@ UpdateOptionPage(
 **/
 EFI_STATUS
 EfiLibDeleteVariable (
-  IN CHAR16   *VarName,
-  IN EFI_GUID *VarGuid
+  IN CHAR16    *VarName,
+  IN EFI_GUID  *VarGuid
   );
 
 /**
@@ -963,7 +966,7 @@ EfiLibDeleteVariable (
 **/
 UINTN
 EfiDevicePathInstanceCount (
-  IN EFI_DEVICE_PATH_PROTOCOL      *DevicePath
+  IN EFI_DEVICE_PATH_PROTOCOL  *DevicePath
   );
 
 /**
@@ -979,7 +982,7 @@ EfiDevicePathInstanceCount (
 **/
 UINT16 *
 EfiLibStrFromDatahub (
-  IN EFI_DEVICE_PATH_PROTOCOL                 *DevPath
+  IN EFI_DEVICE_PATH_PROTOCOL  *DevPath
   );
 
 /**
@@ -994,9 +997,9 @@ EfiLibStrFromDatahub (
 **/
 VOID *
 GetLegacyBootOptionVar (
-  IN  UINTN                            DeviceType,
-  OUT UINTN                            *OptionIndex,
-  OUT UINTN                            *OptionSize
+  IN  UINTN  DeviceType,
+  OUT UINTN  *OptionIndex,
+  OUT UINTN  *OptionSize
   );
 
 /**
@@ -1009,11 +1012,10 @@ GetLegacyBootOptionVar (
 **/
 VOID
 DiscardChangeHandler (
-  IN  BMM_CALLBACK_DATA               *Private,
-  IN  BMM_FAKE_NV_DATA                *CurrentFakeNVMap
+  IN  BMM_CALLBACK_DATA  *Private,
+  IN  BMM_FAKE_NV_DATA   *CurrentFakeNVMap
   );
 
-
 /**
   This function is to clean some useless data before submit changes.
 
@@ -1022,7 +1024,7 @@ DiscardChangeHandler (
 **/
 VOID
 CleanUselessBeforeSubmit (
-  IN  BMM_CALLBACK_DATA               *Private
+  IN  BMM_CALLBACK_DATA  *Private
   );
 
 /**
@@ -1034,8 +1036,8 @@ CleanUselessBeforeSubmit (
 **/
 VOID
 UpdatePageId (
-  BMM_CALLBACK_DATA              *Private,
-  UINT16                         NewPageId
+  BMM_CALLBACK_DATA  *Private,
+  UINT16             NewPageId
   );
 
 /**
@@ -1043,7 +1045,7 @@ UpdatePageId (
 
 **/
 VOID
-FreeBMPackage(
+FreeBMPackage (
   VOID
   );
 
@@ -1052,7 +1054,7 @@ FreeBMPackage(
 
 **/
 VOID
-InitBootMaintenance(
+InitBootMaintenance (
   VOID
   );
 
@@ -1066,7 +1068,7 @@ InitBootMaintenance(
 **/
 VOID
 GetConsoleInCheck (
-  IN  BMM_CALLBACK_DATA    *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -1079,7 +1081,7 @@ GetConsoleInCheck (
 **/
 VOID
 GetConsoleOutCheck (
-  IN  BMM_CALLBACK_DATA    *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -1092,7 +1094,7 @@ GetConsoleOutCheck (
 **/
 VOID
 GetConsoleErrCheck (
-  IN  BMM_CALLBACK_DATA    *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -1105,7 +1107,7 @@ GetConsoleErrCheck (
 **/
 VOID
 GetTerminalAttribute (
-  IN  BMM_CALLBACK_DATA    *CallbackData
+  IN  BMM_CALLBACK_DATA  *CallbackData
   );
 
 /**
@@ -1123,7 +1125,6 @@ BmmSetConsoleMode (
   BOOLEAN  IsSetupMode
   );
 
-
 /**
   This function converts an input device structure to a Unicode string.
 
@@ -1134,7 +1135,7 @@ BmmSetConsoleMode (
 **/
 CHAR16 *
 UiDevicePathToStr (
-  IN EFI_DEVICE_PATH_PROTOCOL     *DevPath
+  IN EFI_DEVICE_PATH_PROTOCOL  *DevPath
   );
 
 /**
@@ -1148,7 +1149,7 @@ UiDevicePathToStr (
 **/
 CHAR16 *
 ExtractFileNameFromDevicePath (
-  IN   EFI_DEVICE_PATH_PROTOCOL *DevicePath
+  IN   EFI_DEVICE_PATH_PROTOCOL  *DevicePath
   );
 
 /**
@@ -1175,10 +1176,10 @@ ExtractFileNameFromDevicePath (
 EFI_STATUS
 EFIAPI
 BootMaintExtractConfig (
-  IN  CONST EFI_HII_CONFIG_ACCESS_PROTOCOL   *This,
-  IN  CONST EFI_STRING                       Request,
-  OUT EFI_STRING                             *Progress,
-  OUT EFI_STRING                             *Results
+  IN  CONST EFI_HII_CONFIG_ACCESS_PROTOCOL  *This,
+  IN  CONST EFI_STRING                      Request,
+  OUT EFI_STRING                            *Progress,
+  OUT EFI_STRING                            *Results
   );
 
 /**
@@ -1217,9 +1218,9 @@ BootMaintExtractConfig (
 EFI_STATUS
 EFIAPI
 BootMaintRouteConfig (
-  IN CONST EFI_HII_CONFIG_ACCESS_PROTOCOL *This,
-  IN CONST EFI_STRING                     Configuration,
-  OUT EFI_STRING                          *Progress
+  IN CONST EFI_HII_CONFIG_ACCESS_PROTOCOL  *This,
+  IN CONST EFI_STRING                      Configuration,
+  OUT EFI_STRING                           *Progress
   );
 
 /**
@@ -1243,12 +1244,12 @@ BootMaintRouteConfig (
 EFI_STATUS
 EFIAPI
 BootMaintCallback (
-  IN  CONST EFI_HII_CONFIG_ACCESS_PROTOCOL         *This,
-  IN        EFI_BROWSER_ACTION                     Action,
-  IN        EFI_QUESTION_ID                        QuestionId,
-  IN        UINT8                                  Type,
-  IN        EFI_IFR_TYPE_VALUE                     *Value,
-  OUT       EFI_BROWSER_ACTION_REQUEST             *ActionRequest
+  IN  CONST EFI_HII_CONFIG_ACCESS_PROTOCOL  *This,
+  IN        EFI_BROWSER_ACTION              Action,
+  IN        EFI_QUESTION_ID                 QuestionId,
+  IN        UINT8                           Type,
+  IN        EFI_IFR_TYPE_VALUE              *Value,
+  OUT       EFI_BROWSER_ACTION_REQUEST      *ActionRequest
   );
 
 /**
@@ -1263,7 +1264,7 @@ BootMaintCallback (
 BOOLEAN
 EFIAPI
 CreateBootOptionFromFile (
-  IN EFI_DEVICE_PATH_PROTOCOL    *FilePath
+  IN EFI_DEVICE_PATH_PROTOCOL  *FilePath
   );
 
 /**
@@ -1277,7 +1278,7 @@ CreateBootOptionFromFile (
 BOOLEAN
 EFIAPI
 CreateDriverOptionFromFile (
-  IN EFI_DEVICE_PATH_PROTOCOL    *FilePath
+  IN EFI_DEVICE_PATH_PROTOCOL  *FilePath
   );
 
 /**
@@ -1292,37 +1293,37 @@ CreateDriverOptionFromFile (
 BOOLEAN
 EFIAPI
 BootFromFile (
-  IN EFI_DEVICE_PATH_PROTOCOL    *FilePath
+  IN EFI_DEVICE_PATH_PROTOCOL  *FilePath
   );
 
 //
 // Global variable in this program (defined in data.c)
 //
-extern BM_MENU_OPTION             BootOptionMenu;
-extern BM_MENU_OPTION             DriverOptionMenu;
-extern BM_MENU_OPTION             ConsoleInpMenu;
-extern BM_MENU_OPTION             ConsoleOutMenu;
-extern BM_MENU_OPTION             ConsoleErrMenu;
-extern BM_MENU_OPTION             DriverMenu;
-extern BM_MENU_OPTION             TerminalMenu;
-extern UINT16                     TerminalType[9];
-extern COM_ATTR                   BaudRateList[19];
-extern COM_ATTR                   DataBitsList[4];
-extern COM_ATTR                   ParityList[5];
-extern COM_ATTR                   StopBitsList[3];
-extern EFI_GUID                   TerminalTypeGuid[9];
-extern EFI_DEVICE_PATH_PROTOCOL   EndDevicePath[];
-extern UINT16                     mFlowControlType[2];
-extern UINT32                     mFlowControlValue[2];
+extern BM_MENU_OPTION            BootOptionMenu;
+extern BM_MENU_OPTION            DriverOptionMenu;
+extern BM_MENU_OPTION            ConsoleInpMenu;
+extern BM_MENU_OPTION            ConsoleOutMenu;
+extern BM_MENU_OPTION            ConsoleErrMenu;
+extern BM_MENU_OPTION            DriverMenu;
+extern BM_MENU_OPTION            TerminalMenu;
+extern UINT16                    TerminalType[9];
+extern COM_ATTR                  BaudRateList[19];
+extern COM_ATTR                  DataBitsList[4];
+extern COM_ATTR                  ParityList[5];
+extern COM_ATTR                  StopBitsList[3];
+extern EFI_GUID                  TerminalTypeGuid[9];
+extern EFI_DEVICE_PATH_PROTOCOL  EndDevicePath[];
+extern UINT16                    mFlowControlType[2];
+extern UINT32                    mFlowControlValue[2];
 
 //
 // Shared IFR form update data
 //
-extern VOID                        *mStartOpCodeHandle;
-extern VOID                        *mEndOpCodeHandle;
-extern EFI_IFR_GUID_LABEL          *mStartLabel;
-extern EFI_IFR_GUID_LABEL          *mEndLabel;
-extern BMM_CALLBACK_DATA           gBootMaintenancePrivate;
-extern BMM_CALLBACK_DATA           *mBmmCallbackInfo;
+extern VOID                *mStartOpCodeHandle;
+extern VOID                *mEndOpCodeHandle;
+extern EFI_IFR_GUID_LABEL  *mStartLabel;
+extern EFI_IFR_GUID_LABEL  *mEndLabel;
+extern BMM_CALLBACK_DATA   gBootMaintenancePrivate;
+extern BMM_CALLBACK_DATA   *mBmmCallbackInfo;
 
 #endif
