@@ -378,7 +378,7 @@ ProcessExternedOpcode (
 }
 
 /**
-  Validate the input screen diemenstion info.
+  Validate the input screen dimension info.
 
   @param  FormData               The input form data info.
 
@@ -387,7 +387,7 @@ ProcessExternedOpcode (
 
 **/
 EFI_STATUS
-ScreenDiemensionInfoValidate (
+ScreenDimensionInfoValidate (
   IN FORM_DISPLAY_ENGINE_FORM  *FormData
   )
 {
