@@ -20,7 +20,7 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 #include <Library/BaseLib.h>
 #include <Library/DebugLib.h>
 
-EFI_FILE_EXPLORER_PROTOCOL *mProtocol = NULL;
+EFI_FILE_EXPLORER_PROTOCOL  *mProtocol = NULL;
 
 /**
   The constructor function caches the pointer to file explorer protocol.
@@ -37,16 +37,16 @@ EFI_FILE_EXPLORER_PROTOCOL *mProtocol = NULL;
 EFI_STATUS
 EFIAPI
 FileExplorerConstructor (
-  IN EFI_HANDLE                ImageHandle,
-  IN EFI_SYSTEM_TABLE          *SystemTable
+  IN EFI_HANDLE        ImageHandle,
+  IN EFI_SYSTEM_TABLE  *SystemTable
   )
 {
-  EFI_STATUS                   Status;
+  EFI_STATUS  Status;
 
   Status = SystemTable->BootServices->LocateProtocol (
                                         &gEfiFileExplorerProtocolGuid,
                                         NULL,
-                                        (VOID**) &mProtocol
+                                        (VOID **)&mProtocol
                                         );
   ASSERT_EFI_ERROR (Status);
   ASSERT (mProtocol != NULL);
@@ -68,7 +68,7 @@ FileExplorerConstructor (
                            after choose one file.
   @param  File             Return the device path for the last time chosed file.
 
-  @retval EFI_SUCESS             Choose file success.
+  @retval EFI_SUCCESS            Choose file success.
   @retval EFI_INVALID_PARAMETER  Both ChooseHandler and return device path are NULL
                                  One of them must not NULL.
   @retval Other errors           Choose file failed.
@@ -77,11 +77,10 @@ EFI_STATUS
 EFIAPI
 ChooseFile (
   IN  EFI_DEVICE_PATH_PROTOCOL  *RootDirectory,
-  IN  CHAR16                    *FileType,  OPTIONAL
-  IN  CHOOSE_HANDLER            ChooseHandler,  OPTIONAL
+  IN  CHAR16                    *FileType   OPTIONAL,
+  IN  CHOOSE_HANDLER            ChooseHandler   OPTIONAL,
   OUT EFI_DEVICE_PATH_PROTOCOL  **File  OPTIONAL
   )
 {
   return mProtocol->ChooseFile (RootDirectory, FileType, ChooseHandler, File);
 }
-
