@@ -27,6 +27,15 @@ EFI_DRIVER_BINDING_PROTOCOL  gConPlatformTextOutDriverBinding = {
   NULL
 };
 
+//
+// Values from Usb Inteface Association Descriptor Device
+//  Class Code and Usage Model specification (iadclasscode_r10.pdf)
+//  from Usb.org
+//
+#define USB_BASE_CLASS_MISCELLANEOUS       0xEF
+#define USB_MISCELLANEOUS_SUBCLASS_COMMON  0x02
+#define USB_MISCELLANEOUS_PROTOCOL_IAD     0x01
+
 /**
   Entrypoint of this module.
 
@@ -808,10 +817,16 @@ MatchUsbClass (
   DeviceClass    = DevDesc.DeviceClass;
   DeviceSubClass = DevDesc.DeviceSubClass;
   DeviceProtocol = DevDesc.DeviceProtocol;
-  if (DeviceClass == 0) {
+
+  if ((DeviceClass == 0) ||
+      ((DeviceClass == USB_BASE_CLASS_MISCELLANEOUS) &&
+       (DeviceSubClass == USB_MISCELLANEOUS_SUBCLASS_COMMON) &&
+       (DeviceProtocol == USB_MISCELLANEOUS_PROTOCOL_IAD)))
+  {
     //
-    // If Class in Device Descriptor is set to 0, use the Class, SubClass and
-    // Protocol in Interface Descriptor instead.
+    // If Class in Device Descriptor is set to 0 (Device), or
+    // Class/SubClass/Protocol is 0xEF/0x02/0x01 (IAD), use the Class, SubClass
+    // and Protocol in Interface Descriptor instead.
     //
     Status = UsbIo->UsbGetInterfaceDescriptor (UsbIo, &IfDesc);
     if (EFI_ERROR (Status)) {
@@ -1081,10 +1096,8 @@ ConPlatformMatchDevicePaths (
   //
   // If performing Delete operation, the NewDevicePath must not be NULL.
   //
-  if (Delete) {
-    if (NewDevicePath == NULL) {
-      return EFI_INVALID_PARAMETER;
-    }
+  if (Delete && (NewDevicePath == NULL)) {
+    return EFI_INVALID_PARAMETER;
   }
 
   TempDevicePath1 = NULL;
@@ -1163,6 +1176,8 @@ ConPlatformUpdateDeviceVariable (
   EFI_DEVICE_PATH_PROTOCOL  *VariableDevicePath;
   EFI_DEVICE_PATH_PROTOCOL  *NewVariableDevicePath;
 
+  Status = EFI_SUCCESS;
+
   VariableDevicePath    = NULL;
   NewVariableDevicePath = NULL;
 
@@ -1172,7 +1187,7 @@ ConPlatformUpdateDeviceVariable (
   // it is the caller's responsibility to free the memory before return.
   //
   VariableDevicePath = ConPlatformGetVariable (VariableName);
-
+  // At this point, VariableDevicePath may be null.  This is expected.
   if (Operation != Delete) {
     //
     // Match specified DevicePath in Console Variable.
