@@ -3,13 +3,8 @@
 
   Copyright (c) 2016, Intel Corporation. All rights reserved.<BR>
   (C) Copyright 2016-2018 Hewlett Packard Enterprise Development LP<BR>
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  Copyright (c) Microsoft Corporation.<BR>
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -17,7 +12,7 @@
 
 CHAR16  mRamDiskStorageName[] = L"RAM_DISK_CONFIGURATION";
 
-RAM_DISK_CONFIG_PRIVATE_DATA mRamDiskConfigPrivateDataTemplate = {
+RAM_DISK_CONFIG_PRIVATE_DATA  mRamDiskConfigPrivateDataTemplate = {
   RAM_DISK_CONFIG_PRIVATE_DATA_SIGNATURE,
   {
     EFI_PAGE_SIZE,
@@ -30,14 +25,14 @@ RAM_DISK_CONFIG_PRIVATE_DATA mRamDiskConfigPrivateDataTemplate = {
   }
 };
 
-HII_VENDOR_DEVICE_PATH       mRamDiskHiiVendorDevicePath = {
+HII_VENDOR_DEVICE_PATH  mRamDiskHiiVendorDevicePath = {
   {
     {
       HARDWARE_DEVICE_PATH,
       HW_VENDOR_DP,
       {
-        (UINT8) (sizeof (VENDOR_DEVICE_PATH)),
-        (UINT8) ((sizeof (VENDOR_DEVICE_PATH)) >> 8)
+        (UINT8)(sizeof (VENDOR_DEVICE_PATH)),
+        (UINT8)((sizeof (VENDOR_DEVICE_PATH)) >> 8)
       }
     },
     RAM_DISK_FORM_SET_GUID
@@ -46,13 +41,12 @@ HII_VENDOR_DEVICE_PATH       mRamDiskHiiVendorDevicePath = {
     END_DEVICE_PATH_TYPE,
     END_ENTIRE_DEVICE_PATH_SUBTYPE,
     {
-      (UINT8) (END_DEVICE_PATH_LENGTH),
-      (UINT8) ((END_DEVICE_PATH_LENGTH) >> 8)
+      (UINT8)(END_DEVICE_PATH_LENGTH),
+      (UINT8)((END_DEVICE_PATH_LENGTH) >> 8)
     }
   }
 };
 
-
 /**
   This function publish the RAM disk configuration Form.
 
@@ -66,7 +60,7 @@ HII_VENDOR_DEVICE_PATH       mRamDiskHiiVendorDevicePath = {
 **/
 EFI_STATUS
 InstallRamDiskConfigForm (
-  IN OUT RAM_DISK_CONFIG_PRIVATE_DATA       *ConfigPrivateData
+  IN OUT RAM_DISK_CONFIG_PRIVATE_DATA  *ConfigPrivateData
   )
 {
   EFI_STATUS                      Status;
@@ -76,14 +70,14 @@ InstallRamDiskConfigForm (
 
   DriverHandle = NULL;
   ConfigAccess = &ConfigPrivateData->ConfigAccess;
-  Status = gBS->InstallMultipleProtocolInterfaces (
-                  &DriverHandle,
-                  &gEfiDevicePathProtocolGuid,
-                  &mRamDiskHiiVendorDevicePath,
-                  &gEfiHiiConfigAccessProtocolGuid,
-                  ConfigAccess,
-                  NULL
-                  );
+  Status       = gBS->InstallMultipleProtocolInterfaces (
+                        &DriverHandle,
+                        &gEfiDevicePathProtocolGuid,
+                        &mRamDiskHiiVendorDevicePath,
+                        &gEfiHiiConfigAccessProtocolGuid,
+                        ConfigAccess,
+                        NULL
+                        );
   if (EFI_ERROR (Status)) {
     return Status;
   }
@@ -117,7 +111,6 @@ InstallRamDiskConfigForm (
   return EFI_SUCCESS;
 }
 
-
 /**
   This function removes RAM disk configuration Form.
 
@@ -127,7 +120,7 @@ InstallRamDiskConfigForm (
 **/
 VOID
 UninstallRamDiskConfigForm (
-  IN OUT RAM_DISK_CONFIG_PRIVATE_DATA       *ConfigPrivateData
+  IN OUT RAM_DISK_CONFIG_PRIVATE_DATA  *ConfigPrivateData
   )
 {
   //
@@ -156,7 +149,6 @@ UninstallRamDiskConfigForm (
   FreePool (ConfigPrivateData);
 }
 
-
 /**
   Unregister all registered RAM disks.
 
@@ -166,12 +158,12 @@ UnregisterAllRamDisks (
   VOID
   )
 {
-  LIST_ENTRY                      *Entry;
-  LIST_ENTRY                      *NextEntry;
-  RAM_DISK_PRIVATE_DATA           *PrivateData;
+  LIST_ENTRY             *Entry;
+  LIST_ENTRY             *NextEntry;
+  RAM_DISK_PRIVATE_DATA  *PrivateData;
 
-  if (!IsListEmpty(&RegisteredRamDisks)) {
-    EFI_LIST_FOR_EACH_SAFE (Entry, NextEntry, &RegisteredRamDisks) {
+  if (!IsListEmpty (&RegisteredRamDisks)) {
+    BASE_LIST_FOR_EACH_SAFE (Entry, NextEntry, &RegisteredRamDisks) {
       PrivateData = RAM_DISK_PRIVATE_FROM_THIS (Entry);
 
       gBS->UninstallMultipleProtocolInterfaces (
@@ -181,7 +173,7 @@ UnregisterAllRamDisks (
              &gEfiBlockIo2ProtocolGuid,
              &PrivateData->BlockIo2,
              &gEfiDevicePathProtocolGuid,
-             (EFI_DEVICE_PATH_PROTOCOL *) PrivateData->DevicePath,
+             (EFI_DEVICE_PATH_PROTOCOL *)PrivateData->DevicePath,
              NULL
              );
 
@@ -193,7 +185,7 @@ UnregisterAllRamDisks (
         // driver is responsible for freeing the allocated memory for the
         // RAM disk.
         //
-        FreePool ((VOID *)(UINTN) PrivateData->StartingAddr);
+        FreePool ((VOID *)(UINTN)PrivateData->StartingAddr);
       }
 
       FreePool (PrivateData->DevicePath);
@@ -202,7 +194,6 @@ UnregisterAllRamDisks (
   }
 }
 
-
 /**
   This function allows a caller to extract the current configuration for one
   or more named elements from the target driver.
@@ -233,20 +224,20 @@ UnregisterAllRamDisks (
 EFI_STATUS
 EFIAPI
 RamDiskExtractConfig (
-  IN CONST EFI_HII_CONFIG_ACCESS_PROTOCOL   *This,
-  IN CONST EFI_STRING                       Request,
-       OUT EFI_STRING                       *Progress,
-       OUT EFI_STRING                       *Results
+  IN CONST EFI_HII_CONFIG_ACCESS_PROTOCOL  *This,
+  IN CONST EFI_STRING                      Request,
+  OUT EFI_STRING                           *Progress,
+  OUT EFI_STRING                           *Results
   )
 {
-  if (Progress == NULL || Results == NULL) {
+  if ((Progress == NULL) || (Results == NULL)) {
     return EFI_INVALID_PARAMETER;
   }
+
   *Progress = Request;
   return EFI_NOT_FOUND;
 }
 
-
 /**
   This function processes the results of changes in configuration.
 
@@ -268,12 +259,12 @@ RamDiskExtractConfig (
 EFI_STATUS
 EFIAPI
 RamDiskRouteConfig (
-  IN CONST EFI_HII_CONFIG_ACCESS_PROTOCOL   *This,
-  IN CONST EFI_STRING                       Configuration,
-       OUT EFI_STRING                       *Progress
+  IN CONST EFI_HII_CONFIG_ACCESS_PROTOCOL  *This,
+  IN CONST EFI_STRING                      Configuration,
+  OUT EFI_STRING                           *Progress
   )
 {
-  if (Configuration == NULL || Progress == NULL) {
+  if ((Configuration == NULL) || (Progress == NULL)) {
     return EFI_INVALID_PARAMETER;
   }
 
@@ -282,7 +273,6 @@ RamDiskRouteConfig (
   return EFI_NOT_FOUND;
 }
 
-
 /**
   Allocate memory and register the RAM disk created within RamDiskDxe
   driver HII.
@@ -300,18 +290,18 @@ RamDiskRouteConfig (
 **/
 EFI_STATUS
 HiiCreateRamDisk (
-  IN UINT64                                 Size,
-  IN EFI_FILE_HANDLE                        FileHandle,
-  IN UINT8                                  MemoryType
+  IN UINT64           Size,
+  IN EFI_FILE_HANDLE  FileHandle,
+  IN UINT8            MemoryType
   )
 {
-  EFI_STATUS                      Status;
-  UINTN                           BufferSize;
-  UINT64                          *StartingAddr;
-  EFI_INPUT_KEY                   Key;
-  EFI_DEVICE_PATH_PROTOCOL        *DevicePath;
-  RAM_DISK_PRIVATE_DATA           *PrivateData;
-  EFI_FILE_INFO                   *FileInformation;
+  EFI_STATUS                Status;
+  UINTN                     BufferSize;
+  UINT64                    *StartingAddr;
+  EFI_INPUT_KEY             Key;
+  EFI_DEVICE_PATH_PROTOCOL  *DevicePath;
+  RAM_DISK_PRIVATE_DATA     *PrivateData;
+  EFI_FILE_INFO             *FileInformation;
 
   FileInformation = NULL;
   StartingAddr    = NULL;
@@ -343,7 +333,7 @@ HiiCreateRamDisk (
     Size = FileInformation->FileSize;
   }
 
-  if (Size > (UINTN) -1) {
+  if (Size > (UINTN)-1) {
     do {
       CreatePopUp (
         EFI_LIGHTGRAY | EFI_BACKGROUND_BLUE,
@@ -363,19 +353,19 @@ HiiCreateRamDisk (
     Status = gBS->AllocatePool (
                     EfiBootServicesData,
                     (UINTN)Size,
-                    (VOID**)&StartingAddr
+                    (VOID **)&StartingAddr
                     );
   } else if (MemoryType == RAM_DISK_RESERVED_MEMORY) {
     Status = gBS->AllocatePool (
                     EfiReservedMemoryType,
                     (UINTN)Size,
-                    (VOID**)&StartingAddr
+                    (VOID **)&StartingAddr
                     );
   } else {
     Status = EFI_INVALID_PARAMETER;
   }
 
-  if ((StartingAddr == NULL) || EFI_ERROR(Status)) {
+  if ((StartingAddr == NULL) || EFI_ERROR (Status)) {
     do {
       CreatePopUp (
         EFI_LIGHTGRAY | EFI_BACKGROUND_BLUE,
@@ -395,11 +385,11 @@ HiiCreateRamDisk (
     //
     // Copy the file content to the RAM disk.
     //
-    BufferSize = (UINTN) Size;
+    BufferSize = (UINTN)Size;
     FileHandle->Read (
                   FileHandle,
                   &BufferSize,
-                  (VOID *)(UINTN) StartingAddr
+                  (VOID *)(UINTN)StartingAddr
                   );
     if (BufferSize != FileInformation->FileSize) {
       do {
@@ -414,7 +404,8 @@ HiiCreateRamDisk (
           );
       } while (Key.UnicodeChar != CHAR_CARRIAGE_RETURN);
 
-      return EFI_DEVICE_ERROR;
+      Status = EFI_DEVICE_ERROR;
+      goto ErrorExit;
     }
   }
 
@@ -422,7 +413,7 @@ HiiCreateRamDisk (
   // Register the newly created RAM disk.
   //
   Status = RamDiskRegister (
-             ((UINT64)(UINTN) StartingAddr),
+             ((UINT64)(UINTN)StartingAddr),
              Size,
              &gEfiVirtualDiskGuid,
              NULL,
@@ -441,19 +432,22 @@ HiiCreateRamDisk (
         );
     } while (Key.UnicodeChar != CHAR_CARRIAGE_RETURN);
 
-    return Status;
+    goto ErrorExit;
   }
 
   //
   // If RAM disk is created within HII, memory should be freed when the
   // RAM disk is unregisterd.
   //
-  PrivateData = RAM_DISK_PRIVATE_FROM_THIS (RegisteredRamDisks.BackLink);
+  PrivateData               = RAM_DISK_PRIVATE_FROM_THIS (RegisteredRamDisks.BackLink);
   PrivateData->CreateMethod = RamDiskCreateHii;
 
   return EFI_SUCCESS;
-}
 
+ErrorExit:
+  gBS->FreePool (StartingAddr);
+  return Status;
+}
 
 /**
   This function updates the registered RAM disks list on the main form.
@@ -465,19 +459,19 @@ HiiCreateRamDisk (
 **/
 VOID
 UpdateMainForm (
-  IN OUT RAM_DISK_CONFIG_PRIVATE_DATA       *ConfigPrivate
+  IN OUT RAM_DISK_CONFIG_PRIVATE_DATA  *ConfigPrivate
   )
 {
-  VOID                      *StartOpCodeHandle;
-  VOID                      *EndOpCodeHandle;
-  EFI_IFR_GUID_LABEL        *StartLabel;
-  EFI_IFR_GUID_LABEL        *EndLabel;
-  LIST_ENTRY                *Entry;
-  UINTN                     Index;
-  RAM_DISK_PRIVATE_DATA     *PrivateData;
-  CHAR16                    *String;
-  CHAR16                    RamDiskStr[128];
-  EFI_STRING_ID             StringId;
+  VOID                   *StartOpCodeHandle;
+  VOID                   *EndOpCodeHandle;
+  EFI_IFR_GUID_LABEL     *StartLabel;
+  EFI_IFR_GUID_LABEL     *EndLabel;
+  LIST_ENTRY             *Entry;
+  UINTN                  Index;
+  RAM_DISK_PRIVATE_DATA  *PrivateData;
+  CHAR16                 *String;
+  CHAR16                 RamDiskStr[128];
+  EFI_STRING_ID          StringId;
 
   //
   // Init OpCode Handle
@@ -491,32 +485,32 @@ UpdateMainForm (
   //
   // Create Hii Extend Label OpCode as the start opcode
   //
-  StartLabel = (EFI_IFR_GUID_LABEL *) HiiCreateGuidOpCode (
-                                        StartOpCodeHandle,
-                                        &gEfiIfrTianoGuid,
-                                        NULL,
-                                        sizeof (EFI_IFR_GUID_LABEL)
-                                        );
+  StartLabel = (EFI_IFR_GUID_LABEL *)HiiCreateGuidOpCode (
+                                       StartOpCodeHandle,
+                                       &gEfiIfrTianoGuid,
+                                       NULL,
+                                       sizeof (EFI_IFR_GUID_LABEL)
+                                       );
   StartLabel->ExtendOpCode = EFI_IFR_EXTEND_OP_LABEL;
   StartLabel->Number       = MAIN_LABEL_LIST_START;
 
   //
   // Create Hii Extend Label OpCode as the end opcode
   //
-  EndLabel = (EFI_IFR_GUID_LABEL *) HiiCreateGuidOpCode (
-                                      EndOpCodeHandle,
-                                      &gEfiIfrTianoGuid,
-                                      NULL,
-                                      sizeof (EFI_IFR_GUID_LABEL)
-                                      );
+  EndLabel = (EFI_IFR_GUID_LABEL *)HiiCreateGuidOpCode (
+                                     EndOpCodeHandle,
+                                     &gEfiIfrTianoGuid,
+                                     NULL,
+                                     sizeof (EFI_IFR_GUID_LABEL)
+                                     );
   EndLabel->ExtendOpCode = EFI_IFR_EXTEND_OP_LABEL;
   EndLabel->Number       = MAIN_LABEL_LIST_END;
 
   Index = 0;
-  EFI_LIST_FOR_EACH (Entry, &RegisteredRamDisks) {
-    PrivateData                  = RAM_DISK_PRIVATE_FROM_THIS (Entry);
-    PrivateData->CheckBoxId      = (EFI_QUESTION_ID)
-                                   (MAIN_CHECKBOX_QUESTION_ID_START + Index);
+  BASE_LIST_FOR_EACH (Entry, &RegisteredRamDisks) {
+    PrivateData             = RAM_DISK_PRIVATE_FROM_THIS (Entry);
+    PrivateData->CheckBoxId = (EFI_QUESTION_ID)
+                              (MAIN_CHECKBOX_QUESTION_ID_START + Index);
     //
     // CheckBox is unchecked by default.
     //
@@ -562,7 +556,6 @@ UpdateMainForm (
   HiiFreeOpCodeHandle (EndOpCodeHandle);
 }
 
-
 /**
   This function processes the results of changes in configuration.
 
@@ -588,21 +581,21 @@ UpdateMainForm (
 EFI_STATUS
 EFIAPI
 RamDiskCallback (
-  IN CONST EFI_HII_CONFIG_ACCESS_PROTOCOL   *This,
-  IN     EFI_BROWSER_ACTION                 Action,
-  IN     EFI_QUESTION_ID                    QuestionId,
-  IN     UINT8                              Type,
-  IN     EFI_IFR_TYPE_VALUE                 *Value,
-     OUT EFI_BROWSER_ACTION_REQUEST         *ActionRequest
+  IN CONST EFI_HII_CONFIG_ACCESS_PROTOCOL  *This,
+  IN     EFI_BROWSER_ACTION                Action,
+  IN     EFI_QUESTION_ID                   QuestionId,
+  IN     UINT8                             Type,
+  IN     EFI_IFR_TYPE_VALUE                *Value,
+  OUT EFI_BROWSER_ACTION_REQUEST           *ActionRequest
   )
 {
-  EFI_STATUS                      Status;
-  RAM_DISK_PRIVATE_DATA           *PrivateData;
-  RAM_DISK_CONFIG_PRIVATE_DATA    *ConfigPrivate;
-  EFI_DEVICE_PATH_PROTOCOL        *FileDevPath;
-  EFI_FILE_HANDLE                 FileHandle;
-  LIST_ENTRY                      *Entry;
-  LIST_ENTRY                      *NextEntry;
+  EFI_STATUS                    Status;
+  RAM_DISK_PRIVATE_DATA         *PrivateData;
+  RAM_DISK_CONFIG_PRIVATE_DATA  *ConfigPrivate;
+  EFI_DEVICE_PATH_PROTOCOL      *FileDevPath;
+  EFI_FILE_HANDLE               FileHandle;
+  LIST_ENTRY                    *Entry;
+  LIST_ENTRY                    *NextEntry;
 
   if ((This == NULL) || (Value == NULL) || (ActionRequest == NULL)) {
     return EFI_INVALID_PARAMETER;
@@ -613,20 +606,22 @@ RamDiskCallback (
   if (Action == EFI_BROWSER_ACTION_RETRIEVE) {
     Status = EFI_UNSUPPORTED;
     if (QuestionId == CREATE_RAW_SIZE_QUESTION_ID) {
-      Value->u64 = EFI_PAGE_SIZE;
+      Value->u64                      = EFI_PAGE_SIZE;
       ConfigPrivate->ConfigStore.Size = EFI_PAGE_SIZE;
-      Status = EFI_SUCCESS;
+      Status                          = EFI_SUCCESS;
     } else if (QuestionId == CREATE_RAW_MEMORY_TYPE_QUESTION_ID) {
-      Value->u8 = RAM_DISK_BOOT_SERVICE_DATA_MEMORY;
+      Value->u8                          = RAM_DISK_BOOT_SERVICE_DATA_MEMORY;
       ConfigPrivate->ConfigStore.MemType = RAM_DISK_BOOT_SERVICE_DATA_MEMORY;
-      Status = EFI_SUCCESS;
+      Status                             = EFI_SUCCESS;
     }
+
     return Status;
   }
 
   if ((Action != EFI_BROWSER_ACTION_CHANGED) &&
       (Action != EFI_BROWSER_ACTION_CHANGING) &&
-      (Action != EFI_BROWSER_ACTION_FORM_OPEN)) {
+      (Action != EFI_BROWSER_ACTION_FORM_OPEN))
+  {
     return EFI_UNSUPPORTED;
   }
 
@@ -639,6 +634,7 @@ RamDiskCallback (
       UpdateMainForm (ConfigPrivate);
       Status = EFI_SUCCESS;
     }
+
     return Status;
   }
 
@@ -646,33 +642,85 @@ RamDiskCallback (
 
   if (Action == EFI_BROWSER_ACTION_CHANGING) {
     switch (QuestionId) {
-    case MAIN_GOTO_FILE_EXPLORER_ID:
-      Status = ChooseFile (NULL, NULL, NULL, &FileDevPath);
-      if (EFI_ERROR (Status)) {
-        break;
-      }
-
-      if (FileDevPath != NULL) {
-        //
-        // Open the file.
-        //
-        Status = EfiOpenFileByDevicePath (
-                   &FileDevPath,
-                   &FileHandle,
-                   EFI_FILE_MODE_READ,
-                   0
-                   );
+      case MAIN_GOTO_FILE_EXPLORER_ID:
+        Status = ChooseFile (NULL, NULL, NULL, &FileDevPath);
         if (EFI_ERROR (Status)) {
           break;
         }
 
+        if (FileDevPath != NULL) {
+          //
+          // Open the file.
+          //
+          Status = EfiOpenFileByDevicePath (
+                     &FileDevPath,
+                     &FileHandle,
+                     EFI_FILE_MODE_READ,
+                     0
+                     );
+          if (EFI_ERROR (Status)) {
+            break;
+          }
+
+          //
+          // Create from file, RAM disk size is zero. It will be updated
+          // according to the file size.
+          //
+          Status = HiiCreateRamDisk (
+                     0,
+                     FileHandle,
+                     ConfigPrivate->ConfigStore.MemType
+                     );
+          if (EFI_ERROR (Status)) {
+            break;
+          }
+
+          //
+          // Refresh the registered RAM disks list.
+          //
+          UpdateMainForm (ConfigPrivate);
+        }
+
+        break;
+
+      default:
+        break;
+    }
+  } else if (Action == EFI_BROWSER_ACTION_CHANGED) {
+    switch (QuestionId) {
+      case MAIN_REMOVE_RD_QUESTION_ID:
         //
-        // Create from file, RAM disk size is zero. It will be updated
-        // according to the file size.
+        // Remove the selected RAM disks
+        //
+        BASE_LIST_FOR_EACH_SAFE (Entry, NextEntry, &RegisteredRamDisks) {
+          PrivateData = RAM_DISK_PRIVATE_FROM_THIS (Entry);
+          if (PrivateData->CheckBoxChecked) {
+            RamDiskUnregister (
+              (EFI_DEVICE_PATH_PROTOCOL *)PrivateData->DevicePath
+              );
+          }
+        }
+
+        UpdateMainForm (ConfigPrivate);
+
+        *ActionRequest = EFI_BROWSER_ACTION_REQUEST_FORM_APPLY;
+        break;
+
+      case CREATE_RAW_SIZE_QUESTION_ID:
+        ConfigPrivate->ConfigStore.Size = Value->u64;
+        break;
+
+      case CREATE_RAW_MEMORY_TYPE_QUESTION_ID:
+        ConfigPrivate->ConfigStore.MemType = Value->u8;
+        break;
+
+      case CREATE_RAW_SUBMIT_QUESTION_ID:
+        //
+        // Create raw, FileHandle is NULL.
         //
         Status = HiiCreateRamDisk (
-                   0,
-                   FileHandle,
+                   ConfigPrivate->ConfigStore.Size,
+                   NULL,
                    ConfigPrivate->ConfigStore.MemType
                    );
         if (EFI_ERROR (Status)) {
@@ -683,79 +731,30 @@ RamDiskCallback (
         // Refresh the registered RAM disks list.
         //
         UpdateMainForm (ConfigPrivate);
-      }
-      break;
 
-    default:
-      break;
-    }
-  } else if (Action == EFI_BROWSER_ACTION_CHANGED) {
-    switch (QuestionId) {
-    case MAIN_REMOVE_RD_QUESTION_ID:
-      //
-      // Remove the selected RAM disks
-      //
-      EFI_LIST_FOR_EACH_SAFE (Entry, NextEntry, &RegisteredRamDisks) {
-        PrivateData = RAM_DISK_PRIVATE_FROM_THIS (Entry);
-        if (PrivateData->CheckBoxChecked) {
-          RamDiskUnregister (
-            (EFI_DEVICE_PATH_PROTOCOL *) PrivateData->DevicePath
-            );
-        }
-      }
-
-      UpdateMainForm (ConfigPrivate);
-
-      *ActionRequest = EFI_BROWSER_ACTION_REQUEST_FORM_APPLY;
-      break;
-
-    case CREATE_RAW_SIZE_QUESTION_ID:
-      ConfigPrivate->ConfigStore.Size = Value->u64;
-      break;
-
-    case CREATE_RAW_MEMORY_TYPE_QUESTION_ID:
-      ConfigPrivate->ConfigStore.MemType = Value->u8;
-      break;
-
-    case CREATE_RAW_SUBMIT_QUESTION_ID:
-      //
-      // Create raw, FileHandle is NULL.
-      //
-      Status = HiiCreateRamDisk (
-                 ConfigPrivate->ConfigStore.Size,
-                 NULL,
-                 ConfigPrivate->ConfigStore.MemType
-                 );
-      if (EFI_ERROR (Status)) {
+        *ActionRequest = EFI_BROWSER_ACTION_REQUEST_FORM_DISCARD_EXIT;
         break;
-      }
 
-      //
-      // Refresh the registered RAM disks list.
-      //
-      UpdateMainForm (ConfigPrivate);
+      case CREATE_RAW_DISCARD_QUESTION_ID:
+        *ActionRequest = EFI_BROWSER_ACTION_REQUEST_FORM_DISCARD_EXIT;
+        break;
 
-      *ActionRequest = EFI_BROWSER_ACTION_REQUEST_FORM_DISCARD_EXIT;
-      break;
-
-    case CREATE_RAW_DISCARD_QUESTION_ID:
-      *ActionRequest = EFI_BROWSER_ACTION_REQUEST_FORM_DISCARD_EXIT;
-      break;
-
-    default:
-      //
-      // QuestionIds for checkboxes
-      //
-      if ((QuestionId >= MAIN_CHECKBOX_QUESTION_ID_START) &&
-          (QuestionId < CREATE_RAW_RAM_DISK_FORM_ID)) {
-        EFI_LIST_FOR_EACH (Entry, &RegisteredRamDisks) {
-          PrivateData = RAM_DISK_PRIVATE_FROM_THIS (Entry);
-          if (PrivateData->CheckBoxId == QuestionId) {
-            PrivateData->CheckBoxChecked = (BOOLEAN) (Value->u8 != 0);
+      default:
+        //
+        // QuestionIds for checkboxes
+        //
+        if ((QuestionId >= MAIN_CHECKBOX_QUESTION_ID_START) &&
+            (QuestionId < CREATE_RAW_RAM_DISK_FORM_ID))
+        {
+          BASE_LIST_FOR_EACH (Entry, &RegisteredRamDisks) {
+            PrivateData = RAM_DISK_PRIVATE_FROM_THIS (Entry);
+            if (PrivateData->CheckBoxId == QuestionId) {
+              PrivateData->CheckBoxChecked = (BOOLEAN)(Value->u8 != 0);
+            }
           }
         }
-      }
-      break;
+
+        break;
     }
   }
 
