@@ -2,7 +2,7 @@
 Utility functions for UI presentation.
 
 Copyright (c) 2004 - 2018, Intel Corporation. All rights reserved.<BR>
-(C) Copyright 2015 Hewlett Packard Enterprise Development LP<BR>
+(C) Copyright 2015 - 2022 Hewlett Packard Enterprise Development LP<BR>
 SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
@@ -21,7 +21,6 @@ LIST_ENTRY                mRefreshEventList  = INITIALIZE_LIST_HEAD_VARIABLE (mR
 UINT16                    mCurFakeQestId;
 FORM_DISPLAY_ENGINE_FORM  gDisplayFormData;
 BOOLEAN                   mFinishRetrieveCall = FALSE;
-BOOLEAN                   mDynamicFormUpdated = FALSE;
 
 /**
   Check whether the ConfigAccess protocol is available.
@@ -1005,7 +1004,7 @@ UpdateStatementStatus (
   @param Action                  The user input action request info.
   @param DefaultId               The user input default Id info.
 
-  @retval EFI_SUCESSS            This function always return successfully for now.
+  @retval EFI_SUCCESS            This function always return successfully for now.
 
 **/
 EFI_STATUS
@@ -1558,7 +1557,7 @@ ProcessQuestionConfig (
 
   @param UserInput               The user input data.
 
-  @retval EFI_SUCESSS            This function always return successfully for now.
+  @retval EFI_SUCCESS            This function always return successfully for now.
 
 **/
 EFI_STATUS
@@ -1688,7 +1687,7 @@ ProcessUserInput (
 
   Display form and wait for user to select one menu option, then return it.
 
-  @retval EFI_SUCESSS            This function always return successfully for now.
+  @retval EFI_SUCCESS            This function always return successfully for now.
 
 **/
 EFI_STATUS
@@ -1791,7 +1790,6 @@ FormUpdateNotify (
   )
 {
   mHiiPackageListUpdated = TRUE;
-  mDynamicFormUpdated    = TRUE;
 
   return EFI_SUCCESS;
 }
@@ -2147,6 +2145,15 @@ ProcessCallBackFunction (
               gCallbackReconnect = TRUE;
               break;
 
+            case EFI_BROWSER_ACTION_REQUEST_QUESTION_APPLY:
+              Status = SetQuestionValue (
+                         gCurrentSelection->FormSet,
+                         gCurrentSelection->Form,
+                         Statement,
+                         GetSetValueWithHiiDriver
+                         );
+              break;
+
             default:
               break;
           }
