@@ -1,9 +1,10 @@
 /** @file
   ACPI 6.1 definitions from the ACPI Specification Revision 6.1 January, 2016.
 
-  Copyright (c) 2016 - 2018, Intel Corporation. All rights reserved.<BR>
+  Copyright (c) 2016 - 2022, Intel Corporation. All rights reserved.<BR>
  (C) Copyright 2016 Hewlett Packard Enterprise Development LP<BR>
   Copyright (c) 2020, ARM Ltd. All rights reserved.<BR>
+  Copyright (C) 2025, Advanced Micro Devices, Inc. All rights reserved.
   SPDX-License-Identifier: BSD-2-Clause-Patent
 **/
 
@@ -12,6 +13,26 @@
 
 #include <IndustryStandard/Acpi60.h>
 
+///
+/// _CSD Revision for ACPI 6.1
+///
+#define EFI_ACPI_6_1_AML_CSD_REVISION  0
+
+///
+/// _CSD NumEntries for ACPI 6.1
+///
+#define EFI_ACPI_6_1_AML_CSD_NUM_ENTRIES  6
+
+///
+/// _PSD Revision for ACPI 6.1
+///
+#define EFI_ACPI_6_1_AML_PSD_REVISION  0
+
+///
+/// _CPC Revision for ACPI 6.1
+///
+#define EFI_ACPI_6_1_AML_CPC_REVISION  2
+
 //
 // Ensure proper structure formats
 //
@@ -1566,10 +1587,16 @@ typedef struct {
 //
 // Boot Error Severity types
 //
-#define EFI_ACPI_6_1_ERROR_SEVERITY_CORRECTABLE  0x00
+#define EFI_ACPI_6_1_ERROR_SEVERITY_RECOVERABLE  0x00
 #define EFI_ACPI_6_1_ERROR_SEVERITY_FATAL        0x01
 #define EFI_ACPI_6_1_ERROR_SEVERITY_CORRECTED    0x02
 #define EFI_ACPI_6_1_ERROR_SEVERITY_NONE         0x03
+//
+// The term 'Correctable' is no longer being used as an error severity of the
+// reported error since ACPI Specification Version 5.1 Errata B.
+// The below macro is considered as deprecated and should no longer be used.
+//
+#define EFI_ACPI_6_1_ERROR_SEVERITY_CORRECTABLE  0x00
 
 ///
 /// Generic Error Data Entry Definition
@@ -1963,6 +1990,8 @@ typedef struct {
 #define EFI_ACPI_6_1_EINJ_EXECUTE_OPERATION               0x05
 #define EFI_ACPI_6_1_EINJ_CHECK_BUSY_STATUS               0x06
 #define EFI_ACPI_6_1_EINJ_GET_COMMAND_STATUS              0x07
+#define EFI_ACPI_6_1_EINJ_SET_ERROR_TYPE_WITH_ADDRESS     0x08
+#define EFI_ACPI_6_1_EINJ_GET_EXECUTE_OPERATION_TIMINGS   0x09
 #define EFI_ACPI_6_1_EINJ_TRIGGER_ERROR                   0xFF
 
 ///
