@@ -16,12 +16,13 @@
 #include <Library/PcdLib.h>
 #include <Library/BaseMemoryLib.h>
 #include <Library/DebugPrintErrorLevelLib.h>
+#include <Library/MemDebugLogLib.h>
 #include "DebugLibDetect.h"
 
 //
 // Define the maximum debug and assert message length that this library supports
 //
-#define MAX_DEBUG_MESSAGE_LENGTH  0x100
+#define MAX_DEBUG_MESSAGE_LENGTH  0x200
 
 //
 // VA_LIST can not initialize to NULL for all compiler, so we use this to
@@ -93,10 +94,11 @@ DebugPrintMarker (
   ASSERT (Format != NULL);
 
   //
-  // Check if the global mask disables this message or the device is inactive
+  // If the global mask disables this message OR the debug I/O port is not
+  // present and Memory Debug Logging is disabled, there's nothing to do.
   //
   if (((ErrorLevel & GetDebugPrintErrorLevel ()) == 0) ||
-      !PlatformDebugLibIoPortFound ())
+      (!PlatformDebugLibIoPortFound () && !MemDebugLogEnabled ()))
   {
     return;
   }
@@ -111,9 +113,18 @@ DebugPrintMarker (
   }
 
   //
-  // Send the print string to the debug I/O port
+  // Send the print string to the debug I/O port, if present
   //
-  IoWriteFifo8 (PcdGet16 (PcdDebugIoPort), Length, Buffer);
+  if (PlatformDebugLibIoPortFound ()) {
+    IoWriteFifo8 (PcdGet16 (PcdDebugIoPort), Length, Buffer);
+  }
+
+  //
+  // Send string to Memory Debug Log if enabled
+  //
+  if (MemDebugLogEnabled ()) {
+    MemDebugLogWrite (Buffer, Length);
+  }
 }
 
 /**
@@ -221,6 +232,13 @@ DebugAssert (
     IoWriteFifo8 (PcdGet16 (PcdDebugIoPort), Length, Buffer);
   }
 
+  //
+  // Send the string to Memory Debug Log
+  //
+  if (MemDebugLogEnabled ()) {
+    MemDebugLogWrite (Buffer, Length);
+  }
+
   //
   // Generate a Breakpoint, DeadLoop, or NOP based on PCD settings
   //
@@ -259,7 +277,7 @@ DebugClearMemory (
   ASSERT (Buffer != NULL);
 
   //
-  // SetMem() checks for the the ASSERT() condition on Length and returns Buffer
+  // SetMem() checks for the ASSERT() condition on Length and returns Buffer
   //
   return SetMem (Buffer, Length, PcdGet8 (PcdDebugClearMemoryValue));
 }
