@@ -3,19 +3,12 @@
   Driver Binding code and its private helpers for the virtio-net driver.
 
   Copyright (C) 2013, Red Hat, Inc.
-  Copyright (c) 2006 - 2010, Intel Corporation. All rights reserved.<BR>
+  Copyright (c) 2006 - 2014, Intel Corporation. All rights reserved.<BR>
 
-  This program and the accompanying materials are licensed and made available
-  under the terms and conditions of the BSD License which accompanies this
-  distribution. The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS, WITHOUT
-  WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
-#include <IndustryStandard/Pci.h>
 #include <Library/BaseMemoryLib.h>
 #include <Library/DevicePathLib.h>
 #include <Library/MemoryAllocationLib.h>
@@ -23,7 +16,7 @@
 
 #include "VirtioNet.h"
 
-#define RECEIVE_FILTERS_NO_MCAST ((UINT32) (       \
+#define RECEIVE_FILTERS_NO_MCAST  ((UINT32) (      \
           EFI_SIMPLE_NETWORK_RECEIVE_UNICAST     | \
           EFI_SIMPLE_NETWORK_RECEIVE_BROADCAST   | \
           EFI_SIMPLE_NETWORK_RECEIVE_PROMISCUOUS   \
@@ -50,43 +43,43 @@
                                     unused.
 
   @retval EFI_UNSUPPORTED           The host doesn't supply a MAC address.
-  @return                           Status codes from Dev->PciIo->Io.Read(),
-                                    VIRTIO_CFG_READ() and VIRTIO_CFG_WRITE().
+  @return                           Status codes from VirtIo protocol members.
   @retval EFI_SUCCESS               Configuration values retrieved.
 */
 STATIC
 EFI_STATUS
 EFIAPI
 VirtioNetGetFeatures (
-  IN OUT  VNET_DEV        *Dev,
-  OUT     EFI_MAC_ADDRESS *MacAddress,
-  OUT     BOOLEAN         *MediaPresentSupported,
-  OUT     BOOLEAN         *MediaPresent
+  IN OUT  VNET_DEV         *Dev,
+  OUT     EFI_MAC_ADDRESS  *MacAddress,
+  OUT     BOOLEAN          *MediaPresentSupported,
+  OUT     BOOLEAN          *MediaPresent
   )
 {
-  EFI_STATUS Status;
-  UINT8      NextDevStat;
-  UINT32     Features;
-  UINT16     LinkStatus;
+  EFI_STATUS  Status;
+  UINT8       NextDevStat;
+  UINT64      Features;
+  UINTN       MacIdx;
+  UINT16      LinkStatus;
 
   //
   // Interrogate the device for features (virtio-0.9.5, 2.2.1 Device
   // Initialization Sequence), but don't complete setting it up.
   //
   NextDevStat = 0;             // step 1 -- reset device
-  Status = VIRTIO_CFG_WRITE (Dev, Generic.VhdrDeviceStatus, NextDevStat);
+  Status      = Dev->VirtIo->SetDeviceStatus (Dev->VirtIo, NextDevStat);
   if (EFI_ERROR (Status)) {
     return Status;
   }
 
   NextDevStat |= VSTAT_ACK;    // step 2 -- acknowledge device presence
-  Status = VIRTIO_CFG_WRITE (Dev, Generic.VhdrDeviceStatus, NextDevStat);
+  Status       = Dev->VirtIo->SetDeviceStatus (Dev->VirtIo, NextDevStat);
   if (EFI_ERROR (Status)) {
     goto YieldDevice;
   }
 
   NextDevStat |= VSTAT_DRIVER; // step 3 -- we know how to drive it
-  Status = VIRTIO_CFG_WRITE (Dev, Generic.VhdrDeviceStatus, NextDevStat);
+  Status       = Dev->VirtIo->SetDeviceStatus (Dev->VirtIo, NextDevStat);
   if (EFI_ERROR (Status)) {
     goto YieldDevice;
   }
@@ -94,7 +87,7 @@ VirtioNetGetFeatures (
   //
   // step 4a -- retrieve and validate features
   //
-  Status = VIRTIO_CFG_READ (Dev, Generic.VhdrDeviceFeatureBits, &Features);
+  Status = Dev->VirtIo->GetDeviceFeatures (Dev->VirtIo, &Features);
   if (EFI_ERROR (Status)) {
     goto YieldDevice;
   }
@@ -106,16 +99,18 @@ VirtioNetGetFeatures (
     Status = EFI_UNSUPPORTED;
     goto YieldDevice;
   }
-  Status = Dev->PciIo->Io.Read (Dev->PciIo,           // PciIo
-                            EfiPciIoWidthUint8,       // Width
-                            PCI_BAR_IDX0,             // BarIndex
-                            OFFSET_OF_VNET (VhdrMac), // Offset
-                            SIZE_OF_VNET (VhdrMac),   // Count
-                            MacAddress                // Buffer
-                            );
 
-  if (EFI_ERROR (Status)) {
-    goto YieldDevice;
+  for (MacIdx = 0; MacIdx < SIZE_OF_VNET (Mac); ++MacIdx) {
+    Status = Dev->VirtIo->ReadDevice (
+                            Dev->VirtIo,
+                            OFFSET_OF_VNET (Mac) + MacIdx, // Offset
+                            1,                             // FieldSize
+                            1,                             // BufferSize
+                            &MacAddress->Addr[MacIdx]      // Buffer
+                            );
+    if (EFI_ERROR (Status)) {
+      goto YieldDevice;
+    }
   }
 
   //
@@ -123,24 +118,25 @@ VirtioNetGetFeatures (
   //
   if ((Features & VIRTIO_NET_F_STATUS) == 0) {
     *MediaPresentSupported = FALSE;
-  }
-  else {
+  } else {
     *MediaPresentSupported = TRUE;
-    Status = VIRTIO_CFG_READ (Dev, VhdrLinkStatus, &LinkStatus);
+    Status                 = VIRTIO_CFG_READ (Dev, LinkStatus, &LinkStatus);
     if (EFI_ERROR (Status)) {
       goto YieldDevice;
     }
-    *MediaPresent = !!(LinkStatus & VIRTIO_NET_S_LINK_UP);
+
+    *MediaPresent = (BOOLEAN)((LinkStatus & VIRTIO_NET_S_LINK_UP) != 0);
   }
 
 YieldDevice:
-  VIRTIO_CFG_WRITE (Dev, Generic.VhdrDeviceStatus,
-    EFI_ERROR (Status) ? VSTAT_FAILED : 0);
+  Dev->VirtIo->SetDeviceStatus (
+                 Dev->VirtIo,
+                 EFI_ERROR (Status) ? VSTAT_FAILED : 0
+                 );
 
   return Status;
 }
 
-
 /**
   Set up the Simple Network Protocol fields, the Simple Network Mode fields,
   and the Exit Boot Services Event of the virtio-net driver instance.
@@ -158,10 +154,10 @@ STATIC
 EFI_STATUS
 EFIAPI
 VirtioNetSnpPopulate (
-  IN OUT VNET_DEV *Dev
+  IN OUT VNET_DEV  *Dev
   )
 {
-  EFI_STATUS Status;
+  EFI_STATUS  Status;
 
   //
   // We set up a function here that is asynchronously callable by an
@@ -184,8 +180,13 @@ VirtioNetSnpPopulate (
   // I/O)". Because none of our functions block, we'd satisfy an even stronger
   // requirement.
   //
-  Status = gBS->CreateEvent (EVT_NOTIFY_WAIT, TPL_CALLBACK,
-                  &VirtioNetIsPacketAvailable, Dev, &Dev->Snp.WaitForPacket);
+  Status = gBS->CreateEvent (
+                  EVT_NOTIFY_WAIT,
+                  TPL_CALLBACK,
+                  &VirtioNetIsPacketAvailable,
+                  Dev,
+                  &Dev->Snp.WaitForPacket
+                  );
   if (EFI_ERROR (Status)) {
     return Status;
   }
@@ -206,32 +207,40 @@ VirtioNetSnpPopulate (
   Dev->Snp.Receive        = &VirtioNetReceive;
   Dev->Snp.Mode           = &Dev->Snm;
 
-  Dev->Snm.State                 = EfiSimpleNetworkStopped;
-  Dev->Snm.HwAddressSize         = SIZE_OF_VNET (VhdrMac);
-  Dev->Snm.MediaHeaderSize       = SIZE_OF_VNET (VhdrMac) + // dst MAC
-                                   SIZE_OF_VNET (VhdrMac) + // src MAC
-                                   2;                       // Ethertype
-  Dev->Snm.MaxPacketSize         = 1500;
-  Dev->Snm.NvRamSize             = 0;
-  Dev->Snm.NvRamAccessSize       = 0;
-  Dev->Snm.ReceiveFilterMask     = RECEIVE_FILTERS_NO_MCAST;
-  Dev->Snm.ReceiveFilterSetting  = RECEIVE_FILTERS_NO_MCAST;
-  Dev->Snm.MaxMCastFilterCount   = 0;
-  Dev->Snm.MCastFilterCount      = 0;
-  Dev->Snm.IfType                = 1; // ethernet
-  Dev->Snm.MacAddressChangeable  = FALSE;
-  Dev->Snm.MultipleTxSupported   = TRUE;
+  Dev->Snm.State           = EfiSimpleNetworkStopped;
+  Dev->Snm.HwAddressSize   = SIZE_OF_VNET (Mac);
+  Dev->Snm.MediaHeaderSize = SIZE_OF_VNET (Mac) +       // dst MAC
+                             SIZE_OF_VNET (Mac) +       // src MAC
+                             2;                         // Ethertype
+  Dev->Snm.MaxPacketSize        = 1500;
+  Dev->Snm.NvRamSize            = 0;
+  Dev->Snm.NvRamAccessSize      = 0;
+  Dev->Snm.ReceiveFilterMask    = RECEIVE_FILTERS_NO_MCAST;
+  Dev->Snm.ReceiveFilterSetting = RECEIVE_FILTERS_NO_MCAST;
+  Dev->Snm.MaxMCastFilterCount  = 0;
+  Dev->Snm.MCastFilterCount     = 0;
+  Dev->Snm.IfType               = 1;  // ethernet
+  Dev->Snm.MacAddressChangeable = FALSE;
+  Dev->Snm.MultipleTxSupported  = TRUE;
 
-  ASSERT (SIZE_OF_VNET (VhdrMac) <= sizeof (EFI_MAC_ADDRESS));
+  ASSERT (SIZE_OF_VNET (Mac) <= sizeof (EFI_MAC_ADDRESS));
 
-  Status = VirtioNetGetFeatures (Dev, &Dev->Snm.CurrentAddress,
-             &Dev->Snm.MediaPresentSupported, &Dev->Snm.MediaPresent);
+  Status = VirtioNetGetFeatures (
+             Dev,
+             &Dev->Snm.CurrentAddress,
+             &Dev->Snm.MediaPresentSupported,
+             &Dev->Snm.MediaPresent
+             );
   if (EFI_ERROR (Status)) {
     goto CloseWaitForPacket;
   }
-  CopyMem (&Dev->Snm.PermanentAddress, &Dev->Snm.CurrentAddress,
-    SIZE_OF_VNET (VhdrMac));
-  SetMem (&Dev->Snm.BroadcastAddress, SIZE_OF_VNET (VhdrMac), 0xFF);
+
+  CopyMem (
+    &Dev->Snm.PermanentAddress,
+    &Dev->Snm.CurrentAddress,
+    SIZE_OF_VNET (Mac)
+    );
+  SetMem (&Dev->Snm.BroadcastAddress, SIZE_OF_VNET (Mac), 0xFF);
 
   //
   // VirtioNetExitBoot() is queued by ExitBootServices(); its purpose is to
@@ -242,8 +251,13 @@ VirtioNetSnpPopulate (
   // could be entered immediately. VirtioNetExitBoot() checks Dev->Snm.State,
   // so we're safe.
   //
-  Status = gBS->CreateEvent (EVT_SIGNAL_EXIT_BOOT_SERVICES, TPL_CALLBACK,
-                  &VirtioNetExitBoot, Dev, &Dev->ExitBoot);
+  Status = gBS->CreateEvent (
+                  EVT_SIGNAL_EXIT_BOOT_SERVICES,
+                  TPL_CALLBACK,
+                  &VirtioNetExitBoot,
+                  Dev,
+                  &Dev->ExitBoot
+                  );
   if (EFI_ERROR (Status)) {
     goto CloseWaitForPacket;
   }
@@ -255,7 +269,6 @@ CloseWaitForPacket:
   return Status;
 }
 
-
 /**
   Release any resources allocated by VirtioNetSnpPopulate().
 
@@ -270,7 +283,7 @@ STATIC
 VOID
 EFIAPI
 VirtioNetSnpEvacuate (
-  IN OUT VNET_DEV *Dev
+  IN OUT VNET_DEV  *Dev
   )
 {
   //
@@ -283,7 +296,6 @@ VirtioNetSnpEvacuate (
   gBS->CloseEvent (Dev->Snp.WaitForPacket);
 }
 
-
 /**
   Tests to see if this driver supports a given controller. If a child device is
   provided, it further tests to see if this driver supports creating a handle
@@ -337,47 +349,54 @@ VirtioNetSnpEvacuate (
                                    RemainingDevicePath is not supported by the
                                    driver specified by This.
 **/
-
 STATIC
 EFI_STATUS
 EFIAPI
 VirtioNetDriverBindingSupported (
-  IN EFI_DRIVER_BINDING_PROTOCOL *This,
-  IN EFI_HANDLE                  DeviceHandle,
-  IN EFI_DEVICE_PATH_PROTOCOL    *RemainingDevicePath
+  IN EFI_DRIVER_BINDING_PROTOCOL  *This,
+  IN EFI_HANDLE                   DeviceHandle,
+  IN EFI_DEVICE_PATH_PROTOCOL     *RemainingDevicePath
   )
 {
-  EFI_STATUS          Status;
-  EFI_PCI_IO_PROTOCOL *PciIo;
-  PCI_TYPE00          Pci;
+  EFI_STATUS              Status;
+  VIRTIO_DEVICE_PROTOCOL  *VirtIo;
 
-  Status = gBS->OpenProtocol (DeviceHandle, &gEfiPciIoProtocolGuid,
-                  (VOID **)&PciIo, This->DriverBindingHandle, DeviceHandle,
-                  EFI_OPEN_PROTOCOL_BY_DRIVER);
+  //
+  // Attempt to open the device with the VirtIo set of interfaces. On success,
+  // the protocol is "instantiated" for the VirtIo device. Covers duplicate open
+  // attempts (EFI_ALREADY_STARTED).
+  //
+  Status = gBS->OpenProtocol (
+                  DeviceHandle,               // candidate device
+                  &gVirtioDeviceProtocolGuid, // for generic VirtIo access
+                  (VOID **)&VirtIo,           // handle to instantiate
+                  This->DriverBindingHandle,  // requestor driver identity
+                  DeviceHandle,               // ControllerHandle, according to
+                                              // the UEFI Driver Model
+                  EFI_OPEN_PROTOCOL_BY_DRIVER // get exclusive VirtIo access to
+                                              // the device; to be released
+                  );
   if (EFI_ERROR (Status)) {
     return Status;
   }
 
-  Status = PciIo->Pci.Read (PciIo, EfiPciIoWidthUint32, 0,
-                        sizeof Pci / sizeof (UINT32), &Pci);
-
-  //
-  // virtio-0.9.5, 2.1 PCI Discovery:
-  // the network device has Subsystem Device ID 1
-  //
-  if (Status == EFI_SUCCESS) {
-    Status = (Pci.Hdr.VendorId == 0x1AF4 &&
-              Pci.Hdr.DeviceId >= 0x1000 && Pci.Hdr.DeviceId <= 0x103F &&
-              Pci.Hdr.RevisionID == 0x00 &&
-              Pci.Device.SubsystemID == VIRTIO_SUBSYSTEM_NETWORK_CARD) ? EFI_SUCCESS : EFI_UNSUPPORTED;
+  if (VirtIo->SubSystemDeviceId != VIRTIO_SUBSYSTEM_NETWORK_CARD) {
+    Status = EFI_UNSUPPORTED;
   }
 
-  gBS->CloseProtocol (DeviceHandle, &gEfiPciIoProtocolGuid,
-         This->DriverBindingHandle, DeviceHandle);
+  //
+  // We needed VirtIo access only transitorily, to see whether we support the
+  // device or not.
+  //
+  gBS->CloseProtocol (
+         DeviceHandle,
+         &gVirtioDeviceProtocolGuid,
+         This->DriverBindingHandle,
+         DeviceHandle
+         );
   return Status;
 }
 
-
 /**
   Starts a device controller or a bus controller.
 
@@ -421,75 +440,66 @@ VirtioNetDriverBindingSupported (
                                    device error.Currently not implemented.
   @retval EFI_OUT_OF_RESOURCES     The request could not be completed due to a
                                    lack of resources.
-  @retval Others                   The driver failded to start the device.
+  @retval Others                   The driver failed to start the device.
 
 **/
-
 STATIC
 EFI_STATUS
 EFIAPI
 VirtioNetDriverBindingStart (
-  IN EFI_DRIVER_BINDING_PROTOCOL *This,
-  IN EFI_HANDLE                  DeviceHandle,
-  IN EFI_DEVICE_PATH_PROTOCOL    *RemainingDevicePath
+  IN EFI_DRIVER_BINDING_PROTOCOL  *This,
+  IN EFI_HANDLE                   DeviceHandle,
+  IN EFI_DEVICE_PATH_PROTOCOL     *RemainingDevicePath
   )
 {
-  EFI_STATUS               Status;
-  VNET_DEV                 *Dev;
-  EFI_DEVICE_PATH_PROTOCOL *DevicePath;
-  MAC_ADDR_DEVICE_PATH     MacNode;
-  VOID                     *ChildPciIo;
+  EFI_STATUS                Status;
+  VNET_DEV                  *Dev;
+  EFI_DEVICE_PATH_PROTOCOL  *DevicePath;
+  MAC_ADDR_DEVICE_PATH      MacNode;
+  VOID                      *ChildVirtIo;
 
   //
   // allocate space for the driver instance
   //
-  Dev = (VNET_DEV *) AllocateZeroPool (sizeof *Dev);
+  Dev = (VNET_DEV *)AllocateZeroPool (sizeof *Dev);
   if (Dev == NULL) {
     return EFI_OUT_OF_RESOURCES;
   }
+
   Dev->Signature = VNET_SIG;
 
-  //
-  // get PCI access to the device and keep it open
-  //
-  Status = gBS->OpenProtocol (DeviceHandle, &gEfiPciIoProtocolGuid,
-                  (VOID **)&Dev->PciIo, This->DriverBindingHandle,
-                  DeviceHandle, EFI_OPEN_PROTOCOL_BY_DRIVER);
+  Status = gBS->OpenProtocol (
+                  DeviceHandle,
+                  &gVirtioDeviceProtocolGuid,
+                  (VOID **)&Dev->VirtIo,
+                  This->DriverBindingHandle,
+                  DeviceHandle,
+                  EFI_OPEN_PROTOCOL_BY_DRIVER
+                  );
   if (EFI_ERROR (Status)) {
     goto FreeVirtioNet;
   }
 
-  //
-  // save original PCI attributes and enable IO space access
-  //
-  Status = Dev->PciIo->Attributes (Dev->PciIo, EfiPciIoAttributeOperationGet,
-                         0, &Dev->OrigPciAttributes);
-  if (EFI_ERROR (Status)) {
-    goto ClosePciIo;
-  }
-
-  Status = Dev->PciIo->Attributes (Dev->PciIo,
-                         EfiPciIoAttributeOperationEnable,
-                         EFI_PCI_IO_ATTRIBUTE_IO, NULL);
-  if (EFI_ERROR (Status)) {
-    goto ClosePciIo;
-  }
-
   //
   // now we can run a basic one-shot virtio-net initialization required to
   // retrieve the MAC address
   //
   Status = VirtioNetSnpPopulate (Dev);
   if (EFI_ERROR (Status)) {
-    goto RestorePciAttributes;
+    goto CloseVirtIo;
   }
 
   //
-  // get the device path of the virtio-net PCI device -- one-shot open
+  // get the device path of the virtio-net device -- one-shot open
   //
-  Status = gBS->OpenProtocol (DeviceHandle, &gEfiDevicePathProtocolGuid,
-                  (VOID **)&DevicePath, This->DriverBindingHandle,
-                  DeviceHandle, EFI_OPEN_PROTOCOL_GET_PROTOCOL);
+  Status = gBS->OpenProtocol (
+                  DeviceHandle,
+                  &gEfiDevicePathProtocolGuid,
+                  (VOID **)&DevicePath,
+                  This->DriverBindingHandle,
+                  DeviceHandle,
+                  EFI_OPEN_PROTOCOL_GET_PROTOCOL
+                  );
   if (EFI_ERROR (Status)) {
     goto Evacuate;
   }
@@ -500,9 +510,12 @@ VirtioNetDriverBindingStart (
   MacNode.Header.Type    = MESSAGING_DEVICE_PATH;
   MacNode.Header.SubType = MSG_MAC_ADDR_DP;
   SetDevicePathNodeLength (&MacNode, sizeof MacNode);
-  CopyMem (&MacNode.MacAddress, &Dev->Snm.CurrentAddress,
-    sizeof (EFI_MAC_ADDRESS));
-  MacNode.IfType         = Dev->Snm.IfType;
+  CopyMem (
+    &MacNode.MacAddress,
+    &Dev->Snm.CurrentAddress,
+    sizeof (EFI_MAC_ADDRESS)
+    );
+  MacNode.IfType = Dev->Snm.IfType;
 
   Dev->MacDevicePath = AppendDevicePathNode (DevicePath, &MacNode.Header);
   if (Dev->MacDevicePath == NULL) {
@@ -514,21 +527,30 @@ VirtioNetDriverBindingStart (
   // create a child handle with the Simple Network Protocol and the new
   // device path installed on it
   //
-  Status = gBS->InstallMultipleProtocolInterfaces (&Dev->MacHandle,
-                  &gEfiSimpleNetworkProtocolGuid, &Dev->Snp,
-                  &gEfiDevicePathProtocolGuid,    Dev->MacDevicePath,
-                  NULL);
+  Status = gBS->InstallMultipleProtocolInterfaces (
+                  &Dev->MacHandle,
+                  &gEfiSimpleNetworkProtocolGuid,
+                  &Dev->Snp,
+                  &gEfiDevicePathProtocolGuid,
+                  Dev->MacDevicePath,
+                  NULL
+                  );
   if (EFI_ERROR (Status)) {
     goto FreeMacDevicePath;
   }
 
   //
-  // make a note that we keep this device open with PciIo for the sake of this
+  // make a note that we keep this device open with VirtIo for the sake of this
   // child
   //
-  Status = gBS->OpenProtocol (DeviceHandle, &gEfiPciIoProtocolGuid,
-                  &ChildPciIo, This->DriverBindingHandle,
-                  Dev->MacHandle, EFI_OPEN_PROTOCOL_BY_CHILD_CONTROLLER);
+  Status = gBS->OpenProtocol (
+                  DeviceHandle,
+                  &gVirtioDeviceProtocolGuid,
+                  &ChildVirtIo,
+                  This->DriverBindingHandle,
+                  Dev->MacHandle,
+                  EFI_OPEN_PROTOCOL_BY_CHILD_CONTROLLER
+                  );
   if (EFI_ERROR (Status)) {
     goto UninstallMultiple;
   }
@@ -536,10 +558,14 @@ VirtioNetDriverBindingStart (
   return EFI_SUCCESS;
 
 UninstallMultiple:
-  gBS->UninstallMultipleProtocolInterfaces (Dev->MacHandle,
-         &gEfiDevicePathProtocolGuid,    Dev->MacDevicePath,
-         &gEfiSimpleNetworkProtocolGuid, &Dev->Snp,
-         NULL);
+  gBS->UninstallMultipleProtocolInterfaces (
+         Dev->MacHandle,
+         &gEfiDevicePathProtocolGuid,
+         Dev->MacDevicePath,
+         &gEfiSimpleNetworkProtocolGuid,
+         &Dev->Snp,
+         NULL
+         );
 
 FreeMacDevicePath:
   FreePool (Dev->MacDevicePath);
@@ -547,13 +573,13 @@ FreeMacDevicePath:
 Evacuate:
   VirtioNetSnpEvacuate (Dev);
 
-RestorePciAttributes:
-  Dev->PciIo->Attributes (Dev->PciIo, EfiPciIoAttributeOperationSet,
-                Dev->OrigPciAttributes, NULL);
-
-ClosePciIo:
-  gBS->CloseProtocol (DeviceHandle, &gEfiPciIoProtocolGuid,
-         This->DriverBindingHandle, DeviceHandle);
+CloseVirtIo:
+  gBS->CloseProtocol (
+         DeviceHandle,
+         &gVirtioDeviceProtocolGuid,
+         This->DriverBindingHandle,
+         DeviceHandle
+         );
 
 FreeVirtioNet:
   FreePool (Dev);
@@ -561,7 +587,6 @@ FreeVirtioNet:
   return Status;
 }
 
-
 /**
   Stops a device controller or a bus controller.
 
@@ -599,10 +624,10 @@ STATIC
 EFI_STATUS
 EFIAPI
 VirtioNetDriverBindingStop (
-  IN EFI_DRIVER_BINDING_PROTOCOL *This,
-  IN EFI_HANDLE                  DeviceHandle,
-  IN UINTN                       NumberOfChildren,
-  IN EFI_HANDLE                  *ChildHandleBuffer
+  IN EFI_DRIVER_BINDING_PROTOCOL  *This,
+  IN EFI_HANDLE                   DeviceHandle,
+  IN UINTN                        NumberOfChildren,
+  IN EFI_HANDLE                   *ChildHandleBuffer
   )
 {
   if (NumberOfChildren > 0) {
@@ -610,17 +635,21 @@ VirtioNetDriverBindingStop (
     // free all resources for whose access we need the child handle, because
     // the child handle is going away
     //
-    EFI_STATUS                  Status;
-    EFI_SIMPLE_NETWORK_PROTOCOL *Snp;
-    VNET_DEV                    *Dev;
-    EFI_TPL                     OldTpl;
+    EFI_STATUS                   Status;
+    EFI_SIMPLE_NETWORK_PROTOCOL  *Snp;
+    VNET_DEV                     *Dev;
+    EFI_TPL                      OldTpl;
 
     ASSERT (NumberOfChildren == 1);
 
-    Status = gBS->OpenProtocol (ChildHandleBuffer[0],
-                    &gEfiSimpleNetworkProtocolGuid, (VOID **)&Snp,
-                    This->DriverBindingHandle, DeviceHandle,
-                    EFI_OPEN_PROTOCOL_GET_PROTOCOL);
+    Status = gBS->OpenProtocol (
+                    ChildHandleBuffer[0],
+                    &gEfiSimpleNetworkProtocolGuid,
+                    (VOID **)&Snp,
+                    This->DriverBindingHandle,
+                    DeviceHandle,
+                    EFI_OPEN_PROTOCOL_GET_PROTOCOL
+                    );
     ASSERT_EFI_ERROR (Status);
     Dev = VIRTIO_NET_FROM_SNP (Snp);
 
@@ -635,18 +664,23 @@ VirtioNetDriverBindingStop (
       // device in use, cannot stop driver instance
       //
       Status = EFI_DEVICE_ERROR;
-    }
-    else {
-      gBS->CloseProtocol (DeviceHandle, &gEfiPciIoProtocolGuid,
-             This->DriverBindingHandle, Dev->MacHandle);
-      gBS->UninstallMultipleProtocolInterfaces (Dev->MacHandle,
-             &gEfiDevicePathProtocolGuid,    Dev->MacDevicePath,
-             &gEfiSimpleNetworkProtocolGuid, &Dev->Snp,
-             NULL);
+    } else {
+      gBS->CloseProtocol (
+             DeviceHandle,
+             &gVirtioDeviceProtocolGuid,
+             This->DriverBindingHandle,
+             Dev->MacHandle
+             );
+      gBS->UninstallMultipleProtocolInterfaces (
+             Dev->MacHandle,
+             &gEfiDevicePathProtocolGuid,
+             Dev->MacDevicePath,
+             &gEfiSimpleNetworkProtocolGuid,
+             &Dev->Snp,
+             NULL
+             );
       FreePool (Dev->MacDevicePath);
       VirtioNetSnpEvacuate (Dev);
-      Dev->PciIo->Attributes (Dev->PciIo, EfiPciIoAttributeOperationSet,
-                    Dev->OrigPciAttributes, NULL);
       FreePool (Dev);
     }
 
@@ -657,14 +691,17 @@ VirtioNetDriverBindingStop (
   //
   // release remaining resources, tied directly to the parent handle
   //
-  gBS->CloseProtocol (DeviceHandle, &gEfiPciIoProtocolGuid,
-         This->DriverBindingHandle, DeviceHandle);
+  gBS->CloseProtocol (
+         DeviceHandle,
+         &gVirtioDeviceProtocolGuid,
+         This->DriverBindingHandle,
+         DeviceHandle
+         );
 
   return EFI_SUCCESS;
 }
 
-
-EFI_DRIVER_BINDING_PROTOCOL gVirtioNetDriverBinding = {
+EFI_DRIVER_BINDING_PROTOCOL  gVirtioNetDriverBinding = {
   &VirtioNetDriverBindingSupported,
   &VirtioNetDriverBindingStart,
   &VirtioNetDriverBindingStop,
