@@ -10,7 +10,10 @@ SPDX-License-Identifier: BSD-2-Clause-Patent
 #ifndef LIBSPDM_STDBOOL_ALT_H
 #define LIBSPDM_STDBOOL_ALT_H
 
+// In C23, bool is a built-in type
+#if __STDC_VERSION__ < 202311L
 typedef BOOLEAN bool;
+#endif
 
 #ifndef true
 #define true  TRUE
