@@ -190,6 +190,12 @@ ShellCommandRunDevices (
       Lang = ShellCommandLineGetValue (Package, L"-l");
       if (Lang != NULL) {
         Language = AllocateZeroPool (StrSize (Lang));
+        if (Language == NULL) {
+          ShellPrintHiiEx (-1, -1, NULL, STRING_TOKEN (STR_GEN_OUT_MEM), gShellDriver1HiiHandle, L"devices");
+          ShellCommandLineFreeVarList (Package);
+          return (SHELL_OUT_OF_RESOURCES);
+        }
+
         AsciiSPrint (Language, StrSize (Lang), "%S", Lang);
       } else if (!ShellCommandLineGetFlag (Package, L"-l")) {
         ASSERT (Language == NULL);
