@@ -47,9 +47,12 @@
 #include <Library/PeCoffGetEntryPointLib.h>
 #include <Library/StandaloneMmMemLib.h>
 #include <Library/HobLib.h>
+#include <Library/PerformanceLib.h>
 
 #include "StandaloneMmCorePrivateData.h"
 
+extern EFI_MM_SYSTEM_TABLE  *mMemoryAllocationMmst;
+
 //
 // Used to build a table of MMI Handlers that the MM Core registers
 //
