@@ -4,28 +4,20 @@
 
   Copyright (c) 2009 - 2014, Intel Corporation. All rights reserved.<BR>
   Copyright (c) 2016 - 2018, ARM Limited. All rights reserved.<BR>
-  This program and the accompanying materials are licensed and made available
-  under the terms and conditions of the BSD License which accompanies this
-  distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
 #ifndef _STANDALONE_MM_CORE_PRIVATE_DATA_H_
 #define _STANDALONE_MM_CORE_PRIVATE_DATA_H_
 
-#include <Guid/MmCoreData.h>
-
 //
 // Page management
 //
 
 typedef struct {
-  LIST_ENTRY  Link;
-  UINTN       NumberOfPages;
+  LIST_ENTRY    Link;
+  UINTN         NumberOfPages;
 } FREE_PAGE_LIST;
 
 extern LIST_ENTRY  mMmMemoryMap;
@@ -52,13 +44,13 @@ extern LIST_ENTRY  mMmMemoryMap;
 #define MAX_POOL_INDEX  (MAX_POOL_SHIFT - MIN_POOL_SHIFT + 1)
 
 typedef struct {
-  UINTN        Size;
-  BOOLEAN      Available;
+  UINTN      Size;
+  BOOLEAN    Available;
 } POOL_HEADER;
 
 typedef struct {
-  POOL_HEADER  Header;
-  LIST_ENTRY   Link;
+  POOL_HEADER    Header;
+  LIST_ENTRY     Link;
 } FREE_POOL_HEADER;
 
 extern LIST_ENTRY  mMmPoolLists[MAX_POOL_INDEX];
