@@ -1,19 +1,17 @@
 /** @file
   CPU Register Table Library definitions.
 
-  Copyright (c) 2017, Intel Corporation. All rights reserved.<BR>
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  Copyright (c) 2017 - 2020, Intel Corporation. All rights reserved.<BR>
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
 #ifndef _REGISTER_CPU_FEATURES_H_
 #define _REGISTER_CPU_FEATURES_H_
+#include <PiPei.h>
+#include <PiDxe.h>
+#include <Ppi/MpServices2.h>
+#include <Protocol/MpService.h>
 
 #include <Library/BaseLib.h>
 #include <Library/DebugLib.h>
@@ -29,62 +27,66 @@
 
 #define CPU_FEATURE_ENTRY_SIGNATURE  SIGNATURE_32 ('C', 'F', 'E', 'S')
 
-#define CPU_FEATURE_NAME_SIZE        128
+#define CPU_FEATURE_NAME_SIZE  128
 
 typedef struct {
-  REGISTER_CPU_FEATURE_INFORMATION     CpuInfo;
-  UINT8                                *FeaturesSupportedMask;
-  LIST_ENTRY                           OrderList;
+  REGISTER_CPU_FEATURE_INFORMATION    CpuInfo;
+  UINT8                               *FeaturesSupportedMask;
+  LIST_ENTRY                          OrderList;
 } CPU_FEATURES_INIT_ORDER;
 
 typedef struct {
-  UINT32                       Signature;
-  LIST_ENTRY                   Link;
-  UINT8                        *FeatureMask;
-  CHAR8                        *FeatureName;
-  CPU_FEATURE_GET_CONFIG_DATA  GetConfigDataFunc;
-  CPU_FEATURE_SUPPORT          SupportFunc;
-  CPU_FEATURE_INITIALIZE       InitializeFunc;
-  UINT8                        *BeforeFeatureBitMask;
-  UINT8                        *AfterFeatureBitMask;
-  UINT8                        *CoreBeforeFeatureBitMask;
-  UINT8                        *CoreAfterFeatureBitMask;
-  UINT8                        *PackageBeforeFeatureBitMask;
-  UINT8                        *PackageAfterFeatureBitMask;
-  VOID                         *ConfigData;
-  BOOLEAN                      BeforeAll;
-  BOOLEAN                      AfterAll;
+  UINT32                         Signature;
+  LIST_ENTRY                     Link;
+  UINT8                          *FeatureMask;
+  CHAR8                          *FeatureName;
+  CPU_FEATURE_GET_CONFIG_DATA    GetConfigDataFunc;
+  CPU_FEATURE_SUPPORT            SupportFunc;
+  CPU_FEATURE_INITIALIZE         InitializeFunc;
+  UINT8                          *ThreadBeforeFeatureBitMask;
+  UINT8                          *ThreadAfterFeatureBitMask;
+  UINT8                          *CoreBeforeFeatureBitMask;
+  UINT8                          *CoreAfterFeatureBitMask;
+  UINT8                          *PackageBeforeFeatureBitMask;
+  UINT8                          *PackageAfterFeatureBitMask;
+  VOID                           *ConfigData;
+  BOOLEAN                        BeforeAll;
+  BOOLEAN                        AfterAll;
 } CPU_FEATURES_ENTRY;
 
 //
 // Flags used when program the register.
 //
 typedef struct {
-  volatile UINTN           ConsoleLogLock;          // Spinlock used to control console.
-  volatile UINTN           MemoryMappedLock;        // Spinlock used to program mmio
-  volatile UINT32          *CoreSemaphoreCount;     // Semaphore containers used to program Core semaphore.
-  volatile UINT32          *PackageSemaphoreCount;  // Semaphore containers used to program Package semaphore.
+  volatile UINTN     MemoryMappedLock;              // Spinlock used to program mmio
+  volatile UINT32    *CoreSemaphoreCount;           // Semaphore containers used to program Core semaphore.
+  volatile UINT32    *PackageSemaphoreCount;        // Semaphore containers used to program Package semaphore.
 } PROGRAM_CPU_REGISTER_FLAGS;
 
+typedef union {
+  EFI_MP_SERVICES_PROTOCOL    *Protocol;
+  EFI_PEI_MP_SERVICES2_PPI    *Ppi;
+} MP_SERVICES;
+
 typedef struct {
-  UINTN                    FeaturesCount;
-  UINT32                   BitMaskSize;
-  LIST_ENTRY               FeatureList;
+  UINTN                         FeaturesCount;
+  UINT32                        BitMaskSize;
+  LIST_ENTRY                    FeatureList;
 
-  CPU_FEATURES_INIT_ORDER  *InitOrder;
-  UINT8                    *SupportPcd;
-  UINT8                    *CapabilityPcd;
-  UINT8                    *ConfigurationPcd;
-  UINT8                    *SettingPcd;
+  CPU_FEATURES_INIT_ORDER       *InitOrder;
+  UINT8                         *CapabilityPcd;
+  UINT8                         *SettingPcd;
 
-  UINT32                   NumberOfCpus;
-  ACPI_CPU_DATA            *AcpiCpuData;
+  UINT32                        NumberOfCpus;
+  ACPI_CPU_DATA                 *AcpiCpuData;
 
-  CPU_REGISTER_TABLE       *RegisterTable;
-  CPU_REGISTER_TABLE       *PreSmmRegisterTable;
-  UINTN                    BspNumber;
+  CPU_REGISTER_TABLE            *RegisterTable;
+  CPU_REGISTER_TABLE            *PreSmmRegisterTable;
+  UINTN                         BspNumber;
 
-  PROGRAM_CPU_REGISTER_FLAGS  CpuFlags;
+  PROGRAM_CPU_REGISTER_FLAGS    CpuFlags;
+
+  MP_SERVICES                   MpService;
 } CPU_FEATURES_DATA;
 
 #define CPU_FEATURE_ENTRY_FROM_LINK(a) \
@@ -108,11 +110,13 @@ GetCpuFeaturesData (
 /**
   Worker function to return processor index.
 
+  @param  CpuFeaturesData    Cpu Feature Data structure.
+
   @return  The processor index.
 **/
 UINTN
 GetProcessorIndex (
-  VOID
+  IN CPU_FEATURES_DATA  *CpuFeaturesData
   );
 
 /**
@@ -127,8 +131,8 @@ GetProcessorIndex (
 **/
 EFI_STATUS
 GetProcessorInformation (
-  IN  UINTN                            ProcessorNumber,
-  OUT EFI_PROCESSOR_INFORMATION        *ProcessorInfoBuffer
+  IN  UINTN                      ProcessorNumber,
+  OUT EFI_PROCESSOR_INFORMATION  *ProcessorInfoBuffer
   );
 
 /**
@@ -140,9 +144,9 @@ GetProcessorInformation (
                                       to check whether procedure has done.
 **/
 VOID
-StartupAPsWorker (
-  IN  EFI_AP_PROCEDURE                 Procedure,
-  IN  EFI_EVENT                        MpEvent
+StartupAllAPsWorker (
+  IN  EFI_AP_PROCEDURE  Procedure,
+  IN  EFI_EVENT         MpEvent
   );
 
 /**
@@ -157,8 +161,8 @@ StartupAPsWorker (
 **/
 VOID
 GetNumberOfProcessor (
-  OUT UINTN                            *NumberOfCpus,
-  OUT UINTN                            *NumberOfEnabledProcessors
+  OUT UINTN  *NumberOfCpus,
+  OUT UINTN  *NumberOfEnabledProcessors
   );
 
 /**
@@ -168,27 +172,33 @@ GetNumberOfProcessor (
 **/
 VOID
 SwitchNewBsp (
-  IN  UINTN                            ProcessorNumber
+  IN  UINTN  ProcessorNumber
   );
 
 /**
   Function that uses DEBUG() macros to display the contents of a a CPU feature bit mask.
 
   @param[in]  FeatureMask  A pointer to the CPU feature bit mask.
+  @param[in]  BitMaskSize  CPU feature bits mask buffer size.
+
 **/
 VOID
 DumpCpuFeatureMask (
-  IN UINT8               *FeatureMask
+  IN UINT8   *FeatureMask,
+  IN UINT32  BitMaskSize
   );
 
 /**
   Dump CPU feature name or CPU feature bit mask.
 
   @param[in]  CpuFeature   Pointer to CPU_FEATURES_ENTRY
+  @param[in]  BitMaskSize  CPU feature bits mask buffer size.
+
 **/
 VOID
 DumpCpuFeature (
-  IN CPU_FEATURES_ENTRY  *CpuFeature
+  IN CPU_FEATURES_ENTRY  *CpuFeature,
+  IN UINT32              BitMaskSize
   );
 
 /**
@@ -202,9 +212,9 @@ DumpCpuFeature (
 **/
 CPU_FEATURE_DEPENDENCE_TYPE
 DetectFeatureScope (
-  IN CPU_FEATURES_ENTRY         *CpuFeature,
-  IN BOOLEAN                    Before,
-  IN UINT8                      *NextCpuFeatureMask
+  IN CPU_FEATURES_ENTRY  *CpuFeature,
+  IN BOOLEAN             Before,
+  IN UINT8               *NextCpuFeatureMask
   );
 
 /**
@@ -218,9 +228,9 @@ DetectFeatureScope (
 **/
 CPU_FEATURE_DEPENDENCE_TYPE
 DetectNoneNeighborhoodFeatureScope (
-  IN CPU_FEATURES_ENTRY         *CpuFeature,
-  IN BOOLEAN                    Before,
-  IN LIST_ENTRY                 *FeatureList
+  IN CPU_FEATURES_ENTRY  *CpuFeature,
+  IN BOOLEAN             Before,
+  IN LIST_ENTRY          *FeatureList
   );
 
 /**
@@ -232,7 +242,7 @@ DetectNoneNeighborhoodFeatureScope (
 VOID
 EFIAPI
 SetProcessorRegister (
-  IN OUT VOID            *Buffer
+  IN OUT VOID  *Buffer
   );
 
 /**
@@ -245,4 +255,14 @@ GetAcpiCpuData (
   VOID
   );
 
+/**
+  Worker function to get MP service pointer.
+
+  @return MP_SERVICES variable.
+**/
+MP_SERVICES
+GetMpService (
+  VOID
+  );
+
 #endif
